/* =======================================================================
 *
 * Soldier aka "Guard" animations.
 *
 * =======================================================================
 */
 
#define FRAME_attak101        	0
#define FRAME_attak102        	1
#define FRAME_attak103        	2
#define FRAME_attak104        	3
#define FRAME_attak105        	4
#define FRAME_attak106        	5
#define FRAME_attak107        	6
#define FRAME_attak108        	7
#define FRAME_attak109        	8
#define FRAME_attak110        	9
#define FRAME_attak111        	10
#define FRAME_attak112        	11
#define FRAME_attak201        	12
#define FRAME_attak202        	13
#define FRAME_attak203        	14
#define FRAME_attak204        	15
#define FRAME_attak205        	16
#define FRAME_attak206        	17
#define FRAME_attak207        	18
#define FRAME_attak208        	19
#define FRAME_attak209        	20
#define FRAME_attak210        	21
#define FRAME_attak211        	22
#define FRAME_attak212        	23
#define FRAME_attak213        	24
#define FRAME_attak214        	25
#define FRAME_attak215        	26
#define FRAME_attak216        	27
#define FRAME_attak217        	28
#define FRAME_attak218        	29
#define FRAME_attak301        	30
#define FRAME_attak302        	31
#define FRAME_attak303        	32
#define FRAME_attak304        	33
#define FRAME_attak305        	34
#define FRAME_attak306        	35
#define FRAME_attak307        	36
#define FRAME_attak308        	37
#define FRAME_attak309        	38
#define FRAME_attak401        	39
#define FRAME_attak402        	40
#define FRAME_attak403        	41
#define FRAME_attak404        	42
#define FRAME_attak405        	43
#define FRAME_attak406        	44
#define FRAME_duck01          	45
#define FRAME_duck02          	46
#define FRAME_duck03          	47
#define FRAME_duck04          	48
#define FRAME_duck05          	49
#define FRAME_pain101         	50
#define FRAME_pain102         	51
#define FRAME_pain103         	52
#define FRAME_pain104         	53
#define FRAME_pain105         	54
#define FRAME_pain201         	55
#define FRAME_pain202         	56
#define FRAME_pain203         	57
#define FRAME_pain204         	58
#define FRAME_pain205         	59
#define FRAME_pain206         	60
#define FRAME_pain207         	61
#define FRAME_pain301         	62
#define FRAME_pain302         	63
#define FRAME_pain303         	64
#define FRAME_pain304         	65
#define FRAME_pain305         	66
#define FRAME_pain306         	67
#define FRAME_pain307         	68
#define FRAME_pain308         	69
#define FRAME_pain309         	70
#define FRAME_pain310         	71
#define FRAME_pain311         	72
#define FRAME_pain312         	73
#define FRAME_pain313         	74
#define FRAME_pain314         	75
#define FRAME_pain315         	76
#define FRAME_pain316         	77
#define FRAME_pain317         	78
#define FRAME_pain318         	79
#define FRAME_pain401         	80
#define FRAME_pain402         	81
#define FRAME_pain403         	82
#define FRAME_pain404         	83
#define FRAME_pain405         	84
#define FRAME_pain406         	85
#define FRAME_pain407         	86
#define FRAME_pain408         	87
#define FRAME_pain409         	88
#define FRAME_pain410         	89
#define FRAME_pain411         	90
#define FRAME_pain412         	91
#define FRAME_pain413         	92
#define FRAME_pain414         	93
#define FRAME_pain415         	94
#define FRAME_pain416         	95
#define FRAME_pain417         	96
#define FRAME_run01           	97
#define FRAME_run02           	98
#define FRAME_run03           	99
#define FRAME_run04           	100
#define FRAME_run05           	101
#define FRAME_run06           	102
#define FRAME_run07           	103
#define FRAME_run08           	104
#define FRAME_run09           	105
#define FRAME_run10           	106
#define FRAME_run11           	107
#define FRAME_run12           	108
#define FRAME_runs01          	109
#define FRAME_runs02          	110
#define FRAME_runs03          	111
#define FRAME_runs04          	112
#define FRAME_runs05          	113
#define FRAME_runs06          	114
#define FRAME_runs07          	115
#define FRAME_runs08          	116
#define FRAME_runs09          	117
#define FRAME_runs10          	118
#define FRAME_runs11          	119
#define FRAME_runs12          	120
#define FRAME_runs13          	121
#define FRAME_runs14          	122
#define FRAME_runs15          	123
#define FRAME_runs16          	124
#define FRAME_runs17          	125
#define FRAME_runs18          	126
#define FRAME_runt01          	127
#define FRAME_runt02          	128
#define FRAME_runt03          	129
#define FRAME_runt04          	130
#define FRAME_runt05          	131
#define FRAME_runt06          	132
#define FRAME_runt07          	133
#define FRAME_runt08          	134
#define FRAME_runt09          	135
#define FRAME_runt10          	136
#define FRAME_runt11          	137
#define FRAME_runt12          	138
#define FRAME_runt13          	139
#define FRAME_runt14          	140
#define FRAME_runt15          	141
#define FRAME_runt16          	142
#define FRAME_runt17          	143
#define FRAME_runt18          	144
#define FRAME_runt19          	145
#define FRAME_stand101        	146
#define FRAME_stand102        	147
#define FRAME_stand103        	148
#define FRAME_stand104        	149
#define FRAME_stand105        	150
#define FRAME_stand106        	151
#define FRAME_stand107        	152
#define FRAME_stand108        	153
#define FRAME_stand109        	154
#define FRAME_stand110        	155
#define FRAME_stand111        	156
#define FRAME_stand112        	157
#define FRAME_stand113        	158
#define FRAME_stand114        	159
#define FRAME_stand115        	160
#define FRAME_stand116        	161
#define FRAME_stand117        	162
#define FRAME_stand118        	163
#define FRAME_stand119        	164
#define FRAME_stand120        	165
#define FRAME_stand121        	166
#define FRAME_stand122        	167
#define FRAME_stand123        	168
#define FRAME_stand124        	169
#define FRAME_stand125        	170
#define FRAME_stand126        	171
#define FRAME_stand127        	172
#define FRAME_stand128        	173
#define FRAME_stand129        	174
#define FRAME_stand130        	175
#define FRAME_stand301        	176
#define FRAME_stand302        	177
#define FRAME_stand303        	178
#define FRAME_stand304        	179
#define FRAME_stand305        	180
#define FRAME_stand306        	181
#define FRAME_stand307        	182
#define FRAME_stand308        	183
#define FRAME_stand309        	184
#define FRAME_stand310        	185
#define FRAME_stand311        	186
#define FRAME_stand312        	187
#define FRAME_stand313        	188
#define FRAME_stand314        	189
#define FRAME_stand315        	190
#define FRAME_stand316        	191
#define FRAME_stand317        	192
#define FRAME_stand318        	193
#define FRAME_stand319        	194
#define FRAME_stand320        	195
#define FRAME_stand321        	196
#define FRAME_stand322        	197
#define FRAME_stand323        	198
#define FRAME_stand324        	199
#define FRAME_stand325        	200
#define FRAME_stand326        	201
#define FRAME_stand327        	202
#define FRAME_stand328        	203
#define FRAME_stand329        	204
#define FRAME_stand330        	205
#define FRAME_stand331        	206
#define FRAME_stand332        	207
#define FRAME_stand333        	208
#define FRAME_stand334        	209
#define FRAME_stand335        	210
#define FRAME_stand336        	211
#define FRAME_stand337        	212
#define FRAME_stand338        	213
#define FRAME_stand339        	214
#define FRAME_walk101         	215
#define FRAME_walk102         	216
#define FRAME_walk103         	217
#define FRAME_walk104         	218
#define FRAME_walk105         	219
#define FRAME_walk106         	220
#define FRAME_walk107         	221
#define FRAME_walk108         	222
#define FRAME_walk109         	223
#define FRAME_walk110         	224
#define FRAME_walk111         	225
#define FRAME_walk112         	226
#define FRAME_walk113         	227
#define FRAME_walk114         	228
#define FRAME_walk115         	229
#define FRAME_walk116         	230
#define FRAME_walk117         	231
#define FRAME_walk118         	232
#define FRAME_walk119         	233
#define FRAME_walk120         	234
#define FRAME_walk121         	235
#define FRAME_walk122         	236
#define FRAME_walk123         	237
#define FRAME_walk124         	238
#define FRAME_walk125         	239
#define FRAME_walk126         	240
#define FRAME_walk127         	241
#define FRAME_walk128         	242
#define FRAME_walk129         	243
#define FRAME_walk130         	244
#define FRAME_walk131         	245
#define FRAME_walk132         	246
#define FRAME_walk133         	247
#define FRAME_walk201         	248
#define FRAME_walk202         	249
#define FRAME_walk203         	250
#define FRAME_walk204         	251
#define FRAME_walk205         	252
#define FRAME_walk206         	253
#define FRAME_walk207         	254
#define FRAME_walk208         	255
#define FRAME_walk209         	256
#define FRAME_walk210         	257
#define FRAME_walk211         	258
#define FRAME_walk212         	259
#define FRAME_walk213         	260
#define FRAME_walk214         	261
#define FRAME_walk215         	262
#define FRAME_walk216         	263
#define FRAME_walk217         	264
#define FRAME_walk218         	265
#define FRAME_walk219         	266
#define FRAME_walk220         	267
#define FRAME_walk221         	268
#define FRAME_walk222         	269
#define FRAME_walk223         	270
#define FRAME_walk224         	271
#define FRAME_death101        	272
#define FRAME_death102        	273
#define FRAME_death103        	274
#define FRAME_death104        	275
#define FRAME_death105        	276
#define FRAME_death106        	277
#define FRAME_death107        	278
#define FRAME_death108        	279
#define FRAME_death109        	280
#define FRAME_death110        	281
#define FRAME_death111        	282
#define FRAME_death112        	283
#define FRAME_death113        	284
#define FRAME_death114        	285
#define FRAME_death115        	286
#define FRAME_death116        	287
#define FRAME_death117        	288
#define FRAME_death118        	289
#define FRAME_death119        	290
#define FRAME_death120        	291
#define FRAME_death121        	292
#define FRAME_death122        	293
#define FRAME_death123        	294
#define FRAME_death124        	295
#define FRAME_death125        	296
#define FRAME_death126        	297
#define FRAME_death127        	298
#define FRAME_death128        	299
#define FRAME_death129        	300
#define FRAME_death130        	301
#define FRAME_death131        	302
#define FRAME_death132        	303
#define FRAME_death133        	304
#define FRAME_death134        	305
#define FRAME_death135        	306
#define FRAME_death136        	307
#define FRAME_death201        	308
#define FRAME_death202        	309
#define FRAME_death203        	310
#define FRAME_death204        	311
#define FRAME_death205        	312
#define FRAME_death206        	313
#define FRAME_death207        	314
#define FRAME_death208        	315
#define FRAME_death209        	316
#define FRAME_death210        	317
#define FRAME_death211        	318
#define FRAME_death212        	319
#define FRAME_death213        	320
#define FRAME_death214        	321
#define FRAME_death215        	322
#define FRAME_death216        	323
#define FRAME_death217        	324
#define FRAME_death218        	325
#define FRAME_death219        	326
#define FRAME_death220        	327
#define FRAME_death221        	328
#define FRAME_death222        	329
#define FRAME_death223        	330
#define FRAME_death224        	331
#define FRAME_death225        	332
#define FRAME_death226        	333
#define FRAME_death227        	334
#define FRAME_death228        	335
#define FRAME_death229        	336
#define FRAME_death230        	337
#define FRAME_death231        	338
#define FRAME_death232        	339
#define FRAME_death233        	340
#define FRAME_death234        	341
#define FRAME_death235        	342
#define FRAME_death301        	343
#define FRAME_death302        	344
#define FRAME_death303        	345
#define FRAME_death304        	346
#define FRAME_death305        	347
#define FRAME_death306        	348
#define FRAME_death307        	349
#define FRAME_death308        	350
#define FRAME_death309        	351
#define FRAME_death310        	352
#define FRAME_death311        	353
#define FRAME_death312        	354
#define FRAME_death313        	355
#define FRAME_death314        	356
#define FRAME_death315        	357
#define FRAME_death316        	358
#define FRAME_death317        	359
#define FRAME_death318        	360
#define FRAME_death319        	361
#define FRAME_death320        	362
#define FRAME_death321        	363
#define FRAME_death322        	364
#define FRAME_death323        	365
#define FRAME_death324        	366
#define FRAME_death325        	367
#define FRAME_death326        	368
#define FRAME_death327        	369
#define FRAME_death328        	370
#define FRAME_death329        	371
#define FRAME_death330        	372
#define FRAME_death331        	373
#define FRAME_death332        	374
#define FRAME_death333        	375
#define FRAME_death334        	376
#define FRAME_death335        	377
#define FRAME_death336        	378
#define FRAME_death337        	379
#define FRAME_death338        	380
#define FRAME_death339        	381
#define FRAME_death340        	382
#define FRAME_death341        	383
#define FRAME_death342        	384
#define FRAME_death343        	385
#define FRAME_death344        	386
#define FRAME_death345        	387
#define FRAME_death401        	388
#define FRAME_death402        	389
#define FRAME_death403        	390
#define FRAME_death404        	391
#define FRAME_death405        	392
#define FRAME_death406        	393
#define FRAME_death407        	394
#define FRAME_death408        	395
#define FRAME_death409        	396
#define FRAME_death410        	397
#define FRAME_death411        	398
#define FRAME_death412        	399
#define FRAME_death413        	400
#define FRAME_death414        	401
#define FRAME_death415        	402
#define FRAME_death416        	403
#define FRAME_death417        	404
#define FRAME_death418        	405
#define FRAME_death419        	406
#define FRAME_death420        	407
#define FRAME_death421        	408
#define FRAME_death422        	409
#define FRAME_death423        	410
#define FRAME_death424        	411
#define FRAME_death425        	412
#define FRAME_death426        	413
#define FRAME_death427        	414
#define FRAME_death428        	415
#define FRAME_death429        	416
#define FRAME_death430        	417
#define FRAME_death431        	418
#define FRAME_death432        	419
#define FRAME_death433        	420
#define FRAME_death434        	421
#define FRAME_death435        	422
#define FRAME_death436        	423
#define FRAME_death437        	424
#define FRAME_death438        	425
#define FRAME_death439        	426
#define FRAME_death440        	427
#define FRAME_death441        	428
#define FRAME_death442        	429
#define FRAME_death443        	430
#define FRAME_death444        	431
#define FRAME_death445        	432
#define FRAME_death446        	433
#define FRAME_death447        	434
#define FRAME_death448        	435
#define FRAME_death449        	436
#define FRAME_death450        	437
#define FRAME_death451        	438
#define FRAME_death452        	439
#define FRAME_death453        	440
#define FRAME_death501        	441
#define FRAME_death502        	442
#define FRAME_death503        	443
#define FRAME_death504        	444
#define FRAME_death505        	445
#define FRAME_death506        	446
#define FRAME_death507        	447
#define FRAME_death508        	448
#define FRAME_death509        	449
#define FRAME_death510        	450
#define FRAME_death511        	451
#define FRAME_death512        	452
#define FRAME_death513        	453
#define FRAME_death514        	454
#define FRAME_death515        	455
#define FRAME_death516        	456
#define FRAME_death517        	457
#define FRAME_death518        	458
#define FRAME_death519        	459
#define FRAME_death520        	460
#define FRAME_death521        	461
#define FRAME_death522        	462
#define FRAME_death523        	463
#define FRAME_death524        	464
#define FRAME_death601        	465
#define FRAME_death602        	466
#define FRAME_death603        	467
#define FRAME_death604        	468
#define FRAME_death605        	469
#define FRAME_death606        	470
#define FRAME_death607        	471
#define FRAME_death608        	472
#define FRAME_death609        	473
#define FRAME_death610        	474
#define MODEL_SCALE				1.200000
