/*
 * =======================================================================
 *
 * Pointers to every mmove_t in the game.so.
 *
 * =======================================================================
 */

{"widow2_move_really_dead", &widow2_move_really_dead},
{"widow2_move_dead", &widow2_move_dead},
{"widow2_move_death", &widow2_move_death},
{"widow2_move_pain", &widow2_move_pain},
{"widow2_move_tongs", &widow2_move_tongs},
{"widow2_move_spawn", &widow2_move_spawn},
{"widow2_move_attack_disrupt", &widow2_move_attack_disrupt},
{"widow2_move_attack_post_beam", &widow2_move_attack_post_beam},
{"widow2_move_attack_beam", &widow2_move_attack_beam},
{"widow2_move_attack_pre_beam", &widow2_move_attack_pre_beam},
{"widow2_move_run", &widow2_move_run},
{"widow2_move_walk", &widow2_move_walk},
{"widow2_move_stand", &widow2_move_stand},
{"widow_move_attack_kick", &widow_move_attack_kick},
{"widow_move_death", &widow_move_death},
{"widow_move_pain_light", &widow_move_pain_light},
{"widow_move_pain_heavy", &widow_move_pain_heavy},
{"widow_move_spawn", &widow_move_spawn},
{"widow_move_attack_pre_rail", &widow_move_attack_pre_rail},
{"widow_move_attack_rail_r", &widow_move_attack_rail_r},
{"widow_move_attack_rail_l", &widow_move_attack_rail_l},
{"widow_move_attack_rail", &widow_move_attack_rail},
{"widow_move_attack_pre_blaster", &widow_move_attack_pre_blaster},
{"widow_move_run_attack", &widow_move_run_attack},
{"widow_move_run", &widow_move_run},
{"widow_move_walk", &widow_move_walk},
{"widow_move_stand", &widow_move_stand},
{"widow_move_attack_blaster", &widow_move_attack_blaster},
{"widow_move_attack_post_blaster_l", &widow_move_attack_post_blaster_l},
{"widow_move_attack_post_blaster_r", &widow_move_attack_post_blaster_r},
{"widow_move_attack_post_blaster", &widow_move_attack_post_blaster},
{"turret_move_run", &turret_move_run},
{"turret_move_seek", &turret_move_seek},
{"turret_move_ready_gun", &turret_move_ready_gun},
{"turret_move_stand", &turret_move_stand},
{"turret_move_fire_blind", &turret_move_fire_blind},
{"turret_move_fire", &turret_move_fire},
{"tank_move_death", &tank_move_death},
{"tank_move_attack_chain", &tank_move_attack_chain},
{"tank_move_attack_post_rocket", &tank_move_attack_post_rocket},
{"tank_move_attack_fire_rocket", &tank_move_attack_fire_rocket},
{"tank_move_attack_pre_rocket", &tank_move_attack_pre_rocket},
{"tank_move_attack_strike", &tank_move_attack_strike},
{"tank_move_attack_post_blast", &tank_move_attack_post_blast},
{"tank_move_reattack_blast", &tank_move_reattack_blast},
{"tank_move_attack_blast", &tank_move_attack_blast},
{"tank_move_pain3", &tank_move_pain3},
{"tank_move_pain2", &tank_move_pain2},
{"tank_move_pain1", &tank_move_pain1},
{"tank_move_stop_run", &tank_move_stop_run},
{"tank_move_run", &tank_move_run},
{"tank_move_start_run", &tank_move_start_run},
{"tank_move_stop_walk", &tank_move_stop_walk},
{"tank_move_walk", &tank_move_walk},
{"tank_move_start_walk", &tank_move_start_walk},
{"tank_move_stand", &tank_move_stand},
{"supertank_move_end_attack1", &supertank_move_end_attack1},
{"supertank_move_attack1", &supertank_move_attack1},
{"supertank_move_attack2", &supertank_move_attack2},
{"supertank_move_attack3", &supertank_move_attack3},
{"supertank_move_attack4", &supertank_move_attack4},
{"supertank_move_backward", &supertank_move_backward},
{"supertank_move_death", &supertank_move_death},
{"supertank_move_pain1", &supertank_move_pain1},
{"supertank_move_pain2", &supertank_move_pain2},
{"supertank_move_pain3", &supertank_move_pain3},
{"supertank_move_turn_left", &supertank_move_turn_left},
{"supertank_move_turn_right", &supertank_move_turn_right},
{"supertank_move_forward", &supertank_move_forward},
{"supertank_move_run", &supertank_move_run},
{"supertank_move_stand", &supertank_move_stand},
{"stalker_move_death", &stalker_move_death},
{"stalker_move_jump_down", &stalker_move_jump_down},
{"stalker_move_jump_up", &stalker_move_jump_up},
{"stalker_move_dodge_run", &stalker_move_dodge_run},
{"stalker_move_jump_straightup", &stalker_move_jump_straightup},
{"stalker_move_swing_r", &stalker_move_swing_r},
{"stalker_move_swing_l", &stalker_move_swing_l},
{"stalker_move_shoot", &stalker_move_shoot},
{"stalker_move_pain", &stalker_move_pain},
{"stalker_move_false_death_start", &stalker_move_false_death_start},
{"stalker_move_false_death", &stalker_move_false_death},
{"stalker_move_false_death_end", &stalker_move_false_death_end},
{"stalker_move_walk", &stalker_move_walk},
{"stalker_move_run", &stalker_move_run},
{"stalker_move_stand", &stalker_move_stand},
{"stalker_move_idle2", &stalker_move_idle2},
{"stalker_move_idle", &stalker_move_idle},
{"soldier_move_blind", &soldier_move_blind},
{"soldier_move_death6", &soldier_move_death6},
{"soldier_move_death5", &soldier_move_death5},
{"soldier_move_death4", &soldier_move_death4},
{"soldier_move_death3", &soldier_move_death3},
{"soldier_move_death2", &soldier_move_death2},
{"soldier_move_death1", &soldier_move_death1},
{"soldier_move_duck", &soldier_move_duck},
{"soldier_move_attack6", &soldier_move_attack6},
{"soldier_move_attack4", &soldier_move_attack4},
{"soldier_move_attack3", &soldier_move_attack3},
{"soldier_move_attack2", &soldier_move_attack2},
{"soldier_move_attack1", &soldier_move_attack1},
{"soldier_move_pain4", &soldier_move_pain4},
{"soldier_move_pain3", &soldier_move_pain3},
{"soldier_move_pain2", &soldier_move_pain2},
{"soldier_move_pain1", &soldier_move_pain1},
{"soldier_move_run", &soldier_move_run},
{"soldier_move_start_run", &soldier_move_start_run},
{"soldier_move_walk2", &soldier_move_walk2},
{"soldier_move_walk1", &soldier_move_walk1},
{"soldier_move_stand3", &soldier_move_stand3},
{"soldier_move_stand1", &soldier_move_stand1},
{"parasite_move_death", &parasite_move_death},
{"parasite_move_jump_down", &parasite_move_jump_down},
{"parasite_move_jump_up", &parasite_move_jump_up},
{"parasite_move_break", &parasite_move_break},
{"parasite_move_drain", &parasite_move_drain},
{"parasite_move_pain1", &parasite_move_pain1},
{"parasite_move_stop_walk", &parasite_move_stop_walk},
{"parasite_move_start_walk", &parasite_move_start_walk},
{"parasite_move_walk", &parasite_move_walk},
{"parasite_move_stop_run", &parasite_move_stop_run},
{"parasite_move_start_run", &parasite_move_start_run},
{"parasite_move_run", &parasite_move_run},
{"parasite_move_stand", &parasite_move_stand},
{"parasite_move_end_fidget", &parasite_move_end_fidget},
{"parasite_move_fidget", &parasite_move_fidget},
{"parasite_move_start_fidget", &parasite_move_start_fidget},
{"mutant_move_jump_down", &mutant_move_jump_down},
{"mutant_move_jump_up", &mutant_move_jump_up},
{"mutant_move_death2", &mutant_move_death2},
{"mutant_move_death1", &mutant_move_death1},
{"mutant_move_pain3", &mutant_move_pain3},
{"mutant_move_pain2", &mutant_move_pain2},
{"mutant_move_pain1", &mutant_move_pain1},
{"mutant_move_jump", &mutant_move_jump},
{"mutant_move_attack", &mutant_move_attack},
{"mutant_move_run", &mutant_move_run},
{"mutant_move_start_walk", &mutant_move_start_walk},
{"mutant_move_walk", &mutant_move_walk},
{"mutant_move_idle", &mutant_move_idle},
{"mutant_move_stand", &mutant_move_stand},
{"medic_move_callReinforcements", &medic_move_callReinforcements},
{"medic_move_attackCable", &medic_move_attackCable},
{"medic_move_attackBlaster", &medic_move_attackBlaster},
{"medic_move_attackHyperBlaster", &medic_move_attackHyperBlaster},
{"medic_move_duck", &medic_move_duck},
{"medic_move_death", &medic_move_death},
{"medic_move_pain2", &medic_move_pain2},
{"medic_move_pain1", &medic_move_pain1},
{"medic_move_run", &medic_move_run},
{"medic_move_walk", &medic_move_walk},
{"medic_move_stand", &medic_move_stand},
{"insane_move_struggle_cross", &insane_move_struggle_cross},
{"insane_move_cross", &insane_move_cross},
{"insane_move_crawl_death", &insane_move_crawl_death},
{"insane_move_crawl_pain", &insane_move_crawl_pain},
{"insane_move_runcrawl", &insane_move_runcrawl},
{"insane_move_crawl", &insane_move_crawl},
{"insane_move_stand_death", &insane_move_stand_death},
{"insane_move_stand_pain", &insane_move_stand_pain},
{"insane_move_run_insane", &insane_move_run_insane},
{"insane_move_walk_insane", &insane_move_walk_insane},
{"insane_move_run_normal", &insane_move_run_normal},
{"insane_move_walk_normal", &insane_move_walk_normal},
{"insane_move_down", &insane_move_down},
{"insane_move_jumpdown", &insane_move_jumpdown},
{"insane_move_downtoup", &insane_move_downtoup},
{"insane_move_uptodown", &insane_move_uptodown},
{"insane_move_stand_insane", &insane_move_stand_insane},
{"insane_move_stand_normal", &insane_move_stand_normal},
{"infantry_move_jump2", &infantry_move_jump2},
{"infantry_move_jump", &infantry_move_jump},
{"infantry_move_attack2", &infantry_move_attack2},
{"infantry_move_attack1", &infantry_move_attack1},
{"infantry_move_duck", &infantry_move_duck},
{"infantry_move_death3", &infantry_move_death3},
{"infantry_move_death2", &infantry_move_death2},
{"infantry_move_death1", &infantry_move_death1},
{"infantry_move_pain2", &infantry_move_pain2},
{"infantry_move_pain1", &infantry_move_pain1},
{"infantry_move_run", &infantry_move_run},
{"infantry_move_walk", &infantry_move_walk},
{"infantry_move_fidget", &infantry_move_fidget},
{"infantry_move_stand", &infantry_move_stand},
{"hover_move_start_attack2", &hover_move_start_attack2},
{"hover_move_attack2", &hover_move_attack2},
{"hover_move_end_attack2", &hover_move_end_attack2},
{"hover_move_end_attack", &hover_move_end_attack},
{"hover_move_attack1", &hover_move_attack1},
{"hover_move_start_attack", &hover_move_start_attack},
{"hover_move_death1", &hover_move_death1},
{"hover_move_run", &hover_move_run},
{"hover_move_walk", &hover_move_walk},
{"hover_move_pain1", &hover_move_pain1},
{"hover_move_pain2", &hover_move_pain2},
{"hover_move_pain3", &hover_move_pain3},
{"hover_move_stand", &hover_move_stand},
{"gunner_move_jump2", &gunner_move_jump2},
{"gunner_move_jump", &gunner_move_jump},
{"gunner_move_attack_grenade", &gunner_move_attack_grenade},
{"gunner_move_endfire_chain", &gunner_move_endfire_chain},
{"gunner_move_fire_chain", &gunner_move_fire_chain},
{"gunner_move_attack_chain", &gunner_move_attack_chain},
{"gunner_move_duck", &gunner_move_duck},
{"gunner_move_death", &gunner_move_death},
{"gunner_move_pain1", &gunner_move_pain1},
{"gunner_move_pain2", &gunner_move_pain2},
{"gunner_move_pain3", &gunner_move_pain3},
{"gunner_move_runandshoot", &gunner_move_runandshoot},
{"gunner_move_run", &gunner_move_run},
{"gunner_move_walk", &gunner_move_walk},
{"gunner_move_stand", &gunner_move_stand},
{"gunner_move_fidget", &gunner_move_fidget},
{"gladiator_move_death", &gladiator_move_death},
{"gladiator_move_pain_air", &gladiator_move_pain_air},
{"gladiator_move_pain", &gladiator_move_pain},
{"gladiator_move_attack_gun", &gladiator_move_attack_gun},
{"gladiator_move_attack_melee", &gladiator_move_attack_melee},
{"gladiator_move_run", &gladiator_move_run},
{"gladiator_move_walk", &gladiator_move_walk},
{"gladiator_move_stand", &gladiator_move_stand},
{"flyer_move_loop_melee", &flyer_move_loop_melee},
{"flyer_move_end_melee", &flyer_move_end_melee},
{"flyer_move_start_melee", &flyer_move_start_melee},
{"flyer_move_attack3", &flyer_move_attack3},
{"flyer_move_attack2", &flyer_move_attack2},
{"flyer_move_bankleft", &flyer_move_bankleft},
{"flyer_move_bankright", &flyer_move_bankright},
{"flyer_move_defense", &flyer_move_defense},
{"flyer_move_pain1", &flyer_move_pain1},
{"flyer_move_pain2", &flyer_move_pain2},
{"flyer_move_pain3", &flyer_move_pain3},
{"flyer_move_rollleft", &flyer_move_rollleft},
{"flyer_move_rollright", &flyer_move_rollright},
{"flyer_move_stop", &flyer_move_stop},
{"flyer_move_start", &flyer_move_start},
{"flyer_move_kamikaze", &flyer_move_kamikaze},
{"flyer_move_run", &flyer_move_run},
{"flyer_move_walk", &flyer_move_walk},
{"flyer_move_stand", &flyer_move_stand},
{"floater_move_run", &floater_move_run},
{"floater_move_walk", &floater_move_walk},
{"floater_move_pain3", &floater_move_pain3},
{"floater_move_pain2", &floater_move_pain2},
{"floater_move_pain1", &floater_move_pain1},
{"floater_move_death", &floater_move_death},
{"floater_move_attack3", &floater_move_attack3},
{"floater_move_attack2", &floater_move_attack2},
{"floater_move_attack1a", &floater_move_attack1a},
{"floater_move_attack1", &floater_move_attack1},
{"floater_move_activate", &floater_move_activate},
{"floater_move_stand2", &floater_move_stand2},
{"floater_move_stand1", &floater_move_stand1},
{"flipper_move_death", &flipper_move_death},
{"flipper_move_attack", &flipper_move_attack},
{"flipper_move_pain1", &flipper_move_pain1},
{"flipper_move_pain2", &flipper_move_pain2},
{"flipper_move_start_run", &flipper_move_start_run},
{"flipper_move_walk", &flipper_move_walk},
{"flipper_move_run_start", &flipper_move_run_start},
{"flipper_move_run_loop", &flipper_move_run_loop},
{"flipper_move_stand", &flipper_move_stand},
{"chick_move_start_slash", &chick_move_start_slash},
{"chick_move_end_slash", &chick_move_end_slash},
{"chick_move_slash", &chick_move_slash},
{"chick_move_end_attack1", &chick_move_end_attack1},
{"chick_move_attack1", &chick_move_attack1},
{"chick_move_start_attack1", &chick_move_start_attack1},
{"chick_move_duck", &chick_move_duck},
{"chick_move_death1", &chick_move_death1},
{"chick_move_death2", &chick_move_death2},
{"chick_move_pain3", &chick_move_pain3},
{"chick_move_pain2", &chick_move_pain2},
{"chick_move_pain1", &chick_move_pain1},
{"chick_move_walk", &chick_move_walk},
{"chick_move_run", &chick_move_run},
{"chick_move_start_run", &chick_move_start_run},
{"chick_move_stand", &chick_move_stand},
{"chick_move_fidget", &chick_move_fidget},
{"carrier_move_death", &carrier_move_death},
{"carrier_move_pain_light", &carrier_move_pain_light},
{"carrier_move_pain_heavy", &carrier_move_pain_heavy},
{"carrier_move_spawn", &carrier_move_spawn},
{"carrier_move_attack_rail", &carrier_move_attack_rail},
{"carrier_move_attack_rocket", &carrier_move_attack_rocket},
{"carrier_move_attack_post_gren", &carrier_move_attack_post_gren},
{"carrier_move_attack_gren", &carrier_move_attack_gren},
{"carrier_move_attack_pre_gren", &carrier_move_attack_pre_gren},
{"carrier_move_attack_post_mg", &carrier_move_attack_post_mg},
{"carrier_move_attack_mg", &carrier_move_attack_mg},
{"carrier_move_attack_pre_mg", &carrier_move_attack_pre_mg},
{"carrier_move_run", &carrier_move_run},
{"carrier_move_walk", &carrier_move_walk},
{"carrier_move_stand", &carrier_move_stand},
{"brain_move_run", &brain_move_run},
{"brain_move_attack2", &brain_move_attack2},
{"brain_move_attack1", &brain_move_attack1},
{"brain_move_death1", &brain_move_death1},
{"brain_move_death2", &brain_move_death2},
{"brain_move_duck", &brain_move_duck},
{"brain_move_pain1", &brain_move_pain1},
{"brain_move_pain2", &brain_move_pain2},
{"brain_move_pain3", &brain_move_pain3},
{"brain_move_defense", &brain_move_defense},
{"brain_move_walk1", &brain_move_walk1},
{"brain_move_idle", &brain_move_idle},
{"brain_move_stand", &brain_move_stand},
{"makron_move_attack5", &makron_move_attack5},
{"makron_move_attack4", &makron_move_attack4},
{"makron_move_attack3", &makron_move_attack3},
{"makron_move_sight", &makron_move_sight},
{"makron_move_death3", &makron_move_death3},
{"makron_move_death2", &makron_move_death2},
{"makron_move_pain4", &makron_move_pain4},
{"makron_move_pain5", &makron_move_pain5},
{"makron_move_pain6", &makron_move_pain6},
{"makron_move_walk", &makron_move_walk},
{"makron_move_run", &makron_move_run},
{"makron_move_stand", &makron_move_stand},
{"jorg_move_end_attack1", &jorg_move_end_attack1},
{"jorg_move_attack1", &jorg_move_attack1},
{"jorg_move_start_attack1", &jorg_move_start_attack1},
{"jorg_move_attack2", &jorg_move_attack2},
{"jorg_move_death", &jorg_move_death},
{"jorg_move_pain1", &jorg_move_pain1},
{"jorg_move_pain2", &jorg_move_pain2},
{"jorg_move_pain3", &jorg_move_pain3},
{"jorg_move_end_walk", &jorg_move_end_walk},
{"jorg_move_walk", &jorg_move_walk},
{"jorg_move_start_walk", &jorg_move_start_walk},
{"jorg_move_run", &jorg_move_run},
{"jorg_move_stand", &jorg_move_stand},
{"boss2_move_death", &boss2_move_death},
{"boss2_move_pain_light", &boss2_move_pain_light},
{"boss2_move_pain_heavy", &boss2_move_pain_heavy},
{"boss2_move_attack_rocket", &boss2_move_attack_rocket},
{"boss2_move_attack_post_mg", &boss2_move_attack_post_mg},
{"boss2_move_attack_mg", &boss2_move_attack_mg},
{"boss2_move_attack_pre_mg", &boss2_move_attack_pre_mg},
{"boss2_move_run", &boss2_move_run},
{"boss2_move_walk", &boss2_move_walk},
{"boss2_move_fidget", &boss2_move_fidget},
{"boss2_move_stand", &boss2_move_stand},
{"berserk_move_jump2", &berserk_move_jump2},
{"berserk_move_jump", &berserk_move_jump},
{"berserk_move_death2", &berserk_move_death2},
{"berserk_move_death1", &berserk_move_death1},
{"berserk_move_pain2", &berserk_move_pain2},
{"berserk_move_pain1", &berserk_move_pain1},
{"berserk_move_attack_strike", &berserk_move_attack_strike},
{"berserk_move_attack_club", &berserk_move_attack_club},
{"berserk_move_attack_spike", &berserk_move_attack_spike},
{"berserk_move_run1", &berserk_move_run1},
{"berserk_move_walk", &berserk_move_walk},
{"berserk_move_stand_fidget", &berserk_move_stand_fidget},
{"berserk_move_stand", &berserk_move_stand},
