/* =======================================================================
 *
 * Final boss, stage 2 (makron).
 *
 * =======================================================================
 */
 
#define FRAME_attak101        	0
#define FRAME_attak102        	1
#define FRAME_attak103        	2
#define FRAME_attak104        	3
#define FRAME_attak105        	4
#define FRAME_attak106        	5
#define FRAME_attak107        	6
#define FRAME_attak108        	7
#define FRAME_attak109        	8
#define FRAME_attak110        	9
#define FRAME_attak111        	10
#define FRAME_attak112        	11
#define FRAME_attak113        	12
#define FRAME_attak114        	13
#define FRAME_attak115        	14
#define FRAME_attak116        	15
#define FRAME_attak117        	16
#define FRAME_attak118        	17
#define FRAME_attak201        	18
#define FRAME_attak202        	19
#define FRAME_attak203        	20
#define FRAME_attak204        	21
#define FRAME_attak205        	22
#define FRAME_attak206        	23
#define FRAME_attak207        	24
#define FRAME_attak208        	25
#define FRAME_attak209        	26
#define FRAME_attak210        	27
#define FRAME_attak211        	28
#define FRAME_attak212        	29
#define FRAME_attak213        	30
#define FRAME_death01         	31
#define FRAME_death02         	32
#define FRAME_death03         	33
#define FRAME_death04         	34
#define FRAME_death05         	35
#define FRAME_death06         	36
#define FRAME_death07         	37
#define FRAME_death08         	38
#define FRAME_death09         	39
#define FRAME_death10         	40
#define FRAME_death11         	41
#define FRAME_death12         	42
#define FRAME_death13         	43
#define FRAME_death14         	44
#define FRAME_death15         	45
#define FRAME_death16         	46
#define FRAME_death17         	47
#define FRAME_death18         	48
#define FRAME_death19         	49
#define FRAME_death20         	50
#define FRAME_death21         	51
#define FRAME_death22         	52
#define FRAME_death23         	53
#define FRAME_death24         	54
#define FRAME_death25         	55
#define FRAME_death26         	56
#define FRAME_death27         	57
#define FRAME_death28         	58
#define FRAME_death29         	59
#define FRAME_death30         	60
#define FRAME_death31         	61
#define FRAME_death32         	62
#define FRAME_death33         	63
#define FRAME_death34         	64
#define FRAME_death35         	65
#define FRAME_death36         	66
#define FRAME_death37         	67
#define FRAME_death38         	68
#define FRAME_death39         	69
#define FRAME_death40         	70
#define FRAME_death41         	71
#define FRAME_death42         	72
#define FRAME_death43         	73
#define FRAME_death44         	74
#define FRAME_death45         	75
#define FRAME_death46         	76
#define FRAME_death47         	77
#define FRAME_death48         	78
#define FRAME_death49         	79
#define FRAME_death50         	80
#define FRAME_pain101         	81
#define FRAME_pain102         	82
#define FRAME_pain103         	83
#define FRAME_pain201         	84
#define FRAME_pain202         	85
#define FRAME_pain203         	86
#define FRAME_pain301         	87
#define FRAME_pain302         	88
#define FRAME_pain303         	89
#define FRAME_pain304         	90
#define FRAME_pain305         	91
#define FRAME_pain306         	92
#define FRAME_pain307         	93
#define FRAME_pain308         	94
#define FRAME_pain309         	95
#define FRAME_pain310         	96
#define FRAME_pain311         	97
#define FRAME_pain312         	98
#define FRAME_pain313         	99
#define FRAME_pain314         	100
#define FRAME_pain315         	101
#define FRAME_pain316         	102
#define FRAME_pain317         	103
#define FRAME_pain318         	104
#define FRAME_pain319         	105
#define FRAME_pain320         	106
#define FRAME_pain321         	107
#define FRAME_pain322         	108
#define FRAME_pain323         	109
#define FRAME_pain324         	110
#define FRAME_pain325         	111
#define FRAME_stand01         	112
#define FRAME_stand02         	113
#define FRAME_stand03         	114
#define FRAME_stand04         	115
#define FRAME_stand05         	116
#define FRAME_stand06         	117
#define FRAME_stand07         	118
#define FRAME_stand08         	119
#define FRAME_stand09         	120
#define FRAME_stand10         	121
#define FRAME_stand11         	122
#define FRAME_stand12         	123
#define FRAME_stand13         	124
#define FRAME_stand14         	125
#define FRAME_stand15         	126
#define FRAME_stand16         	127
#define FRAME_stand17         	128
#define FRAME_stand18         	129
#define FRAME_stand19         	130
#define FRAME_stand20         	131
#define FRAME_stand21         	132
#define FRAME_stand22         	133
#define FRAME_stand23         	134
#define FRAME_stand24         	135
#define FRAME_stand25         	136
#define FRAME_stand26         	137
#define FRAME_stand27         	138
#define FRAME_stand28         	139
#define FRAME_stand29         	140
#define FRAME_stand30         	141
#define FRAME_stand31         	142
#define FRAME_stand32         	143
#define FRAME_stand33         	144
#define FRAME_stand34         	145
#define FRAME_stand35         	146
#define FRAME_stand36         	147
#define FRAME_stand37         	148
#define FRAME_stand38         	149
#define FRAME_stand39         	150
#define FRAME_stand40         	151
#define FRAME_stand41         	152
#define FRAME_stand42         	153
#define FRAME_stand43         	154
#define FRAME_stand44         	155
#define FRAME_stand45         	156
#define FRAME_stand46         	157
#define FRAME_stand47         	158
#define FRAME_stand48         	159
#define FRAME_stand49         	160
#define FRAME_stand50         	161
#define FRAME_stand51         	162
#define FRAME_walk01          	163
#define FRAME_walk02          	164
#define FRAME_walk03          	165
#define FRAME_walk04          	166
#define FRAME_walk05          	167
#define FRAME_walk06          	168
#define FRAME_walk07          	169
#define FRAME_walk08          	170
#define FRAME_walk09          	171
#define FRAME_walk10          	172
#define FRAME_walk11          	173
#define FRAME_walk12          	174
#define FRAME_walk13          	175
#define FRAME_walk14          	176
#define FRAME_walk15          	177
#define FRAME_walk16          	178
#define FRAME_walk17          	179
#define FRAME_walk18          	180
#define FRAME_walk19          	181
#define FRAME_walk20          	182
#define FRAME_walk21          	183
#define FRAME_walk22          	184
#define FRAME_walk23          	185
#define FRAME_walk24          	186
#define FRAME_walk25          	187
#define FRAME_active01        	188
#define FRAME_active02        	189
#define FRAME_active03        	190
#define FRAME_active04        	191
#define FRAME_active05        	192
#define FRAME_active06        	193
#define FRAME_active07        	194
#define FRAME_active08        	195
#define FRAME_active09        	196
#define FRAME_active10        	197
#define FRAME_active11        	198
#define FRAME_active12        	199
#define FRAME_active13        	200
#define FRAME_attak301        	201
#define FRAME_attak302        	202
#define FRAME_attak303        	203
#define FRAME_attak304        	204
#define FRAME_attak305        	205
#define FRAME_attak306        	206
#define FRAME_attak307        	207
#define FRAME_attak308        	208
#define FRAME_attak401        	209
#define FRAME_attak402        	210
#define FRAME_attak403        	211
#define FRAME_attak404        	212
#define FRAME_attak405        	213
#define FRAME_attak406        	214
#define FRAME_attak407        	215
#define FRAME_attak408        	216
#define FRAME_attak409        	217
#define FRAME_attak410        	218
#define FRAME_attak411        	219
#define FRAME_attak412        	220
#define FRAME_attak413        	221
#define FRAME_attak414        	222
#define FRAME_attak415        	223
#define FRAME_attak416        	224
#define FRAME_attak417        	225
#define FRAME_attak418        	226
#define FRAME_attak419        	227
#define FRAME_attak420        	228
#define FRAME_attak421        	229
#define FRAME_attak422        	230
#define FRAME_attak423        	231
#define FRAME_attak424        	232
#define FRAME_attak425        	233
#define FRAME_attak426        	234
#define FRAME_attak501        	235
#define FRAME_attak502        	236
#define FRAME_attak503        	237
#define FRAME_attak504        	238
#define FRAME_attak505        	239
#define FRAME_attak506        	240
#define FRAME_attak507        	241
#define FRAME_attak508        	242
#define FRAME_attak509        	243
#define FRAME_attak510        	244
#define FRAME_attak511        	245
#define FRAME_attak512        	246
#define FRAME_attak513        	247
#define FRAME_attak514        	248
#define FRAME_attak515        	249
#define FRAME_attak516        	250
#define FRAME_death201        	251
#define FRAME_death202        	252
#define FRAME_death203        	253
#define FRAME_death204        	254
#define FRAME_death205        	255
#define FRAME_death206        	256
#define FRAME_death207        	257
#define FRAME_death208        	258
#define FRAME_death209        	259
#define FRAME_death210        	260
#define FRAME_death211        	261
#define FRAME_death212        	262
#define FRAME_death213        	263
#define FRAME_death214        	264
#define FRAME_death215        	265
#define FRAME_death216        	266
#define FRAME_death217        	267
#define FRAME_death218        	268
#define FRAME_death219        	269
#define FRAME_death220        	270
#define FRAME_death221        	271
#define FRAME_death222        	272
#define FRAME_death223        	273
#define FRAME_death224        	274
#define FRAME_death225        	275
#define FRAME_death226        	276
#define FRAME_death227        	277
#define FRAME_death228        	278
#define FRAME_death229        	279
#define FRAME_death230        	280
#define FRAME_death231        	281
#define FRAME_death232        	282
#define FRAME_death233        	283
#define FRAME_death234        	284
#define FRAME_death235        	285
#define FRAME_death236        	286
#define FRAME_death237        	287
#define FRAME_death238        	288
#define FRAME_death239        	289
#define FRAME_death240        	290
#define FRAME_death241        	291
#define FRAME_death242        	292
#define FRAME_death243        	293
#define FRAME_death244        	294
#define FRAME_death245        	295
#define FRAME_death246        	296
#define FRAME_death247        	297
#define FRAME_death248        	298
#define FRAME_death249        	299
#define FRAME_death250        	300
#define FRAME_death251        	301
#define FRAME_death252        	302
#define FRAME_death253        	303
#define FRAME_death254        	304
#define FRAME_death255        	305
#define FRAME_death256        	306
#define FRAME_death257        	307
#define FRAME_death258        	308
#define FRAME_death259        	309
#define FRAME_death260        	310
#define FRAME_death261        	311
#define FRAME_death262        	312
#define FRAME_death263        	313
#define FRAME_death264        	314
#define FRAME_death265        	315
#define FRAME_death266        	316
#define FRAME_death267        	317
#define FRAME_death268        	318
#define FRAME_death269        	319
#define FRAME_death270        	320
#define FRAME_death271        	321
#define FRAME_death272        	322
#define FRAME_death273        	323
#define FRAME_death274        	324
#define FRAME_death275        	325
#define FRAME_death276        	326
#define FRAME_death277        	327
#define FRAME_death278        	328
#define FRAME_death279        	329
#define FRAME_death280        	330
#define FRAME_death281        	331
#define FRAME_death282        	332
#define FRAME_death283        	333
#define FRAME_death284        	334
#define FRAME_death285        	335
#define FRAME_death286        	336
#define FRAME_death287        	337
#define FRAME_death288        	338
#define FRAME_death289        	339
#define FRAME_death290        	340
#define FRAME_death291        	341
#define FRAME_death292        	342
#define FRAME_death293        	343
#define FRAME_death294        	344
#define FRAME_death295        	345
#define FRAME_death301        	346
#define FRAME_death302        	347
#define FRAME_death303        	348
#define FRAME_death304        	349
#define FRAME_death305        	350
#define FRAME_death306        	351
#define FRAME_death307        	352
#define FRAME_death308        	353
#define FRAME_death309        	354
#define FRAME_death310        	355
#define FRAME_death311        	356
#define FRAME_death312        	357
#define FRAME_death313        	358
#define FRAME_death314        	359
#define FRAME_death315        	360
#define FRAME_death316        	361
#define FRAME_death317        	362
#define FRAME_death318        	363
#define FRAME_death319        	364
#define FRAME_death320        	365
#define FRAME_jump01          	366
#define FRAME_jump02          	367
#define FRAME_jump03          	368
#define FRAME_jump04          	369
#define FRAME_jump05          	370
#define FRAME_jump06          	371
#define FRAME_jump07          	372
#define FRAME_jump08          	373
#define FRAME_jump09          	374
#define FRAME_jump10          	375
#define FRAME_jump11          	376
#define FRAME_jump12          	377
#define FRAME_jump13          	378
#define FRAME_pain401         	379
#define FRAME_pain402         	380
#define FRAME_pain403         	381
#define FRAME_pain404         	382
#define FRAME_pain501         	383
#define FRAME_pain502         	384
#define FRAME_pain503         	385
#define FRAME_pain504         	386
#define FRAME_pain601         	387
#define FRAME_pain602         	388
#define FRAME_pain603         	389
#define FRAME_pain604         	390
#define FRAME_pain605         	391
#define FRAME_pain606         	392
#define FRAME_pain607         	393
#define FRAME_pain608         	394
#define FRAME_pain609         	395
#define FRAME_pain610         	396
#define FRAME_pain611         	397
#define FRAME_pain612         	398
#define FRAME_pain613         	399
#define FRAME_pain614         	400
#define FRAME_pain615         	401
#define FRAME_pain616         	402
#define FRAME_pain617         	403
#define FRAME_pain618         	404
#define FRAME_pain619         	405
#define FRAME_pain620         	406
#define FRAME_pain621         	407
#define FRAME_pain622         	408
#define FRAME_pain623         	409
#define FRAME_pain624         	410
#define FRAME_pain625         	411
#define FRAME_pain626         	412
#define FRAME_pain627         	413
#define FRAME_stand201        	414
#define FRAME_stand202        	415
#define FRAME_stand203        	416
#define FRAME_stand204        	417
#define FRAME_stand205        	418
#define FRAME_stand206        	419
#define FRAME_stand207        	420
#define FRAME_stand208        	421
#define FRAME_stand209        	422
#define FRAME_stand210        	423
#define FRAME_stand211        	424
#define FRAME_stand212        	425
#define FRAME_stand213        	426
#define FRAME_stand214        	427
#define FRAME_stand215        	428
#define FRAME_stand216        	429
#define FRAME_stand217        	430
#define FRAME_stand218        	431
#define FRAME_stand219        	432
#define FRAME_stand220        	433
#define FRAME_stand221        	434
#define FRAME_stand222        	435
#define FRAME_stand223        	436
#define FRAME_stand224        	437
#define FRAME_stand225        	438
#define FRAME_stand226        	439
#define FRAME_stand227        	440
#define FRAME_stand228        	441
#define FRAME_stand229        	442
#define FRAME_stand230        	443
#define FRAME_stand231        	444
#define FRAME_stand232        	445
#define FRAME_stand233        	446
#define FRAME_stand234        	447
#define FRAME_stand235        	448
#define FRAME_stand236        	449
#define FRAME_stand237        	450
#define FRAME_stand238        	451
#define FRAME_stand239        	452
#define FRAME_stand240        	453
#define FRAME_stand241        	454
#define FRAME_stand242        	455
#define FRAME_stand243        	456
#define FRAME_stand244        	457
#define FRAME_stand245        	458
#define FRAME_stand246        	459
#define FRAME_stand247        	460
#define FRAME_stand248        	461
#define FRAME_stand249        	462
#define FRAME_stand250        	463
#define FRAME_stand251        	464
#define FRAME_stand252        	465
#define FRAME_stand253        	466
#define FRAME_stand254        	467
#define FRAME_stand255        	468
#define FRAME_stand256        	469
#define FRAME_stand257        	470
#define FRAME_stand258        	471
#define FRAME_stand259        	472
#define FRAME_stand260        	473
#define FRAME_walk201         	474
#define FRAME_walk202         	475
#define FRAME_walk203         	476
#define FRAME_walk204         	477
#define FRAME_walk205         	478
#define FRAME_walk206         	479
#define FRAME_walk207         	480
#define FRAME_walk208         	481
#define FRAME_walk209         	482
#define FRAME_walk210         	483
#define FRAME_walk211         	484
#define FRAME_walk212         	485
#define FRAME_walk213         	486
#define FRAME_walk214         	487
#define FRAME_walk215         	488
#define FRAME_walk216         	489
#define FRAME_walk217         	490
#define MODEL_SCALE				1.000000
