/* =======================================================================
 *
 * Tank and Tank Commander animations.
 *
 * =======================================================================
 */
 
#define FRAME_stand01         	0
#define FRAME_stand02         	1
#define FRAME_stand03         	2
#define FRAME_stand04         	3
#define FRAME_stand05         	4
#define FRAME_stand06         	5
#define FRAME_stand07         	6
#define FRAME_stand08         	7
#define FRAME_stand09         	8
#define FRAME_stand10         	9
#define FRAME_stand11         	10
#define FRAME_stand12         	11
#define FRAME_stand13         	12
#define FRAME_stand14         	13
#define FRAME_stand15         	14
#define FRAME_stand16         	15
#define FRAME_stand17         	16
#define FRAME_stand18         	17
#define FRAME_stand19         	18
#define FRAME_stand20         	19
#define FRAME_stand21         	20
#define FRAME_stand22         	21
#define FRAME_stand23         	22
#define FRAME_stand24         	23
#define FRAME_stand25         	24
#define FRAME_stand26         	25
#define FRAME_stand27         	26
#define FRAME_stand28         	27
#define FRAME_stand29         	28
#define FRAME_stand30         	29
#define FRAME_walk01          	30
#define FRAME_walk02          	31
#define FRAME_walk03          	32
#define FRAME_walk04          	33
#define FRAME_walk05          	34
#define FRAME_walk06          	35
#define FRAME_walk07          	36
#define FRAME_walk08          	37
#define FRAME_walk09          	38
#define FRAME_walk10          	39
#define FRAME_walk11          	40
#define FRAME_walk12          	41
#define FRAME_walk13          	42
#define FRAME_walk14          	43
#define FRAME_walk15          	44
#define FRAME_walk16          	45
#define FRAME_walk17          	46
#define FRAME_walk18          	47
#define FRAME_walk19          	48
#define FRAME_walk20          	49
#define FRAME_walk21          	50
#define FRAME_walk22          	51
#define FRAME_walk23          	52
#define FRAME_walk24          	53
#define FRAME_walk25          	54
#define FRAME_attak101        	55
#define FRAME_attak102        	56
#define FRAME_attak103        	57
#define FRAME_attak104        	58
#define FRAME_attak105        	59
#define FRAME_attak106        	60
#define FRAME_attak107        	61
#define FRAME_attak108        	62
#define FRAME_attak109        	63
#define FRAME_attak110        	64
#define FRAME_attak111        	65
#define FRAME_attak112        	66
#define FRAME_attak113        	67
#define FRAME_attak114        	68
#define FRAME_attak115        	69
#define FRAME_attak116        	70
#define FRAME_attak117        	71
#define FRAME_attak118        	72
#define FRAME_attak119        	73
#define FRAME_attak120        	74
#define FRAME_attak121        	75
#define FRAME_attak122        	76
#define FRAME_attak201        	77
#define FRAME_attak202        	78
#define FRAME_attak203        	79
#define FRAME_attak204        	80
#define FRAME_attak205        	81
#define FRAME_attak206        	82
#define FRAME_attak207        	83
#define FRAME_attak208        	84
#define FRAME_attak209        	85
#define FRAME_attak210        	86
#define FRAME_attak211        	87
#define FRAME_attak212        	88
#define FRAME_attak213        	89
#define FRAME_attak214        	90
#define FRAME_attak215        	91
#define FRAME_attak216        	92
#define FRAME_attak217        	93
#define FRAME_attak218        	94
#define FRAME_attak219        	95
#define FRAME_attak220        	96
#define FRAME_attak221        	97
#define FRAME_attak222        	98
#define FRAME_attak223        	99
#define FRAME_attak224        	100
#define FRAME_attak225        	101
#define FRAME_attak226        	102
#define FRAME_attak227        	103
#define FRAME_attak228        	104
#define FRAME_attak229        	105
#define FRAME_attak230        	106
#define FRAME_attak231        	107
#define FRAME_attak232        	108
#define FRAME_attak233        	109
#define FRAME_attak234        	110
#define FRAME_attak235        	111
#define FRAME_attak236        	112
#define FRAME_attak237        	113
#define FRAME_attak238        	114
#define FRAME_attak301        	115
#define FRAME_attak302        	116
#define FRAME_attak303        	117
#define FRAME_attak304        	118
#define FRAME_attak305        	119
#define FRAME_attak306        	120
#define FRAME_attak307        	121
#define FRAME_attak308        	122
#define FRAME_attak309        	123
#define FRAME_attak310        	124
#define FRAME_attak311        	125
#define FRAME_attak312        	126
#define FRAME_attak313        	127
#define FRAME_attak314        	128
#define FRAME_attak315        	129
#define FRAME_attak316        	130
#define FRAME_attak317        	131
#define FRAME_attak318        	132
#define FRAME_attak319        	133
#define FRAME_attak320        	134
#define FRAME_attak321        	135
#define FRAME_attak322        	136
#define FRAME_attak323        	137
#define FRAME_attak324        	138
#define FRAME_attak325        	139
#define FRAME_attak326        	140
#define FRAME_attak327        	141
#define FRAME_attak328        	142
#define FRAME_attak329        	143
#define FRAME_attak330        	144
#define FRAME_attak331        	145
#define FRAME_attak332        	146
#define FRAME_attak333        	147
#define FRAME_attak334        	148
#define FRAME_attak335        	149
#define FRAME_attak336        	150
#define FRAME_attak337        	151
#define FRAME_attak338        	152
#define FRAME_attak339        	153
#define FRAME_attak340        	154
#define FRAME_attak341        	155
#define FRAME_attak342        	156
#define FRAME_attak343        	157
#define FRAME_attak344        	158
#define FRAME_attak345        	159
#define FRAME_attak346        	160
#define FRAME_attak347        	161
#define FRAME_attak348        	162
#define FRAME_attak349        	163
#define FRAME_attak350        	164
#define FRAME_attak351        	165
#define FRAME_attak352        	166
#define FRAME_attak353        	167
#define FRAME_attak401        	168
#define FRAME_attak402        	169
#define FRAME_attak403        	170
#define FRAME_attak404        	171
#define FRAME_attak405        	172
#define FRAME_attak406        	173
#define FRAME_attak407        	174
#define FRAME_attak408        	175
#define FRAME_attak409        	176
#define FRAME_attak410        	177
#define FRAME_attak411        	178
#define FRAME_attak412        	179
#define FRAME_attak413        	180
#define FRAME_attak414        	181
#define FRAME_attak415        	182
#define FRAME_attak416        	183
#define FRAME_attak417        	184
#define FRAME_attak418        	185
#define FRAME_attak419        	186
#define FRAME_attak420        	187
#define FRAME_attak421        	188
#define FRAME_attak422        	189
#define FRAME_attak423        	190
#define FRAME_attak424        	191
#define FRAME_attak425        	192
#define FRAME_attak426        	193
#define FRAME_attak427        	194
#define FRAME_attak428        	195
#define FRAME_attak429        	196
#define FRAME_pain101         	197
#define FRAME_pain102         	198
#define FRAME_pain103         	199
#define FRAME_pain104         	200
#define FRAME_pain201         	201
#define FRAME_pain202         	202
#define FRAME_pain203         	203
#define FRAME_pain204         	204
#define FRAME_pain205         	205
#define FRAME_pain301         	206
#define FRAME_pain302         	207
#define FRAME_pain303         	208
#define FRAME_pain304         	209
#define FRAME_pain305         	210
#define FRAME_pain306         	211
#define FRAME_pain307         	212
#define FRAME_pain308         	213
#define FRAME_pain309         	214
#define FRAME_pain310         	215
#define FRAME_pain311         	216
#define FRAME_pain312         	217
#define FRAME_pain313         	218
#define FRAME_pain314         	219
#define FRAME_pain315         	220
#define FRAME_pain316         	221
#define FRAME_death101        	222
#define FRAME_death102        	223
#define FRAME_death103        	224
#define FRAME_death104        	225
#define FRAME_death105        	226
#define FRAME_death106        	227
#define FRAME_death107        	228
#define FRAME_death108        	229
#define FRAME_death109        	230
#define FRAME_death110        	231
#define FRAME_death111        	232
#define FRAME_death112        	233
#define FRAME_death113        	234
#define FRAME_death114        	235
#define FRAME_death115        	236
#define FRAME_death116        	237
#define FRAME_death117        	238
#define FRAME_death118        	239
#define FRAME_death119        	240
#define FRAME_death120        	241
#define FRAME_death121        	242
#define FRAME_death122        	243
#define FRAME_death123        	244
#define FRAME_death124        	245
#define FRAME_death125        	246
#define FRAME_death126        	247
#define FRAME_death127        	248
#define FRAME_death128        	249
#define FRAME_death129        	250
#define FRAME_death130        	251
#define FRAME_death131        	252
#define FRAME_death132        	253
#define FRAME_recln101        	254
#define FRAME_recln102        	255
#define FRAME_recln103        	256
#define FRAME_recln104        	257
#define FRAME_recln105        	258
#define FRAME_recln106        	259
#define FRAME_recln107        	260
#define FRAME_recln108        	261
#define FRAME_recln109        	262
#define FRAME_recln110        	263
#define FRAME_recln111        	264
#define FRAME_recln112        	265
#define FRAME_recln113        	266
#define FRAME_recln114        	267
#define FRAME_recln115        	268
#define FRAME_recln116        	269
#define FRAME_recln117        	270
#define FRAME_recln118        	271
#define FRAME_recln119        	272
#define FRAME_recln120        	273
#define FRAME_recln121        	274
#define FRAME_recln122        	275
#define FRAME_recln123        	276
#define FRAME_recln124        	277
#define FRAME_recln125        	278
#define FRAME_recln126        	279
#define FRAME_recln127        	280
#define FRAME_recln128        	281
#define FRAME_recln129        	282
#define FRAME_recln130        	283
#define FRAME_recln131        	284
#define FRAME_recln132        	285
#define FRAME_recln133        	286
#define FRAME_recln134        	287
#define FRAME_recln135        	288
#define FRAME_recln136        	289
#define FRAME_recln137        	290
#define FRAME_recln138        	291
#define FRAME_recln139        	292
#define FRAME_recln140        	293
#define MODEL_SCALE				1.000000
