/* =======================================================================
 *
 * Gladiator animations.
 *
 * =======================================================================
 */
 
#define FRAME_stand1          	0
#define FRAME_stand2          	1
#define FRAME_stand3          	2
#define FRAME_stand4          	3
#define FRAME_stand5          	4
#define FRAME_stand6          	5
#define FRAME_stand7          	6
#define FRAME_walk1           	7
#define FRAME_walk2           	8
#define FRAME_walk3           	9
#define FRAME_walk4           	10
#define FRAME_walk5           	11
#define FRAME_walk6           	12
#define FRAME_walk7           	13
#define FRAME_walk8           	14
#define FRAME_walk9           	15
#define FRAME_walk10          	16
#define FRAME_walk11          	17
#define FRAME_walk12          	18
#define FRAME_walk13          	19
#define FRAME_walk14          	20
#define FRAME_walk15          	21
#define FRAME_walk16          	22
#define FRAME_run1            	23
#define FRAME_run2            	24
#define FRAME_run3            	25
#define FRAME_run4            	26
#define FRAME_run5            	27
#define FRAME_run6            	28
#define FRAME_melee1          	29
#define FRAME_melee2          	30
#define FRAME_melee3          	31
#define FRAME_melee4          	32
#define FRAME_melee5          	33
#define FRAME_melee6          	34
#define FRAME_melee7          	35
#define FRAME_melee8          	36
#define FRAME_melee9          	37
#define FRAME_melee10         	38
#define FRAME_melee11         	39
#define FRAME_melee12         	40
#define FRAME_melee13         	41
#define FRAME_melee14         	42
#define FRAME_melee15         	43
#define FRAME_melee16         	44
#define FRAME_melee17         	45
#define FRAME_attack1         	46
#define FRAME_attack2         	47
#define FRAME_attack3         	48
#define FRAME_attack4         	49
#define FRAME_attack5         	50
#define FRAME_attack6         	51
#define FRAME_attack7         	52
#define FRAME_attack8         	53
#define FRAME_attack9         	54
#define FRAME_pain1           	55
#define FRAME_pain2           	56
#define FRAME_pain3           	57
#define FRAME_pain4           	58
#define FRAME_pain5           	59
#define FRAME_pain6           	60
#define FRAME_death1          	61
#define FRAME_death2          	62
#define FRAME_death3          	63
#define FRAME_death4          	64
#define FRAME_death5          	65
#define FRAME_death6          	66
#define FRAME_death7          	67
#define FRAME_death8          	68
#define FRAME_death9          	69
#define FRAME_death10         	70
#define FRAME_death11         	71
#define FRAME_death12         	72
#define FRAME_death13         	73
#define FRAME_death14         	74
#define FRAME_death15         	75
#define FRAME_death16         	76
#define FRAME_death17         	77
#define FRAME_death18         	78
#define FRAME_death19         	79
#define FRAME_death20         	80
#define FRAME_death21         	81
#define FRAME_death22         	82
#define FRAME_painup1         	83
#define FRAME_painup2         	84
#define FRAME_painup3         	85
#define FRAME_painup4         	86
#define FRAME_painup5         	87
#define FRAME_painup6         	88
#define FRAME_painup7         	89
#define MODEL_SCALE				1.000000
