/* =======================================================================
 *
 * Infantry animations.
 *
 * =======================================================================
 */
 
#define FRAME_gun02           	0
#define FRAME_stand01         	1
#define FRAME_stand02         	2
#define FRAME_stand03         	3
#define FRAME_stand04         	4
#define FRAME_stand05         	5
#define FRAME_stand06         	6
#define FRAME_stand07         	7
#define FRAME_stand08         	8
#define FRAME_stand09         	9
#define FRAME_stand10         	10
#define FRAME_stand11         	11
#define FRAME_stand12         	12
#define FRAME_stand13         	13
#define FRAME_stand14         	14
#define FRAME_stand15         	15
#define FRAME_stand16         	16
#define FRAME_stand17         	17
#define FRAME_stand18         	18
#define FRAME_stand19         	19
#define FRAME_stand20         	20
#define FRAME_stand21         	21
#define FRAME_stand22         	22
#define FRAME_stand23         	23
#define FRAME_stand24         	24
#define FRAME_stand25         	25
#define FRAME_stand26         	26
#define FRAME_stand27         	27
#define FRAME_stand28         	28
#define FRAME_stand29         	29
#define FRAME_stand30         	30
#define FRAME_stand31         	31
#define FRAME_stand32         	32
#define FRAME_stand33         	33
#define FRAME_stand34         	34
#define FRAME_stand35         	35
#define FRAME_stand36         	36
#define FRAME_stand37         	37
#define FRAME_stand38         	38
#define FRAME_stand39         	39
#define FRAME_stand40         	40
#define FRAME_stand41         	41
#define FRAME_stand42         	42
#define FRAME_stand43         	43
#define FRAME_stand44         	44
#define FRAME_stand45         	45
#define FRAME_stand46         	46
#define FRAME_stand47         	47
#define FRAME_stand48         	48
#define FRAME_stand49         	49
#define FRAME_stand50         	50
#define FRAME_stand51         	51
#define FRAME_stand52         	52
#define FRAME_stand53         	53
#define FRAME_stand54         	54
#define FRAME_stand55         	55
#define FRAME_stand56         	56
#define FRAME_stand57         	57
#define FRAME_stand58         	58
#define FRAME_stand59         	59
#define FRAME_stand60         	60
#define FRAME_stand61         	61
#define FRAME_stand62         	62
#define FRAME_stand63         	63
#define FRAME_stand64         	64
#define FRAME_stand65         	65
#define FRAME_stand66         	66
#define FRAME_stand67         	67
#define FRAME_stand68         	68
#define FRAME_stand69         	69
#define FRAME_stand70         	70
#define FRAME_stand71         	71
#define FRAME_walk01          	72
#define FRAME_walk02          	73
#define FRAME_walk03          	74
#define FRAME_walk04          	75
#define FRAME_walk05          	76
#define FRAME_walk06          	77
#define FRAME_walk07          	78
#define FRAME_walk08          	79
#define FRAME_walk09          	80
#define FRAME_walk10          	81
#define FRAME_walk11          	82
#define FRAME_walk12          	83
#define FRAME_walk13          	84
#define FRAME_walk14          	85
#define FRAME_walk15          	86
#define FRAME_walk16          	87
#define FRAME_walk17          	88
#define FRAME_walk18          	89
#define FRAME_walk19          	90
#define FRAME_walk20          	91
#define FRAME_run01           	92
#define FRAME_run02           	93
#define FRAME_run03           	94
#define FRAME_run04           	95
#define FRAME_run05           	96
#define FRAME_run06           	97
#define FRAME_run07           	98
#define FRAME_run08           	99
#define FRAME_pain101         	100
#define FRAME_pain102         	101
#define FRAME_pain103         	102
#define FRAME_pain104         	103
#define FRAME_pain105         	104
#define FRAME_pain106         	105
#define FRAME_pain107         	106
#define FRAME_pain108         	107
#define FRAME_pain109         	108
#define FRAME_pain110         	109
#define FRAME_pain201         	110
#define FRAME_pain202         	111
#define FRAME_pain203         	112
#define FRAME_pain204         	113
#define FRAME_pain205         	114
#define FRAME_pain206         	115
#define FRAME_pain207         	116
#define FRAME_pain208         	117
#define FRAME_pain209         	118
#define FRAME_pain210         	119
#define FRAME_duck01          	120
#define FRAME_duck02          	121
#define FRAME_duck03          	122
#define FRAME_duck04          	123
#define FRAME_duck05          	124
#define FRAME_death101        	125
#define FRAME_death102        	126
#define FRAME_death103        	127
#define FRAME_death104        	128
#define FRAME_death105        	129
#define FRAME_death106        	130
#define FRAME_death107        	131
#define FRAME_death108        	132
#define FRAME_death109        	133
#define FRAME_death110        	134
#define FRAME_death111        	135
#define FRAME_death112        	136
#define FRAME_death113        	137
#define FRAME_death114        	138
#define FRAME_death115        	139
#define FRAME_death116        	140
#define FRAME_death117        	141
#define FRAME_death118        	142
#define FRAME_death119        	143
#define FRAME_death120        	144
#define FRAME_death201        	145
#define FRAME_death202        	146
#define FRAME_death203        	147
#define FRAME_death204        	148
#define FRAME_death205        	149
#define FRAME_death206        	150
#define FRAME_death207        	151
#define FRAME_death208        	152
#define FRAME_death209        	153
#define FRAME_death210        	154
#define FRAME_death211        	155
#define FRAME_death212        	156
#define FRAME_death213        	157
#define FRAME_death214        	158
#define FRAME_death215        	159
#define FRAME_death216        	160
#define FRAME_death217        	161
#define FRAME_death218        	162
#define FRAME_death219        	163
#define FRAME_death220        	164
#define FRAME_death221        	165
#define FRAME_death222        	166
#define FRAME_death223        	167
#define FRAME_death224        	168
#define FRAME_death225        	169
#define FRAME_death301        	170
#define FRAME_death302        	171
#define FRAME_death303        	172
#define FRAME_death304        	173
#define FRAME_death305        	174
#define FRAME_death306        	175
#define FRAME_death307        	176
#define FRAME_death308        	177
#define FRAME_death309        	178
#define FRAME_block01         	179
#define FRAME_block02         	180
#define FRAME_block03         	181
#define FRAME_block04         	182
#define FRAME_block05         	183
#define FRAME_attak101        	184
#define FRAME_attak102        	185
#define FRAME_attak103        	186
#define FRAME_attak104        	187
#define FRAME_attak105        	188
#define FRAME_attak106        	189
#define FRAME_attak107        	190
#define FRAME_attak108        	191
#define FRAME_attak109        	192
#define FRAME_attak110        	193
#define FRAME_attak111        	194
#define FRAME_attak112        	195
#define FRAME_attak113        	196
#define FRAME_attak114        	197
#define FRAME_attak115        	198
#define FRAME_attak201        	199
#define FRAME_attak202        	200
#define FRAME_attak203        	201
#define FRAME_attak204        	202
#define FRAME_attak205        	203
#define FRAME_attak206        	204
#define FRAME_attak207        	205
#define FRAME_attak208        	206
#define FRAME_jump01			207
#define FRAME_jump02			208
#define FRAME_jump03			209
#define FRAME_jump04			210
#define FRAME_jump05			211
#define FRAME_jump06			212
#define FRAME_jump07			213
#define FRAME_jump08			214
#define FRAME_jump09			215
#define FRAME_jump10			216
#define MODEL_SCALE				1.000000
