/* =======================================================================
 *
 * Icarus and Daedalus animations.
 *
 * =======================================================================
 */
 
#define FRAME_stand01         	0
#define FRAME_stand02         	1
#define FRAME_stand03         	2
#define FRAME_stand04         	3
#define FRAME_stand05         	4
#define FRAME_stand06         	5
#define FRAME_stand07         	6
#define FRAME_stand08         	7
#define FRAME_stand09         	8
#define FRAME_stand10         	9
#define FRAME_stand11         	10
#define FRAME_stand12         	11
#define FRAME_stand13         	12
#define FRAME_stand14         	13
#define FRAME_stand15         	14
#define FRAME_stand16         	15
#define FRAME_stand17         	16
#define FRAME_stand18         	17
#define FRAME_stand19         	18
#define FRAME_stand20         	19
#define FRAME_stand21         	20
#define FRAME_stand22         	21
#define FRAME_stand23         	22
#define FRAME_stand24         	23
#define FRAME_stand25         	24
#define FRAME_stand26         	25
#define FRAME_stand27         	26
#define FRAME_stand28         	27
#define FRAME_stand29         	28
#define FRAME_stand30         	29
#define FRAME_forwrd01        	30
#define FRAME_forwrd02        	31
#define FRAME_forwrd03        	32
#define FRAME_forwrd04        	33
#define FRAME_forwrd05        	34
#define FRAME_forwrd06        	35
#define FRAME_forwrd07        	36
#define FRAME_forwrd08        	37
#define FRAME_forwrd09        	38
#define FRAME_forwrd10        	39
#define FRAME_forwrd11        	40
#define FRAME_forwrd12        	41
#define FRAME_forwrd13        	42
#define FRAME_forwrd14        	43
#define FRAME_forwrd15        	44
#define FRAME_forwrd16        	45
#define FRAME_forwrd17        	46
#define FRAME_forwrd18        	47
#define FRAME_forwrd19        	48
#define FRAME_forwrd20        	49
#define FRAME_forwrd21        	50
#define FRAME_forwrd22        	51
#define FRAME_forwrd23        	52
#define FRAME_forwrd24        	53
#define FRAME_forwrd25        	54
#define FRAME_forwrd26        	55
#define FRAME_forwrd27        	56
#define FRAME_forwrd28        	57
#define FRAME_forwrd29        	58
#define FRAME_forwrd30        	59
#define FRAME_forwrd31        	60
#define FRAME_forwrd32        	61
#define FRAME_forwrd33        	62
#define FRAME_forwrd34        	63
#define FRAME_forwrd35        	64
#define FRAME_stop101         	65
#define FRAME_stop102         	66
#define FRAME_stop103         	67
#define FRAME_stop104         	68
#define FRAME_stop105         	69
#define FRAME_stop106         	70
#define FRAME_stop107         	71
#define FRAME_stop108         	72
#define FRAME_stop109         	73
#define FRAME_stop201         	74
#define FRAME_stop202         	75
#define FRAME_stop203         	76
#define FRAME_stop204         	77
#define FRAME_stop205         	78
#define FRAME_stop206         	79
#define FRAME_stop207         	80
#define FRAME_stop208         	81
#define FRAME_takeof01        	82
#define FRAME_takeof02        	83
#define FRAME_takeof03        	84
#define FRAME_takeof04        	85
#define FRAME_takeof05        	86
#define FRAME_takeof06        	87
#define FRAME_takeof07        	88
#define FRAME_takeof08        	89
#define FRAME_takeof09        	90
#define FRAME_takeof10        	91
#define FRAME_takeof11        	92
#define FRAME_takeof12        	93
#define FRAME_takeof13        	94
#define FRAME_takeof14        	95
#define FRAME_takeof15        	96
#define FRAME_takeof16        	97
#define FRAME_takeof17        	98
#define FRAME_takeof18        	99
#define FRAME_takeof19        	100
#define FRAME_takeof20        	101
#define FRAME_takeof21        	102
#define FRAME_takeof22        	103
#define FRAME_takeof23        	104
#define FRAME_takeof24        	105
#define FRAME_takeof25        	106
#define FRAME_takeof26        	107
#define FRAME_takeof27        	108
#define FRAME_takeof28        	109
#define FRAME_takeof29        	110
#define FRAME_takeof30        	111
#define FRAME_land01          	112
#define FRAME_pain101         	113
#define FRAME_pain102         	114
#define FRAME_pain103         	115
#define FRAME_pain104         	116
#define FRAME_pain105         	117
#define FRAME_pain106         	118
#define FRAME_pain107         	119
#define FRAME_pain108         	120
#define FRAME_pain109         	121
#define FRAME_pain110         	122
#define FRAME_pain111         	123
#define FRAME_pain112         	124
#define FRAME_pain113         	125
#define FRAME_pain114         	126
#define FRAME_pain115         	127
#define FRAME_pain116         	128
#define FRAME_pain117         	129
#define FRAME_pain118         	130
#define FRAME_pain119         	131
#define FRAME_pain120         	132
#define FRAME_pain121         	133
#define FRAME_pain122         	134
#define FRAME_pain123         	135
#define FRAME_pain124         	136
#define FRAME_pain125         	137
#define FRAME_pain126         	138
#define FRAME_pain127         	139
#define FRAME_pain128         	140
#define FRAME_pain201         	141
#define FRAME_pain202         	142
#define FRAME_pain203         	143
#define FRAME_pain204         	144
#define FRAME_pain205         	145
#define FRAME_pain206         	146
#define FRAME_pain207         	147
#define FRAME_pain208         	148
#define FRAME_pain209         	149
#define FRAME_pain210         	150
#define FRAME_pain211         	151
#define FRAME_pain212         	152
#define FRAME_pain301         	153
#define FRAME_pain302         	154
#define FRAME_pain303         	155
#define FRAME_pain304         	156
#define FRAME_pain305         	157
#define FRAME_pain306         	158
#define FRAME_pain307         	159
#define FRAME_pain308         	160
#define FRAME_pain309         	161
#define FRAME_death101        	162
#define FRAME_death102        	163
#define FRAME_death103        	164
#define FRAME_death104        	165
#define FRAME_death105        	166
#define FRAME_death106        	167
#define FRAME_death107        	168
#define FRAME_death108        	169
#define FRAME_death109        	170
#define FRAME_death110        	171
#define FRAME_death111        	172
#define FRAME_backwd01        	173
#define FRAME_backwd02        	174
#define FRAME_backwd03        	175
#define FRAME_backwd04        	176
#define FRAME_backwd05        	177
#define FRAME_backwd06        	178
#define FRAME_backwd07        	179
#define FRAME_backwd08        	180
#define FRAME_backwd09        	181
#define FRAME_backwd10        	182
#define FRAME_backwd11        	183
#define FRAME_backwd12        	184
#define FRAME_backwd13        	185
#define FRAME_backwd14        	186
#define FRAME_backwd15        	187
#define FRAME_backwd16        	188
#define FRAME_backwd17        	189
#define FRAME_backwd18        	190
#define FRAME_backwd19        	191
#define FRAME_backwd20        	192
#define FRAME_backwd21        	193
#define FRAME_backwd22        	194
#define FRAME_backwd23        	195
#define FRAME_backwd24        	196
#define FRAME_attak101        	197
#define FRAME_attak102        	198
#define FRAME_attak103        	199
#define FRAME_attak104        	200
#define FRAME_attak105        	201
#define FRAME_attak106        	202
#define FRAME_attak107        	203
#define FRAME_attak108        	204
#define MODEL_SCALE				1.000000
