/* =======================================================================
 *
 * Iron Maiden animations.
 *
 * =======================================================================
 */
 
#define FRAME_attak101        	0
#define FRAME_attak102        	1
#define FRAME_attak103        	2
#define FRAME_attak104        	3
#define FRAME_attak105        	4
#define FRAME_attak106        	5
#define FRAME_attak107        	6
#define FRAME_attak108        	7
#define FRAME_attak109        	8
#define FRAME_attak110        	9
#define FRAME_attak111        	10
#define FRAME_attak112        	11
#define FRAME_attak113        	12
#define FRAME_attak114        	13
#define FRAME_attak115        	14
#define FRAME_attak116        	15
#define FRAME_attak117        	16
#define FRAME_attak118        	17
#define FRAME_attak119        	18
#define FRAME_attak120        	19
#define FRAME_attak121        	20
#define FRAME_attak122        	21
#define FRAME_attak123        	22
#define FRAME_attak124        	23
#define FRAME_attak125        	24
#define FRAME_attak126        	25
#define FRAME_attak127        	26
#define FRAME_attak128        	27
#define FRAME_attak129        	28
#define FRAME_attak130        	29
#define FRAME_attak131        	30
#define FRAME_attak132        	31
#define FRAME_attak201        	32
#define FRAME_attak202        	33
#define FRAME_attak203        	34
#define FRAME_attak204        	35
#define FRAME_attak205        	36
#define FRAME_attak206        	37
#define FRAME_attak207        	38
#define FRAME_attak208        	39
#define FRAME_attak209        	40
#define FRAME_attak210        	41
#define FRAME_attak211        	42
#define FRAME_attak212        	43
#define FRAME_attak213        	44
#define FRAME_attak214        	45
#define FRAME_attak215        	46
#define FRAME_attak216        	47
#define FRAME_death101        	48
#define FRAME_death102        	49
#define FRAME_death103        	50
#define FRAME_death104        	51
#define FRAME_death105        	52
#define FRAME_death106        	53
#define FRAME_death107        	54
#define FRAME_death108        	55
#define FRAME_death109        	56
#define FRAME_death110        	57
#define FRAME_death111        	58
#define FRAME_death112        	59
#define FRAME_death201        	60
#define FRAME_death202        	61
#define FRAME_death203        	62
#define FRAME_death204        	63
#define FRAME_death205        	64
#define FRAME_death206        	65
#define FRAME_death207        	66
#define FRAME_death208        	67
#define FRAME_death209        	68
#define FRAME_death210        	69
#define FRAME_death211        	70
#define FRAME_death212        	71
#define FRAME_death213        	72
#define FRAME_death214        	73
#define FRAME_death215        	74
#define FRAME_death216        	75
#define FRAME_death217        	76
#define FRAME_death218        	77
#define FRAME_death219        	78
#define FRAME_death220        	79
#define FRAME_death221        	80
#define FRAME_death222        	81
#define FRAME_death223        	82
#define FRAME_duck01          	83
#define FRAME_duck02          	84
#define FRAME_duck03          	85
#define FRAME_duck04          	86
#define FRAME_duck05          	87
#define FRAME_duck06          	88
#define FRAME_duck07          	89
#define FRAME_pain101         	90
#define FRAME_pain102         	91
#define FRAME_pain103         	92
#define FRAME_pain104         	93
#define FRAME_pain105         	94
#define FRAME_pain201         	95
#define FRAME_pain202         	96
#define FRAME_pain203         	97
#define FRAME_pain204         	98
#define FRAME_pain205         	99
#define FRAME_pain301         	100
#define FRAME_pain302         	101
#define FRAME_pain303         	102
#define FRAME_pain304         	103
#define FRAME_pain305         	104
#define FRAME_pain306         	105
#define FRAME_pain307         	106
#define FRAME_pain308         	107
#define FRAME_pain309         	108
#define FRAME_pain310         	109
#define FRAME_pain311         	110
#define FRAME_pain312         	111
#define FRAME_pain313         	112
#define FRAME_pain314         	113
#define FRAME_pain315         	114
#define FRAME_pain316         	115
#define FRAME_pain317         	116
#define FRAME_pain318         	117
#define FRAME_pain319         	118
#define FRAME_pain320         	119
#define FRAME_pain321         	120
#define FRAME_stand101        	121
#define FRAME_stand102        	122
#define FRAME_stand103        	123
#define FRAME_stand104        	124
#define FRAME_stand105        	125
#define FRAME_stand106        	126
#define FRAME_stand107        	127
#define FRAME_stand108        	128
#define FRAME_stand109        	129
#define FRAME_stand110        	130
#define FRAME_stand111        	131
#define FRAME_stand112        	132
#define FRAME_stand113        	133
#define FRAME_stand114        	134
#define FRAME_stand115        	135
#define FRAME_stand116        	136
#define FRAME_stand117        	137
#define FRAME_stand118        	138
#define FRAME_stand119        	139
#define FRAME_stand120        	140
#define FRAME_stand121        	141
#define FRAME_stand122        	142
#define FRAME_stand123        	143
#define FRAME_stand124        	144
#define FRAME_stand125        	145
#define FRAME_stand126        	146
#define FRAME_stand127        	147
#define FRAME_stand128        	148
#define FRAME_stand129        	149
#define FRAME_stand130        	150
#define FRAME_stand201        	151
#define FRAME_stand202        	152
#define FRAME_stand203        	153
#define FRAME_stand204        	154
#define FRAME_stand205        	155
#define FRAME_stand206        	156
#define FRAME_stand207        	157
#define FRAME_stand208        	158
#define FRAME_stand209        	159
#define FRAME_stand210        	160
#define FRAME_stand211        	161
#define FRAME_stand212        	162
#define FRAME_stand213        	163
#define FRAME_stand214        	164
#define FRAME_stand215        	165
#define FRAME_stand216        	166
#define FRAME_stand217        	167
#define FRAME_stand218        	168
#define FRAME_stand219        	169
#define FRAME_stand220        	170
#define FRAME_stand221        	171
#define FRAME_stand222        	172
#define FRAME_stand223        	173
#define FRAME_stand224        	174
#define FRAME_stand225        	175
#define FRAME_stand226        	176
#define FRAME_stand227        	177
#define FRAME_stand228        	178
#define FRAME_stand229        	179
#define FRAME_stand230        	180
#define FRAME_walk01          	181
#define FRAME_walk02          	182
#define FRAME_walk03          	183
#define FRAME_walk04          	184
#define FRAME_walk05          	185
#define FRAME_walk06          	186
#define FRAME_walk07          	187
#define FRAME_walk08          	188
#define FRAME_walk09          	189
#define FRAME_walk10          	190
#define FRAME_walk11          	191
#define FRAME_walk12          	192
#define FRAME_walk13          	193
#define FRAME_walk14          	194
#define FRAME_walk15          	195
#define FRAME_walk16          	196
#define FRAME_walk17          	197
#define FRAME_walk18          	198
#define FRAME_walk19          	199
#define FRAME_walk20          	200
#define FRAME_walk21          	201
#define FRAME_walk22          	202
#define FRAME_walk23          	203
#define FRAME_walk24          	204
#define FRAME_walk25          	205
#define FRAME_walk26          	206
#define FRAME_walk27          	207
#define FRAME_recln201        	208
#define FRAME_recln202        	209
#define FRAME_recln203        	210
#define FRAME_recln204        	211
#define FRAME_recln205        	212
#define FRAME_recln206        	213
#define FRAME_recln207        	214
#define FRAME_recln208        	215
#define FRAME_recln209        	216
#define FRAME_recln210        	217
#define FRAME_recln211        	218
#define FRAME_recln212        	219
#define FRAME_recln213        	220
#define FRAME_recln214        	221
#define FRAME_recln215        	222
#define FRAME_recln216        	223
#define FRAME_recln217        	224
#define FRAME_recln218        	225
#define FRAME_recln219        	226
#define FRAME_recln220        	227
#define FRAME_recln221        	228
#define FRAME_recln222        	229
#define FRAME_recln223        	230
#define FRAME_recln224        	231
#define FRAME_recln225        	232
#define FRAME_recln226        	233
#define FRAME_recln227        	234
#define FRAME_recln228        	235
#define FRAME_recln229        	236
#define FRAME_recln230        	237
#define FRAME_recln231        	238
#define FRAME_recln232        	239
#define FRAME_recln233        	240
#define FRAME_recln234        	241
#define FRAME_recln235        	242
#define FRAME_recln236        	243
#define FRAME_recln237        	244
#define FRAME_recln238        	245
#define FRAME_recln239        	246
#define FRAME_recln240        	247
#define FRAME_recln101        	248
#define FRAME_recln102        	249
#define FRAME_recln103        	250
#define FRAME_recln104        	251
#define FRAME_recln105        	252
#define FRAME_recln106        	253
#define FRAME_recln107        	254
#define FRAME_recln108        	255
#define FRAME_recln109        	256
#define FRAME_recln110        	257
#define FRAME_recln111        	258
#define FRAME_recln112        	259
#define FRAME_recln113        	260
#define FRAME_recln114        	261
#define FRAME_recln115        	262
#define FRAME_recln116        	263
#define FRAME_recln117        	264
#define FRAME_recln118        	265
#define FRAME_recln119        	266
#define FRAME_recln120        	267
#define FRAME_recln121        	268
#define FRAME_recln122        	269
#define FRAME_recln123        	270
#define FRAME_recln124        	271
#define FRAME_recln125        	272
#define FRAME_recln126        	273
#define FRAME_recln127        	274
#define FRAME_recln128        	275
#define FRAME_recln129        	276
#define FRAME_recln130        	277
#define FRAME_recln131        	278
#define FRAME_recln132        	279
#define FRAME_recln133        	280
#define FRAME_recln134        	281
#define FRAME_recln135        	282
#define FRAME_recln136        	283
#define FRAME_recln137        	284
#define FRAME_recln138        	285
#define FRAME_recln139        	286
#define FRAME_recln140        	287
#define MODEL_SCALE				1.000000
