/* =======================================================================
 *
 * Berserker animations.
 *
 * =======================================================================
 */
 
#define FRAME_stand1          	0
#define FRAME_stand2          	1
#define FRAME_stand3          	2
#define FRAME_stand4          	3
#define FRAME_stand5          	4
#define FRAME_standb1         	5
#define FRAME_standb2         	6
#define FRAME_standb3         	7
#define FRAME_standb4         	8
#define FRAME_standb5         	9
#define FRAME_standb6         	10
#define FRAME_standb7         	11
#define FRAME_standb8         	12
#define FRAME_standb9         	13
#define FRAME_standb10        	14
#define FRAME_standb11        	15
#define FRAME_standb12        	16
#define FRAME_standb13        	17
#define FRAME_standb14        	18
#define FRAME_standb15        	19
#define FRAME_standb16        	20
#define FRAME_standb17        	21
#define FRAME_standb18        	22
#define FRAME_standb19        	23
#define FRAME_standb20        	24
#define FRAME_walkc1          	25
#define FRAME_walkc2          	26
#define FRAME_walkc3          	27
#define FRAME_walkc4          	28
#define FRAME_walkc5          	29
#define FRAME_walkc6          	30
#define FRAME_walkc7          	31
#define FRAME_walkc8          	32
#define FRAME_walkc9          	33
#define FRAME_walkc10         	34
#define FRAME_walkc11         	35
#define FRAME_run1            	36
#define FRAME_run2            	37
#define FRAME_run3            	38
#define FRAME_run4            	39
#define FRAME_run5            	40
#define FRAME_run6            	41
#define FRAME_att_a1          	42
#define FRAME_att_a2          	43
#define FRAME_att_a3          	44
#define FRAME_att_a4          	45
#define FRAME_att_a5          	46
#define FRAME_att_a6          	47
#define FRAME_att_a7          	48
#define FRAME_att_a8          	49
#define FRAME_att_a9          	50
#define FRAME_att_a10         	51
#define FRAME_att_a11         	52
#define FRAME_att_a12         	53
#define FRAME_att_a13         	54
#define FRAME_att_b1          	55
#define FRAME_att_b2          	56
#define FRAME_att_b3          	57
#define FRAME_att_b4          	58
#define FRAME_att_b5          	59
#define FRAME_att_b6          	60
#define FRAME_att_b7          	61
#define FRAME_att_b8          	62
#define FRAME_att_b9          	63
#define FRAME_att_b10         	64
#define FRAME_att_b11         	65
#define FRAME_att_b12         	66
#define FRAME_att_b13         	67
#define FRAME_att_b14         	68
#define FRAME_att_b15         	69
#define FRAME_att_b16         	70
#define FRAME_att_b17         	71
#define FRAME_att_b18         	72
#define FRAME_att_b19         	73
#define FRAME_att_b20         	74
#define FRAME_att_b21         	75
#define FRAME_att_c1          	76
#define FRAME_att_c2          	77
#define FRAME_att_c3          	78
#define FRAME_att_c4          	79
#define FRAME_att_c5          	80
#define FRAME_att_c6          	81
#define FRAME_att_c7          	82
#define FRAME_att_c8          	83
#define FRAME_att_c9          	84
#define FRAME_att_c10         	85
#define FRAME_att_c11         	86
#define FRAME_att_c12         	87
#define FRAME_att_c13         	88
#define FRAME_att_c14         	89
#define FRAME_att_c15         	90
#define FRAME_att_c16         	91
#define FRAME_att_c17         	92
#define FRAME_att_c18         	93
#define FRAME_att_c19         	94
#define FRAME_att_c20         	95
#define FRAME_att_c21         	96
#define FRAME_att_c22         	97
#define FRAME_att_c23         	98
#define FRAME_att_c24         	99
#define FRAME_att_c25         	100
#define FRAME_att_c26         	101
#define FRAME_att_c27         	102
#define FRAME_att_c28         	103
#define FRAME_att_c29         	104
#define FRAME_att_c30         	105
#define FRAME_att_c31         	106
#define FRAME_att_c32         	107
#define FRAME_att_c33         	108
#define FRAME_att_c34         	109
#define FRAME_r_att1          	110
#define FRAME_r_att2          	111
#define FRAME_r_att3          	112
#define FRAME_r_att4          	113
#define FRAME_r_att5          	114
#define FRAME_r_att6          	115
#define FRAME_r_att7          	116
#define FRAME_r_att8          	117
#define FRAME_r_att9          	118
#define FRAME_r_att10         	119
#define FRAME_r_att11         	120
#define FRAME_r_att12         	121
#define FRAME_r_att13         	122
#define FRAME_r_att14         	123
#define FRAME_r_att15         	124
#define FRAME_r_att16         	125
#define FRAME_r_att17         	126
#define FRAME_r_att18         	127
#define FRAME_r_attb1         	128
#define FRAME_r_attb2         	129
#define FRAME_r_attb3         	130
#define FRAME_r_attb4         	131
#define FRAME_r_attb5         	132
#define FRAME_r_attb6         	133
#define FRAME_r_attb7         	134
#define FRAME_r_attb8         	135
#define FRAME_r_attb9         	136
#define FRAME_r_attb10        	137
#define FRAME_r_attb11        	138
#define FRAME_r_attb12        	139
#define FRAME_r_attb13        	140
#define FRAME_r_attb14        	141
#define FRAME_r_attb15        	142
#define FRAME_r_attb16        	143
#define FRAME_r_attb17        	144
#define FRAME_r_attb18        	145
#define FRAME_slam1           	146
#define FRAME_slam2           	147
#define FRAME_slam3           	148
#define FRAME_slam4           	149
#define FRAME_slam5           	150
#define FRAME_slam6           	151
#define FRAME_slam7           	152
#define FRAME_slam8           	153
#define FRAME_slam9           	154
#define FRAME_slam10          	155
#define FRAME_slam11          	156
#define FRAME_slam12          	157
#define FRAME_slam13          	158
#define FRAME_slam14          	159
#define FRAME_slam15          	160
#define FRAME_slam16          	161
#define FRAME_slam17          	162
#define FRAME_slam18          	163
#define FRAME_slam19          	164
#define FRAME_slam20          	165
#define FRAME_slam21          	166
#define FRAME_slam22          	167
#define FRAME_slam23          	168
#define FRAME_duck1           	169
#define FRAME_duck2           	170
#define FRAME_duck3           	171
#define FRAME_duck4           	172
#define FRAME_duck5           	173
#define FRAME_duck6           	174
#define FRAME_duck7           	175
#define FRAME_duck8           	176
#define FRAME_duck9           	177
#define FRAME_duck10          	178
#define FRAME_fall1           	179
#define FRAME_fall2           	180
#define FRAME_fall3           	181
#define FRAME_fall4           	182
#define FRAME_fall5           	183
#define FRAME_fall6           	184
#define FRAME_fall7           	185
#define FRAME_fall8           	186
#define FRAME_fall9           	187
#define FRAME_fall10          	188
#define FRAME_fall11          	189
#define FRAME_fall12          	190
#define FRAME_fall13          	191
#define FRAME_fall14          	192
#define FRAME_fall15          	193
#define FRAME_fall16          	194
#define FRAME_fall17          	195
#define FRAME_fall18          	196
#define FRAME_fall19          	197
#define FRAME_fall20          	198
#define FRAME_painc1          	199
#define FRAME_painc2          	200
#define FRAME_painc3          	201
#define FRAME_painc4          	202
#define FRAME_painb1          	203
#define FRAME_painb2          	204
#define FRAME_painb3          	205
#define FRAME_painb4          	206
#define FRAME_painb5          	207
#define FRAME_painb6          	208
#define FRAME_painb7          	209
#define FRAME_painb8          	210
#define FRAME_painb9          	211
#define FRAME_painb10         	212
#define FRAME_painb11         	213
#define FRAME_painb12         	214
#define FRAME_painb13         	215
#define FRAME_painb14         	216
#define FRAME_painb15         	217
#define FRAME_painb16         	218
#define FRAME_painb17         	219
#define FRAME_painb18         	220
#define FRAME_painb19         	221
#define FRAME_painb20         	222
#define FRAME_death1          	223
#define FRAME_death2          	224
#define FRAME_death3          	225
#define FRAME_death4          	226
#define FRAME_death5          	227
#define FRAME_death6          	228
#define FRAME_death7          	229
#define FRAME_death8          	230
#define FRAME_death9          	231
#define FRAME_death10         	232
#define FRAME_death11         	233
#define FRAME_death12         	234
#define FRAME_death13         	235
#define FRAME_deathc1         	236
#define FRAME_deathc2         	237
#define FRAME_deathc3         	238
#define FRAME_deathc4         	239
#define FRAME_deathc5         	240
#define FRAME_deathc6         	241
#define FRAME_deathc7         	242
#define FRAME_deathc8         	243
#define FRAME_jump1				244
#define FRAME_jump2				245
#define FRAME_jump3				246
#define FRAME_jump4				247
#define FRAME_jump5				248
#define FRAME_jump6				249
#define FRAME_jump7				250
#define FRAME_jump8				251
#define FRAME_jump9				252
#define MODEL_SCALE				1.000000
