/*
 * =======================================================================
 *
 * Fields inside a level to be saved.
 *
 * =======================================================================
 */ 

{"changemap", LLOFS(changemap), F_LSTRING},
{"sight_client", LLOFS(sight_client), F_EDICT},
{"sight_entity", LLOFS(sight_entity), F_EDICT},
{"sound_entity", LLOFS(sound_entity), F_EDICT},
{"sound2_entity", LLOFS(sound2_entity), F_EDICT},
{"disguise_violator", LLOFS(disguise_violator), F_EDICT},
