/* =======================================================================
 *
 * Brain animations.
 *
 * =======================================================================
 *
 */
 
#define FRAME_walk101         	0
#define FRAME_walk102         	1
#define FRAME_walk103         	2
#define FRAME_walk104         	3
#define FRAME_walk105         	4
#define FRAME_walk106         	5
#define FRAME_walk107         	6
#define FRAME_walk108         	7
#define FRAME_walk109         	8
#define FRAME_walk110         	9
#define FRAME_walk111         	10
#define FRAME_walk112         	11
#define FRAME_walk113         	12
#define FRAME_walk201         	13
#define FRAME_walk202         	14
#define FRAME_walk203         	15
#define FRAME_walk204         	16
#define FRAME_walk205         	17
#define FRAME_walk206         	18
#define FRAME_walk207         	19
#define FRAME_walk208         	20
#define FRAME_walk209         	21
#define FRAME_walk210         	22
#define FRAME_walk211         	23
#define FRAME_walk212         	24
#define FRAME_walk213         	25
#define FRAME_walk214         	26
#define FRAME_walk215         	27
#define FRAME_walk216         	28
#define FRAME_walk217         	29
#define FRAME_walk218         	30
#define FRAME_walk219         	31
#define FRAME_walk220         	32
#define FRAME_walk221         	33
#define FRAME_walk222         	34
#define FRAME_walk223         	35
#define FRAME_walk224         	36
#define FRAME_walk225         	37
#define FRAME_walk226         	38
#define FRAME_walk227         	39
#define FRAME_walk228         	40
#define FRAME_walk229         	41
#define FRAME_walk230         	42
#define FRAME_walk231         	43
#define FRAME_walk232         	44
#define FRAME_walk233         	45
#define FRAME_walk234         	46
#define FRAME_walk235         	47
#define FRAME_walk236         	48
#define FRAME_walk237         	49
#define FRAME_walk238         	50
#define FRAME_walk239         	51
#define FRAME_walk240         	52
#define FRAME_attak101        	53
#define FRAME_attak102        	54
#define FRAME_attak103        	55
#define FRAME_attak104        	56
#define FRAME_attak105        	57
#define FRAME_attak106        	58
#define FRAME_attak107        	59
#define FRAME_attak108        	60
#define FRAME_attak109        	61
#define FRAME_attak110        	62
#define FRAME_attak111        	63
#define FRAME_attak112        	64
#define FRAME_attak113        	65
#define FRAME_attak114        	66
#define FRAME_attak115        	67
#define FRAME_attak116        	68
#define FRAME_attak117        	69
#define FRAME_attak118        	70
#define FRAME_attak201        	71
#define FRAME_attak202        	72
#define FRAME_attak203        	73
#define FRAME_attak204        	74
#define FRAME_attak205        	75
#define FRAME_attak206        	76
#define FRAME_attak207        	77
#define FRAME_attak208        	78
#define FRAME_attak209        	79
#define FRAME_attak210        	80
#define FRAME_attak211        	81
#define FRAME_attak212        	82
#define FRAME_attak213        	83
#define FRAME_attak214        	84
#define FRAME_attak215        	85
#define FRAME_attak216        	86
#define FRAME_attak217        	87
#define FRAME_pain101         	88
#define FRAME_pain102         	89
#define FRAME_pain103         	90
#define FRAME_pain104         	91
#define FRAME_pain105         	92
#define FRAME_pain106         	93
#define FRAME_pain107         	94
#define FRAME_pain108         	95
#define FRAME_pain109         	96
#define FRAME_pain110         	97
#define FRAME_pain111         	98
#define FRAME_pain112         	99
#define FRAME_pain113         	100
#define FRAME_pain114         	101
#define FRAME_pain115         	102
#define FRAME_pain116         	103
#define FRAME_pain117         	104
#define FRAME_pain118         	105
#define FRAME_pain119         	106
#define FRAME_pain120         	107
#define FRAME_pain121         	108
#define FRAME_pain201         	109
#define FRAME_pain202         	110
#define FRAME_pain203         	111
#define FRAME_pain204         	112
#define FRAME_pain205         	113
#define FRAME_pain206         	114
#define FRAME_pain207         	115
#define FRAME_pain208         	116
#define FRAME_pain301         	117
#define FRAME_pain302         	118
#define FRAME_pain303         	119
#define FRAME_pain304         	120
#define FRAME_pain305         	121
#define FRAME_pain306         	122
#define FRAME_death101        	123
#define FRAME_death102        	124
#define FRAME_death103        	125
#define FRAME_death104        	126
#define FRAME_death105        	127
#define FRAME_death106        	128
#define FRAME_death107        	129
#define FRAME_death108        	130
#define FRAME_death109        	131
#define FRAME_death110        	132
#define FRAME_death111        	133
#define FRAME_death112        	134
#define FRAME_death113        	135
#define FRAME_death114        	136
#define FRAME_death115        	137
#define FRAME_death116        	138
#define FRAME_death117        	139
#define FRAME_death118        	140
#define FRAME_death201        	141
#define FRAME_death202        	142
#define FRAME_death203        	143
#define FRAME_death204        	144
#define FRAME_death205        	145
#define FRAME_duck01          	146
#define FRAME_duck02          	147
#define FRAME_duck03          	148
#define FRAME_duck04          	149
#define FRAME_duck05          	150
#define FRAME_duck06          	151
#define FRAME_duck07          	152
#define FRAME_duck08          	153
#define FRAME_defens01        	154
#define FRAME_defens02        	155
#define FRAME_defens03        	156
#define FRAME_defens04        	157
#define FRAME_defens05        	158
#define FRAME_defens06        	159
#define FRAME_defens07        	160
#define FRAME_defens08        	161
#define FRAME_stand01         	162
#define FRAME_stand02         	163
#define FRAME_stand03         	164
#define FRAME_stand04         	165
#define FRAME_stand05         	166
#define FRAME_stand06         	167
#define FRAME_stand07         	168
#define FRAME_stand08         	169
#define FRAME_stand09         	170
#define FRAME_stand10         	171
#define FRAME_stand11         	172
#define FRAME_stand12         	173
#define FRAME_stand13         	174
#define FRAME_stand14         	175
#define FRAME_stand15         	176
#define FRAME_stand16         	177
#define FRAME_stand17         	178
#define FRAME_stand18         	179
#define FRAME_stand19         	180
#define FRAME_stand20         	181
#define FRAME_stand21         	182
#define FRAME_stand22         	183
#define FRAME_stand23         	184
#define FRAME_stand24         	185
#define FRAME_stand25         	186
#define FRAME_stand26         	187
#define FRAME_stand27         	188
#define FRAME_stand28         	189
#define FRAME_stand29         	190
#define FRAME_stand30         	191
#define FRAME_stand31         	192
#define FRAME_stand32         	193
#define FRAME_stand33         	194
#define FRAME_stand34         	195
#define FRAME_stand35         	196
#define FRAME_stand36         	197
#define FRAME_stand37         	198
#define FRAME_stand38         	199
#define FRAME_stand39         	200
#define FRAME_stand40         	201
#define FRAME_stand41         	202
#define FRAME_stand42         	203
#define FRAME_stand43         	204
#define FRAME_stand44         	205
#define FRAME_stand45         	206
#define FRAME_stand46         	207
#define FRAME_stand47         	208
#define FRAME_stand48         	209
#define FRAME_stand49         	210
#define FRAME_stand50         	211
#define FRAME_stand51         	212
#define FRAME_stand52         	213
#define FRAME_stand53         	214
#define FRAME_stand54         	215
#define FRAME_stand55         	216
#define FRAME_stand56         	217
#define FRAME_stand57         	218
#define FRAME_stand58         	219
#define FRAME_stand59         	220
#define FRAME_stand60         	221
#define MODEL_SCALE				1.000000
