/*
 * Copyright (C) 1997-2001 Id Software, Inc.
 * Copyright (C) 2011 Yamagi Burmeister
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 *
 * =======================================================================
 *
 * Functionpointers to every function in the game.so.
 *
 * =======================================================================
 */

{"ReadLevel", (byte *)ReadLevel},
{"WriteLevel", (byte *)WriteLevel},
{"ReadGame", (byte *)ReadGame},
{"WriteGame", (byte *)WriteGame},
{"InitGame", (byte *)InitGame},
{"Info_SetValueForKey", (byte *)Info_SetValueForKey},
{"Info_Validate", (byte *)Info_Validate},
{"Info_RemoveKey", (byte *)Info_RemoveKey},
{"Info_ValueForKey", (byte *)Info_ValueForKey},
{"Com_sprintf", (byte *)Com_sprintf},
{"Q_strcasecmp", (byte *)Q_strcasecmp},
{"Q_strncasecmp", (byte *)Q_strncasecmp},
{"Q_stricmp", (byte *)Q_stricmp},
{"Com_PageInMemory", (byte *)Com_PageInMemory},
{"COM_Parse", (byte *)COM_Parse},
{"va", (byte *)va},
{"Swap_Init", (byte *)Swap_Init},
{"LittleFloat", (byte *)LittleFloat},
{"BigFloat", (byte *)BigFloat},
{"LittleLong", (byte *)LittleLong},
{"BigLong", (byte *)BigLong},
{"LittleShort", (byte *)LittleShort},
{"BigShort", (byte *)BigShort},
{"COM_DefaultExtension", (byte *)COM_DefaultExtension},
{"COM_FilePath", (byte *)COM_FilePath},
{"COM_FileBase", (byte *)COM_FileBase},
{"COM_FileExtension", (byte *)COM_FileExtension},
{"COM_StripExtension", (byte *)COM_StripExtension},
{"COM_SkipPath", (byte *)COM_SkipPath},
{"Q_log2", (byte *)Q_log2},
{"VectorScale", (byte *)VectorScale},
{"VectorInverse", (byte *)VectorInverse},
{"VectorLength", (byte *)VectorLength},
{"CrossProduct", (byte *)CrossProduct},
{"_VectorCopy", (byte *)_VectorCopy},
{"_VectorAdd", (byte *)_VectorAdd},
{"_VectorSubtract", (byte *)_VectorSubtract},
{"_DotProduct", (byte *)_DotProduct},
{"VectorMA", (byte *)VectorMA},
{"VectorNormalize2", (byte *)VectorNormalize2},
{"VectorNormalize", (byte *)VectorNormalize},
{"VectorCompare", (byte *)VectorCompare},
{"AddPointToBounds", (byte *)AddPointToBounds},
{"ClearBounds", (byte *)ClearBounds},
{"BoxOnPlaneSide2", (byte *)BoxOnPlaneSide2},
{"anglemod", (byte *)anglemod},
{"LerpAngle", (byte *)LerpAngle},
{"Q_fabs", (byte *)Q_fabs},
{"R_ConcatTransforms", (byte *)R_ConcatTransforms},
{"R_ConcatRotations", (byte *)R_ConcatRotations},
{"PerpendicularVector", (byte *)PerpendicularVector},
{"ProjectPointOnPlane", (byte *)ProjectPointOnPlane},
{"AngleVectors", (byte *)AngleVectors},
{"RotatePointAroundVector", (byte *)RotatePointAroundVector},
{"Weapon_Heatbeam", (byte *)Weapon_Heatbeam},
{"Heatbeam_Fire", (byte *)Heatbeam_Fire},
{"Weapon_ETF_Rifle", (byte *)Weapon_ETF_Rifle},
{"weapon_etf_rifle_fire", (byte *)weapon_etf_rifle_fire},
{"Weapon_Disintegrator", (byte *)Weapon_Disintegrator},
{"weapon_tracker_fire", (byte *)weapon_tracker_fire},
{"Weapon_ChainFist", (byte *)Weapon_ChainFist},
{"chainfist_smoke", (byte *)chainfist_smoke},
{"weapon_chainfist_fire", (byte *)weapon_chainfist_fire},
{"Weapon_BFG", (byte *)Weapon_BFG},
{"weapon_bfg_fire", (byte *)weapon_bfg_fire},
{"Weapon_Railgun", (byte *)Weapon_Railgun},
{"weapon_railgun_fire", (byte *)weapon_railgun_fire},
{"Weapon_SuperShotgun", (byte *)Weapon_SuperShotgun},
{"weapon_supershotgun_fire", (byte *)weapon_supershotgun_fire},
{"Weapon_Shotgun", (byte *)Weapon_Shotgun},
{"weapon_shotgun_fire", (byte *)weapon_shotgun_fire},
{"Weapon_Chaingun", (byte *)Weapon_Chaingun},
{"Chaingun_Fire", (byte *)Chaingun_Fire},
{"Weapon_Machinegun", (byte *)Weapon_Machinegun},
{"Machinegun_Fire", (byte *)Machinegun_Fire},
{"Weapon_HyperBlaster", (byte *)Weapon_HyperBlaster},
{"Weapon_HyperBlaster_Fire", (byte *)Weapon_HyperBlaster_Fire},
{"Weapon_Blaster", (byte *)Weapon_Blaster},
{"Weapon_Blaster_Fire", (byte *)Weapon_Blaster_Fire},
{"Blaster_Fire", (byte *)Blaster_Fire},
{"Weapon_RocketLauncher", (byte *)Weapon_RocketLauncher},
{"Weapon_RocketLauncher_Fire", (byte *)Weapon_RocketLauncher_Fire},
{"Weapon_ProxLauncher", (byte *)Weapon_ProxLauncher},
{"Weapon_GrenadeLauncher", (byte *)Weapon_GrenadeLauncher},
{"weapon_grenadelauncher_fire", (byte *)weapon_grenadelauncher_fire},
{"Weapon_Tesla", (byte *)Weapon_Tesla},
{"Weapon_Grenade", (byte *)Weapon_Grenade},
{"Throw_Generic", (byte *)Throw_Generic},
{"weapon_grenade_fire", (byte *)weapon_grenade_fire},
{"Weapon_Generic", (byte *)Weapon_Generic},
{"Drop_Weapon", (byte *)Drop_Weapon},
{"Use_Weapon", (byte *)Use_Weapon},
{"Think_Weapon", (byte *)Think_Weapon},
{"NoAmmoWeaponChange", (byte *)NoAmmoWeaponChange},
{"ChangeWeapon", (byte *)ChangeWeapon},
{"Pickup_Weapon", (byte *)Pickup_Weapon},
{"PlayerNoise", (byte *)PlayerNoise},
{"P_ProjectSource2", (byte *)P_ProjectSource2},
{"P_ProjectSource", (byte *)P_ProjectSource},
{"P_DamageModifier", (byte *)P_DamageModifier},
{"ClientEndServerFrame", (byte *)ClientEndServerFrame},
{"G_SetClientFrame", (byte *)G_SetClientFrame},
{"G_SetClientSound", (byte *)G_SetClientSound},
{"G_SetClientEvent", (byte *)G_SetClientEvent},
{"G_SetClientEffects", (byte *)G_SetClientEffects},
{"P_WorldEffects", (byte *)P_WorldEffects},
{"P_FallingDamage", (byte *)P_FallingDamage},
{"SV_CalcBlend", (byte *)SV_CalcBlend},
{"SV_AddBlend", (byte *)SV_AddBlend},
{"SV_CalcGunOffset", (byte *)SV_CalcGunOffset},
{"SV_CalcViewOffset", (byte *)SV_CalcViewOffset},
{"P_DamageFeedback", (byte *)P_DamageFeedback},
{"SV_CalcRoll", (byte *)SV_CalcRoll},
{"PlayerTrail_LastSpot", (byte *)PlayerTrail_LastSpot},
{"PlayerTrail_PickNext", (byte *)PlayerTrail_PickNext},
{"PlayerTrail_PickFirst", (byte *)PlayerTrail_PickFirst},
{"PlayerTrail_New", (byte *)PlayerTrail_New},
{"PlayerTrail_Add", (byte *)PlayerTrail_Add},
{"PlayerTrail_Init", (byte *)PlayerTrail_Init},
{"G_SetSpectatorStats", (byte *)G_SetSpectatorStats},
{"G_CheckChaseStats", (byte *)G_CheckChaseStats},
{"G_SetStats", (byte *)G_SetStats},
{"InventoryMessage", (byte *)InventoryMessage},
{"HelpComputerMessage", (byte *)HelpComputerMessage},
{"DeathmatchScoreboardMessage", (byte *)DeathmatchScoreboardMessage},
{"BeginIntermission", (byte *)BeginIntermission},
{"MoveClientToIntermission", (byte *)MoveClientToIntermission},
{"RemoveAttackingPainDaemons", (byte *)RemoveAttackingPainDaemons},
{"ClientBeginServerFrame", (byte *)ClientBeginServerFrame},
{"ClientThink", (byte *)ClientThink},
{"PrintPmove", (byte *)PrintPmove},
{"CheckBlock", (byte *)CheckBlock},
{"PM_trace", (byte *)PM_trace},
{"ClientDisconnect", (byte *)ClientDisconnect},
{"ClientConnect", (byte *)ClientConnect},
{"ClientUserinfoChanged", (byte *)ClientUserinfoChanged},
{"ClientBegin", (byte *)ClientBegin},
{"ClientBeginDeathmatch", (byte *)ClientBeginDeathmatch},
{"PutClientInServer", (byte *)PutClientInServer},
{"spectator_respawn", (byte *)spectator_respawn},
{"respawn", (byte *)respawn},
{"CopyToBodyQue", (byte *)CopyToBodyQue},
{"body_die", (byte *)body_die},
{"InitBodyQue", (byte *)InitBodyQue},
{"SelectSpawnPoint", (byte *)SelectSpawnPoint},
{"SelectCoopSpawnPoint", (byte *)SelectCoopSpawnPoint},
{"SelectLavaCoopSpawnPoint", (byte *)SelectLavaCoopSpawnPoint},
{"SelectDeathmatchSpawnPoint", (byte *)SelectDeathmatchSpawnPoint},
{"SelectFarthestDeathmatchSpawnPoint", (byte *)SelectFarthestDeathmatchSpawnPoint},
{"SelectRandomDeathmatchSpawnPoint", (byte *)SelectRandomDeathmatchSpawnPoint},
{"PlayersRangeFromSpot", (byte *)PlayersRangeFromSpot},
{"FetchClientEntData", (byte *)FetchClientEntData},
{"SaveClientData", (byte *)SaveClientData},
{"InitClientResp", (byte *)InitClientResp},
{"InitClientPersistant", (byte *)InitClientPersistant},
{"player_die", (byte *)player_die},
{"LookAtKiller", (byte *)LookAtKiller},
{"TossClientWeapon", (byte *)TossClientWeapon},
{"ClientObituary", (byte *)ClientObituary},
{"IsNeutral", (byte *)IsNeutral},
{"IsFemale", (byte *)IsFemale},
{"player_pain", (byte *)player_pain},
{"SP_info_player_intermission", (byte *)SP_info_player_intermission},
{"SP_info_player_coop_lava", (byte *)SP_info_player_coop_lava},
{"SP_info_player_coop", (byte *)SP_info_player_coop},
{"SP_info_player_deathmatch", (byte *)SP_info_player_deathmatch},
{"SP_info_player_start", (byte *)SP_info_player_start},
{"ThrowArm2", (byte *)ThrowArm2},
{"ThrowArm1", (byte *)ThrowArm1},
{"WidowExplosionLeg", (byte *)WidowExplosionLeg},
{"WidowExplosion7", (byte *)WidowExplosion7},
{"WidowExplosion6", (byte *)WidowExplosion6},
{"WidowExplosion5", (byte *)WidowExplosion5},
{"WidowExplosion4", (byte *)WidowExplosion4},
{"WidowExplosion3", (byte *)WidowExplosion3},
{"WidowExplosion2", (byte *)WidowExplosion2},
{"WidowExplosion1", (byte *)WidowExplosion1},
{"WidowExplode", (byte *)WidowExplode},
{"ThrowMoreStuff", (byte *)ThrowMoreStuff},
{"ThrowSmallStuff", (byte *)ThrowSmallStuff},
{"ThrowWidowGibReal", (byte *)ThrowWidowGibReal},
{"ThrowWidowGibSized", (byte *)ThrowWidowGibSized},
{"ThrowWidowGibLoc", (byte *)ThrowWidowGibLoc},
{"ThrowWidowGib", (byte *)ThrowWidowGib},
{"widow_gib_touch", (byte *)widow_gib_touch},
{"WidowVelocityForDamage", (byte *)WidowVelocityForDamage},
{"SP_monster_widow2", (byte *)SP_monster_widow2},
{"Widow2Precache", (byte *)Widow2Precache},
{"Widow2_CheckAttack", (byte *)Widow2_CheckAttack},
{"widow2_die", (byte *)widow2_die},
{"KillChildren", (byte *)KillChildren},
{"widow2_dead", (byte *)widow2_dead},
{"widow2_pain", (byte *)widow2_pain},
{"widow2_reattack_beam", (byte *)widow2_reattack_beam},
{"widow2_attack_beam", (byte *)widow2_attack_beam},
{"widow2_attack", (byte *)widow2_attack},
{"widow2_melee", (byte *)widow2_melee},
{"widow2_walk", (byte *)widow2_walk},
{"widow2_run", (byte *)widow2_run},
{"widow2_stand", (byte *)widow2_stand},
{"widow2_finaldeath", (byte *)widow2_finaldeath},
{"widow2_keep_searching", (byte *)widow2_keep_searching},
{"widow2_start_searching", (byte *)widow2_start_searching},
{"Widow2Toss", (byte *)Widow2Toss},
{"Widow2Crunch", (byte *)Widow2Crunch},
{"Widow2TonguePull", (byte *)Widow2TonguePull},
{"Widow2Tongue", (byte *)Widow2Tongue},
{"widow2_tongue_attack_ok", (byte *)widow2_tongue_attack_ok},
{"Widow2StartSweep", (byte *)Widow2StartSweep},
{"Widow2BeamTargetRemove", (byte *)Widow2BeamTargetRemove},
{"Widow2SaveBeamTarget", (byte *)Widow2SaveBeamTarget},
{"widow2_disrupt_reattack", (byte *)widow2_disrupt_reattack},
{"Widow2SaveDisruptLoc", (byte *)Widow2SaveDisruptLoc},
{"WidowDisrupt", (byte *)WidowDisrupt},
{"widow2_ready_spawn", (byte *)widow2_ready_spawn},
{"widow2_spawn_check", (byte *)widow2_spawn_check},
{"Widow2Spawn", (byte *)Widow2Spawn},
{"Widow2Beam", (byte *)Widow2Beam},
{"widow2_search", (byte *)widow2_search},
{"pauseme", (byte *)pauseme},
{"SP_monster_widow", (byte *)SP_monster_widow},
{"WidowPrecache", (byte *)WidowPrecache},
{"WidowCalcSlots", (byte *)WidowCalcSlots},
{"widow_blocked", (byte *)widow_blocked},
{"Widow_CheckAttack", (byte *)Widow_CheckAttack},
{"WidowPowerups", (byte *)WidowPowerups},
{"WidowRespondPowerup", (byte *)WidowRespondPowerup},
{"WidowPowerArmor", (byte *)WidowPowerArmor},
{"WidowPent", (byte *)WidowPent},
{"WidowDouble", (byte *)WidowDouble},
{"WidowGoinQuad", (byte *)WidowGoinQuad},
{"widow_melee", (byte *)widow_melee},
{"widow_die", (byte *)widow_die},
{"widow_dead", (byte *)widow_dead},
{"widow_pain", (byte *)widow_pain},
{"widow_reattack_blaster", (byte *)widow_reattack_blaster},
{"widow_attack_blaster", (byte *)widow_attack_blaster},
{"widow_attack", (byte *)widow_attack},
{"widow_walk", (byte *)widow_walk},
{"widow_run", (byte *)widow_run},
{"widow_stand", (byte *)widow_stand},
{"widow_attack_kick", (byte *)widow_attack_kick},
{"spawn_out_do", (byte *)spawn_out_do},
{"spawn_out_start", (byte *)spawn_out_start},
{"widow_done_spawn", (byte *)widow_done_spawn},
{"widow_start_spawn", (byte *)widow_start_spawn},
{"widow_attack_rail", (byte *)widow_attack_rail},
{"widow_rail_done", (byte *)widow_rail_done},
{"widow_start_rail", (byte *)widow_start_rail},
{"WidowSaveLoc", (byte *)WidowSaveLoc},
{"WidowRail", (byte *)WidowRail},
{"widow_start_run_12", (byte *)widow_start_run_12},
{"widow_start_run_10", (byte *)widow_start_run_10},
{"widow_start_run_5", (byte *)widow_start_run_5},
{"widow_stepshoot", (byte *)widow_stepshoot},
{"widow_step", (byte *)widow_step},
{"widow_ready_spawn", (byte *)widow_ready_spawn},
{"widow_spawn_check", (byte *)widow_spawn_check},
{"WidowSpawn", (byte *)WidowSpawn},
{"WidowBlaster", (byte *)WidowBlaster},
{"WidowTorso", (byte *)WidowTorso},
{"target_angle", (byte *)target_angle},
{"widow_sight", (byte *)widow_sight},
{"widow_search", (byte *)widow_search},
{"showme", (byte *)showme},
{"SP_monster_turret", (byte *)SP_monster_turret},
{"turret_checkattack", (byte *)turret_checkattack},
{"turret_activate", (byte *)turret_activate},
{"turret_wake", (byte *)turret_wake},
{"turret_wall_spawn", (byte *)turret_wall_spawn},
{"turret_die", (byte *)turret_die},
{"turret_pain", (byte *)turret_pain},
{"turret_attack", (byte *)turret_attack},
{"TurretFireBlind", (byte *)TurretFireBlind},
{"TurretFire", (byte *)TurretFire},
{"turret_run", (byte *)turret_run},
{"turret_walk", (byte *)turret_walk},
{"turret_ready_gun", (byte *)turret_ready_gun},
{"turret_stand", (byte *)turret_stand},
{"turret_search", (byte *)turret_search},
{"turret_sight", (byte *)turret_sight},
{"TurretAim", (byte *)TurretAim},
{"SP_monster_tank", (byte *)SP_monster_tank},
{"tank_die", (byte *)tank_die},
{"tank_dead", (byte *)tank_dead},
{"tank_attack", (byte *)tank_attack},
{"tank_doattack_rocket", (byte *)tank_doattack_rocket},
{"tank_refire_rocket", (byte *)tank_refire_rocket},
{"tank_poststrike", (byte *)tank_poststrike},
{"tank_reattack_blaster", (byte *)tank_reattack_blaster},
{"TankMachineGun", (byte *)TankMachineGun},
{"TankRocket", (byte *)TankRocket},
{"TankStrike", (byte *)TankStrike},
{"TankBlaster", (byte *)TankBlaster},
{"tank_pain", (byte *)tank_pain},
{"tank_run", (byte *)tank_run},
{"tank_walk", (byte *)tank_walk},
{"tank_stand", (byte *)tank_stand},
{"tank_idle", (byte *)tank_idle},
{"tank_windup", (byte *)tank_windup},
{"tank_thud", (byte *)tank_thud},
{"tank_footstep", (byte *)tank_footstep},
{"tank_sight", (byte *)tank_sight},
{"tank_blocked", (byte *)tank_blocked},
{"SP_monster_supertank", (byte *)SP_monster_supertank},
{"supertank_blocked", (byte *)supertank_blocked},
{"supertank_die", (byte *)supertank_die},
{"BossExplode", (byte *)BossExplode},
{"supertank_dead", (byte *)supertank_dead},
{"supertank_attack", (byte *)supertank_attack},
{"supertankMachineGun", (byte *)supertankMachineGun},
{"supertankRocket", (byte *)supertankRocket},
{"supertank_pain", (byte *)supertank_pain},
{"supertank_reattack1", (byte *)supertank_reattack1},
{"supertank_run", (byte *)supertank_run},
{"supertank_walk", (byte *)supertank_walk},
{"supertank_forward", (byte *)supertank_forward},
{"supertank_stand", (byte *)supertank_stand},
{"supertank_search", (byte *)supertank_search},
{"TreadSound", (byte *)TreadSound},
{"SP_monster_stalker", (byte *)SP_monster_stalker},
{"stalker_die", (byte *)stalker_die},
{"stalker_dead", (byte *)stalker_dead},
{"stalker_blocked", (byte *)stalker_blocked},
{"stalker_jump", (byte *)stalker_jump},
{"stalker_jump_wait_land", (byte *)stalker_jump_wait_land},
{"stalker_jump_up", (byte *)stalker_jump_up},
{"stalker_jump_down", (byte *)stalker_jump_down},
{"stalker_dodge", (byte *)stalker_dodge},
{"stalker_dodge_jump", (byte *)stalker_dodge_jump},
{"stalker_jump_straightup", (byte *)stalker_jump_straightup},
{"stalker_do_pounce", (byte *)stalker_do_pounce},
{"stalker_check_lz", (byte *)stalker_check_lz},
{"calcJumpAngle", (byte *)calcJumpAngle},
{"stalker_attack_melee", (byte *)stalker_attack_melee},
{"stalker_swing_attack", (byte *)stalker_swing_attack},
{"stalker_attack_ranged", (byte *)stalker_attack_ranged},
{"stalker_shoot_attack2", (byte *)stalker_shoot_attack2},
{"stalker_shoot_attack", (byte *)stalker_shoot_attack},
{"stalker_pain", (byte *)stalker_pain},
{"stalker_false_death_start", (byte *)stalker_false_death_start},
{"stalker_false_death", (byte *)stalker_false_death},
{"stalker_heal", (byte *)stalker_heal},
{"stalker_reactivate", (byte *)stalker_reactivate},
{"stalker_walk", (byte *)stalker_walk},
{"stalker_run", (byte *)stalker_run},
{"stalker_stand", (byte *)stalker_stand},
{"stalker_idle", (byte *)stalker_idle},
{"stalker_idle_noise", (byte *)stalker_idle_noise},
{"stalker_sight", (byte *)stalker_sight},
{"stalker_ok_to_transition", (byte *)stalker_ok_to_transition},
{"SP_monster_soldier_ss", (byte *)SP_monster_soldier_ss},
{"SP_monster_soldier", (byte *)SP_monster_soldier},
{"SP_monster_soldier_light", (byte *)SP_monster_soldier_light},
{"SP_monster_soldier_x", (byte *)SP_monster_soldier_x},
{"soldier_footstep", (byte *)soldier_footstep},
{"soldier_blind", (byte *)soldier_blind},
{"soldier_duck", (byte *)soldier_duck},
{"soldier_sidestep", (byte *)soldier_sidestep},
{"soldier_die", (byte *)soldier_die},
{"soldier_dead2", (byte *)soldier_dead2},
{"soldier_dead", (byte *)soldier_dead},
{"soldier_fire7", (byte *)soldier_fire7},
{"soldier_fire6", (byte *)soldier_fire6},
{"soldier_blocked", (byte *)soldier_blocked},
{"soldier_sight", (byte *)soldier_sight},
{"soldier_attack", (byte *)soldier_attack},
{"soldier_attack6_refire", (byte *)soldier_attack6_refire},
{"soldier_fire8", (byte *)soldier_fire8},
{"soldier_fire4", (byte *)soldier_fire4},
{"soldier_attack3_refire", (byte *)soldier_attack3_refire},
{"soldier_fire3", (byte *)soldier_fire3},
{"soldier_attack2_refire2", (byte *)soldier_attack2_refire2},
{"soldier_attack2_refire1", (byte *)soldier_attack2_refire1},
{"soldier_fire2", (byte *)soldier_fire2},
{"soldier_attack1_refire2", (byte *)soldier_attack1_refire2},
{"soldier_attack1_refire1", (byte *)soldier_attack1_refire1},
{"soldier_fire1", (byte *)soldier_fire1},
{"soldier_fire", (byte *)soldier_fire},
{"soldier_pain", (byte *)soldier_pain},
{"soldier_run", (byte *)soldier_run},
{"soldier_fire_run", (byte *)soldier_fire_run},
{"soldier_walk", (byte *)soldier_walk},
{"soldier_walk1_random", (byte *)soldier_walk1_random},
{"soldier_stand", (byte *)soldier_stand},
{"soldier_cock", (byte *)soldier_cock},
{"soldier_idle", (byte *)soldier_idle},
{"soldier_stop_charge", (byte *)soldier_stop_charge},
{"soldier_start_charge", (byte *)soldier_start_charge},
{"SP_monster_parasite", (byte *)SP_monster_parasite},
{"parasite_die", (byte *)parasite_die},
{"parasite_dead", (byte *)parasite_dead},
{"parasite_checkattack", (byte *)parasite_checkattack},
{"parasite_blocked", (byte *)parasite_blocked},
{"parasite_jump", (byte *)parasite_jump},
{"parasite_jump_wait_land", (byte *)parasite_jump_wait_land},
{"parasite_jump_up", (byte *)parasite_jump_up},
{"parasite_jump_down", (byte *)parasite_jump_down},
{"parasite_attack", (byte *)parasite_attack},
{"parasite_drain_attack", (byte *)parasite_drain_attack},
{"parasite_drain_attack_ok", (byte *)parasite_drain_attack_ok},
{"parasite_pain", (byte *)parasite_pain},
{"parasite_walk", (byte *)parasite_walk},
{"parasite_start_walk", (byte *)parasite_start_walk},
{"parasite_run", (byte *)parasite_run},
{"parasite_start_run", (byte *)parasite_start_run},
{"parasite_stand", (byte *)parasite_stand},
{"parasite_idle", (byte *)parasite_idle},
{"parasite_refidget", (byte *)parasite_refidget},
{"parasite_do_fidget", (byte *)parasite_do_fidget},
{"parasite_end_fidget", (byte *)parasite_end_fidget},
{"parasite_search", (byte *)parasite_search},
{"parasite_scratch", (byte *)parasite_scratch},
{"parasite_tap", (byte *)parasite_tap},
{"parasite_sight", (byte *)parasite_sight},
{"parasite_reel_in", (byte *)parasite_reel_in},
{"parasite_launch", (byte *)parasite_launch},
{"SP_monster_mutant", (byte *)SP_monster_mutant},
{"mutant_blocked", (byte *)mutant_blocked},
{"mutant_die", (byte *)mutant_die},
{"mutant_dead", (byte *)mutant_dead},
{"mutant_pain", (byte *)mutant_pain},
{"mutant_checkattack", (byte *)mutant_checkattack},
{"mutant_check_jump", (byte *)mutant_check_jump},
{"mutant_check_melee", (byte *)mutant_check_melee},
{"mutant_jump", (byte *)mutant_jump},
{"mutant_check_landing", (byte *)mutant_check_landing},
{"mutant_jump_takeoff", (byte *)mutant_jump_takeoff},
{"mutant_jump_touch", (byte *)mutant_jump_touch},
{"mutant_melee", (byte *)mutant_melee},
{"mutant_check_refire", (byte *)mutant_check_refire},
{"mutant_hit_right", (byte *)mutant_hit_right},
{"mutant_hit_left", (byte *)mutant_hit_left},
{"mutant_run", (byte *)mutant_run},
{"mutant_walk", (byte *)mutant_walk},
{"mutant_walk_loop", (byte *)mutant_walk_loop},
{"mutant_idle", (byte *)mutant_idle},
{"mutant_idle_loop", (byte *)mutant_idle_loop},
{"mutant_stand", (byte *)mutant_stand},
{"mutant_swing", (byte *)mutant_swing},
{"mutant_search", (byte *)mutant_search},
{"mutant_sight", (byte *)mutant_sight},
{"mutant_step", (byte *)mutant_step},
{"M_walkmove", (byte *)M_walkmove},
{"M_MoveToGoal", (byte *)M_MoveToGoal},
{"SV_CloseEnough", (byte *)SV_CloseEnough},
{"SV_NewChaseDir", (byte *)SV_NewChaseDir},
{"SV_FixCheckBottom", (byte *)SV_FixCheckBottom},
{"SV_StepDirection", (byte *)SV_StepDirection},
{"M_ChangeYaw", (byte *)M_ChangeYaw},
{"SV_movestep", (byte *)SV_movestep},
{"IsBadAhead", (byte *)IsBadAhead},
{"M_CheckBottom", (byte *)M_CheckBottom},
{"SP_monster_medic", (byte *)SP_monster_medic},
{"medic_blocked", (byte *)medic_blocked},
{"medic_sidestep", (byte *)medic_sidestep},
{"medic_duck", (byte *)medic_duck},
{"medic_footstep", (byte *)medic_footstep},
{"medic_checkattack", (byte *)medic_checkattack},
{"medic_attack", (byte *)medic_attack},
{"medic_finish_spawn", (byte *)medic_finish_spawn},
{"medic_spawngrows", (byte *)medic_spawngrows},
{"medic_determine_spawn", (byte *)medic_determine_spawn},
{"medic_start_spawn", (byte *)medic_start_spawn},
{"medic_hook_retract", (byte *)medic_hook_retract},
{"medic_cable_attack", (byte *)medic_cable_attack},
{"medic_hook_launch", (byte *)medic_hook_launch},
{"medic_continue", (byte *)medic_continue},
{"medic_die", (byte *)medic_die},
{"medic_dead", (byte *)medic_dead},
{"medic_fire_blaster", (byte *)medic_fire_blaster},
{"medic_pain", (byte *)medic_pain},
{"medic_run", (byte *)medic_run},
{"medic_walk", (byte *)medic_walk},
{"medic_stand", (byte *)medic_stand},
{"medic_sight", (byte *)medic_sight},
{"medic_search", (byte *)medic_search},
{"medic_idle", (byte *)medic_idle},
{"cleanupHeal", (byte *)cleanupHeal},
{"abortHeal", (byte *)abortHeal},
{"canReach", (byte *)canReach},
{"medic_FindDeadMonster", (byte *)medic_FindDeadMonster},
{"SP_misc_insane", (byte *)SP_misc_insane},
{"insane_footstep", (byte *)insane_footstep},
{"insane_die", (byte *)insane_die},
{"insane_dead", (byte *)insane_dead},
{"insane_stand", (byte *)insane_stand},
{"insane_checkup", (byte *)insane_checkup},
{"insane_checkdown", (byte *)insane_checkdown},
{"insane_onground", (byte *)insane_onground},
{"insane_pain", (byte *)insane_pain},
{"insane_run", (byte *)insane_run},
{"insane_walk", (byte *)insane_walk},
{"insane_cross", (byte *)insane_cross},
{"insane_scream", (byte *)insane_scream},
{"insane_moan", (byte *)insane_moan},
{"insane_shake", (byte *)insane_shake},
{"insane_fist", (byte *)insane_fist},
{"SP_monster_infantry", (byte *)SP_monster_infantry},
{"infantry_footstep", (byte *)infantry_footstep},
{"infantry_sidestep", (byte *)infantry_sidestep},
{"infantry_duck", (byte *)infantry_duck},
{"infantry_blocked", (byte *)infantry_blocked},
{"infantry_jump", (byte *)infantry_jump},
{"infantry_jump_wait_land", (byte *)infantry_jump_wait_land},
{"infantry_jump2_now", (byte *)infantry_jump2_now},
{"infantry_jump_now", (byte *)infantry_jump_now},
{"infantry_attack", (byte *)infantry_attack},
{"infantry_smack", (byte *)infantry_smack},
{"infantry_swing", (byte *)infantry_swing},
{"infantry_fire_prep", (byte *)infantry_fire_prep},
{"infantry_fire", (byte *)infantry_fire},
{"infantry_cock_gun", (byte *)infantry_cock_gun},
{"infantry_die", (byte *)infantry_die},
{"infantry_dead", (byte *)infantry_dead},
{"infantry_sight", (byte *)infantry_sight},
{"InfantryMachineGun", (byte *)InfantryMachineGun},
{"infantry_pain", (byte *)infantry_pain},
{"infantry_run", (byte *)infantry_run},
{"infantry_walk", (byte *)infantry_walk},
{"infantry_fidget", (byte *)infantry_fidget},
{"infantry_stand", (byte *)infantry_stand},
{"SP_monster_hover", (byte *)SP_monster_hover},
{"hover_blocked", (byte *)hover_blocked},
{"hover_die", (byte *)hover_die},
{"hover_dead", (byte *)hover_dead},
{"hover_deadthink", (byte *)hover_deadthink},
{"hover_pain", (byte *)hover_pain},
{"hover_attack", (byte *)hover_attack},
{"hover_start_attack", (byte *)hover_start_attack},
{"hover_walk", (byte *)hover_walk},
{"hover_run", (byte *)hover_run},
{"hover_stand", (byte *)hover_stand},
{"hover_fire_blaster", (byte *)hover_fire_blaster},
{"hover_reattack", (byte *)hover_reattack},
{"hover_search", (byte *)hover_search},
{"hover_sight", (byte *)hover_sight},
{"SP_monster_gunner", (byte *)SP_monster_gunner},
{"gunner_footstep", (byte *)gunner_footstep},
{"gunner_sidestep", (byte *)gunner_sidestep},
{"gunner_duck", (byte *)gunner_duck},
{"gunner_blocked", (byte *)gunner_blocked},
{"gunner_jump", (byte *)gunner_jump},
{"gunner_jump_wait_land", (byte *)gunner_jump_wait_land},
{"gunner_jump2_now", (byte *)gunner_jump2_now},
{"gunner_jump_now", (byte *)gunner_jump_now},
{"gunner_refire_chain", (byte *)gunner_refire_chain},
{"gunner_fire_chain", (byte *)gunner_fire_chain},
{"gunner_attack", (byte *)gunner_attack},
{"gunner_blind_check", (byte *)gunner_blind_check},
{"GunnerGrenade", (byte *)GunnerGrenade},
{"gunner_grenade_check", (byte *)gunner_grenade_check},
{"GunnerFire", (byte *)GunnerFire},
{"gunner_opengun", (byte *)gunner_opengun},
{"gunner_duck_down", (byte *)gunner_duck_down},
{"gunner_die", (byte *)gunner_die},
{"gunner_dead", (byte *)gunner_dead},
{"gunner_pain", (byte *)gunner_pain},
{"gunner_runandshoot", (byte *)gunner_runandshoot},
{"gunner_run", (byte *)gunner_run},
{"gunner_walk", (byte *)gunner_walk},
{"gunner_stand", (byte *)gunner_stand},
{"gunner_fidget", (byte *)gunner_fidget},
{"gunner_search", (byte *)gunner_search},
{"gunner_sight", (byte *)gunner_sight},
{"gunner_idlesound", (byte *)gunner_idlesound},
{"SP_monster_gladiator", (byte *)SP_monster_gladiator},
{"gladiator_footstep", (byte *)gladiator_footstep},
{"gladiator_blocked", (byte *)gladiator_blocked},
{"gladiator_die", (byte *)gladiator_die},
{"gladiator_dead", (byte *)gladiator_dead},
{"gladiator_pain", (byte *)gladiator_pain},
{"gladiator_attack", (byte *)gladiator_attack},
{"GladiatorGun", (byte *)GladiatorGun},
{"gladiator_melee", (byte *)gladiator_melee},
{"GaldiatorMelee", (byte *)GaldiatorMelee},
{"gladiator_run", (byte *)gladiator_run},
{"gladiator_walk", (byte *)gladiator_walk},
{"gladiator_stand", (byte *)gladiator_stand},
{"gladiator_cleaver_swing", (byte *)gladiator_cleaver_swing},
{"gladiator_search", (byte *)gladiator_search},
{"gladiator_sight", (byte *)gladiator_sight},
{"gladiator_idle", (byte *)gladiator_idle},
{"SP_monster_kamikaze", (byte *)SP_monster_kamikaze},
{"SP_monster_flyer", (byte *)SP_monster_flyer},
{"flyer_blocked", (byte *)flyer_blocked},
{"flyer_die", (byte *)flyer_die},
{"flyer_pain", (byte *)flyer_pain},
{"flyer_check_melee", (byte *)flyer_check_melee},
{"flyer_melee", (byte *)flyer_melee},
{"flyer_nextmove", (byte *)flyer_nextmove},
{"flyer_setstart", (byte *)flyer_setstart},
{"flyer_attack", (byte *)flyer_attack},
{"flyer_loop_melee", (byte *)flyer_loop_melee},
{"flyer_slash_right", (byte *)flyer_slash_right},
{"flyer_slash_left", (byte *)flyer_slash_left},
{"flyer_fireright", (byte *)flyer_fireright},
{"flyer_fireleft", (byte *)flyer_fireleft},
{"flyer_fire", (byte *)flyer_fire},
{"flyer_start", (byte *)flyer_start},
{"flyer_stop", (byte *)flyer_stop},
{"flyer_kamikaze_check", (byte *)flyer_kamikaze_check},
{"flyer_kamikaze", (byte *)flyer_kamikaze},
{"flyer_kamikaze_explode", (byte *)flyer_kamikaze_explode},
{"flyer_stand", (byte *)flyer_stand},
{"flyer_walk", (byte *)flyer_walk},
{"flyer_run", (byte *)flyer_run},
{"flyer_pop_blades", (byte *)flyer_pop_blades},
{"flyer_idle", (byte *)flyer_idle},
{"flyer_sight", (byte *)flyer_sight},
{"SP_monster_floater", (byte *)SP_monster_floater},
{"floater_blocked", (byte *)floater_blocked},
{"floater_die", (byte *)floater_die},
{"floater_dead", (byte *)floater_dead},
{"floater_pain", (byte *)floater_pain},
{"floater_melee", (byte *)floater_melee},
{"floater_attack", (byte *)floater_attack},
{"floater_zap", (byte *)floater_zap},
{"floater_wham", (byte *)floater_wham},
{"floater_walk", (byte *)floater_walk},
{"floater_run", (byte *)floater_run},
{"floater_stand", (byte *)floater_stand},
{"floater_fire_blaster", (byte *)floater_fire_blaster},
{"floater_idle", (byte *)floater_idle},
{"floater_sight", (byte *)floater_sight},
{"SP_monster_flipper", (byte *)SP_monster_flipper},
{"flipper_die", (byte *)flipper_die},
{"flipper_sight", (byte *)flipper_sight},
{"flipper_dead", (byte *)flipper_dead},
{"flipper_pain", (byte *)flipper_pain},
{"flipper_melee", (byte *)flipper_melee},
{"flipper_preattack", (byte *)flipper_preattack},
{"flipper_bite", (byte *)flipper_bite},
{"flipper_start_run", (byte *)flipper_start_run},
{"flipper_walk", (byte *)flipper_walk},
{"flipper_run", (byte *)flipper_run},
{"flipper_run_loop", (byte *)flipper_run_loop},
{"flipper_stand", (byte *)flipper_stand},
{"SP_monster_chick", (byte *)SP_monster_chick},
{"chick_footstep", (byte *)chick_footstep},
{"chick_sidestep", (byte *)chick_sidestep},
{"chick_duck", (byte *)chick_duck},
{"chick_blocked", (byte *)chick_blocked},
{"chick_sight", (byte *)chick_sight},
{"chick_attack", (byte *)chick_attack},
{"chick_melee", (byte *)chick_melee},
{"chick_slash", (byte *)chick_slash},
{"chick_reslash", (byte *)chick_reslash},
{"chick_attack1", (byte *)chick_attack1},
{"chick_rerocket", (byte *)chick_rerocket},
{"ChickReload", (byte *)ChickReload},
{"Chick_PreAttack1", (byte *)Chick_PreAttack1},
{"ChickRocket", (byte *)ChickRocket},
{"ChickSlash", (byte *)ChickSlash},
{"chick_die", (byte *)chick_die},
{"chick_dead", (byte *)chick_dead},
{"chick_pain", (byte *)chick_pain},
{"chick_run", (byte *)chick_run},
{"chick_walk", (byte *)chick_walk},
{"chick_stand", (byte *)chick_stand},
{"chick_fidget", (byte *)chick_fidget},
{"ChickMoan", (byte *)ChickMoan},
{"SP_monster_carrier", (byte *)SP_monster_carrier},
{"CarrierPrecache", (byte *)CarrierPrecache},
{"Carrier_CheckAttack", (byte *)Carrier_CheckAttack},
{"carrier_die", (byte *)carrier_die},
{"carrier_dead", (byte *)carrier_dead},
{"carrier_pain", (byte *)carrier_pain},
{"carrier_reattack_gren", (byte *)carrier_reattack_gren},
{"carrier_attack_gren", (byte *)carrier_attack_gren},
{"carrier_reattack_mg", (byte *)carrier_reattack_mg},
{"carrier_attack_mg", (byte *)carrier_attack_mg},
{"carrier_attack", (byte *)carrier_attack},
{"CarrierMachineGunHold", (byte *)CarrierMachineGunHold},
{"carrier_walk", (byte *)carrier_walk},
{"carrier_run", (byte *)carrier_run},
{"carrier_stand", (byte *)carrier_stand},
{"CarrierSaveLoc", (byte *)CarrierSaveLoc},
{"CarrierRail", (byte *)CarrierRail},
{"carrier_start_spawn", (byte *)carrier_start_spawn},
{"carrier_ready_spawn", (byte *)carrier_ready_spawn},
{"carrier_spawn_check", (byte *)carrier_spawn_check},
{"carrier_prep_spawn", (byte *)carrier_prep_spawn},
{"CarrierSpawn", (byte *)CarrierSpawn},
{"CarrierMachineGun", (byte *)CarrierMachineGun},
{"carrier_firebullet_left", (byte *)carrier_firebullet_left},
{"carrier_firebullet_right", (byte *)carrier_firebullet_right},
{"CarrierRocket", (byte *)CarrierRocket},
{"CarrierPredictiveRocket", (byte *)CarrierPredictiveRocket},
{"CarrierGrenade", (byte *)CarrierGrenade},
{"CarrierCoopCheck", (byte *)CarrierCoopCheck},
{"carrier_sight", (byte *)carrier_sight},
{"SP_monster_brain", (byte *)SP_monster_brain},
{"brain_footstep", (byte *)brain_footstep},
{"brain_duck", (byte *)brain_duck},
{"brain_die", (byte *)brain_die},
{"brain_dead", (byte *)brain_dead},
{"brain_pain", (byte *)brain_pain},
{"brain_run", (byte *)brain_run},
{"brain_melee", (byte *)brain_melee},
{"brain_chest_closed", (byte *)brain_chest_closed},
{"brain_tentacle_attack", (byte *)brain_tentacle_attack},
{"brain_chest_open", (byte *)brain_chest_open},
{"brain_hit_left", (byte *)brain_hit_left},
{"brain_swing_left", (byte *)brain_swing_left},
{"brain_hit_right", (byte *)brain_hit_right},
{"brain_swing_right", (byte *)brain_swing_right},
{"brain_walk", (byte *)brain_walk},
{"brain_idle", (byte *)brain_idle},
{"brain_stand", (byte *)brain_stand},
{"brain_search", (byte *)brain_search},
{"brain_sight", (byte *)brain_sight},
{"MakronToss", (byte *)MakronToss},
{"MakronSpawn", (byte *)MakronSpawn},
{"SP_monster_makron", (byte *)SP_monster_makron},
{"MakronPrecache", (byte *)MakronPrecache},
{"Makron_CheckAttack", (byte *)Makron_CheckAttack},
{"makron_die", (byte *)makron_die},
{"makron_torso_die", (byte *)makron_torso_die},
{"makron_dead", (byte *)makron_dead},
{"makron_torso", (byte *)makron_torso},
{"makron_torso_think", (byte *)makron_torso_think},
{"makron_attack", (byte *)makron_attack},
{"makron_sight", (byte *)makron_sight},
{"makron_pain", (byte *)makron_pain},
{"MakronHyperblaster", (byte *)MakronHyperblaster},
{"MakronRailgun", (byte *)MakronRailgun},
{"MakronSaveloc", (byte *)MakronSaveloc},
{"makronBFG", (byte *)makronBFG},
{"makron_run", (byte *)makron_run},
{"makron_walk", (byte *)makron_walk},
{"makron_prerailgun", (byte *)makron_prerailgun},
{"makron_brainsplorch", (byte *)makron_brainsplorch},
{"makron_step_right", (byte *)makron_step_right},
{"makron_step_left", (byte *)makron_step_left},
{"makron_popup", (byte *)makron_popup},
{"makron_hit", (byte *)makron_hit},
{"makron_stand", (byte *)makron_stand},
{"makron_taunt", (byte *)makron_taunt},
{"SP_monster_jorg", (byte *)SP_monster_jorg},
{"Jorg_CheckAttack", (byte *)Jorg_CheckAttack},
{"jorg_die", (byte *)jorg_die},
{"jorg_dead", (byte *)jorg_dead},
{"jorg_attack", (byte *)jorg_attack},
{"jorg_firebullet", (byte *)jorg_firebullet},
{"jorg_firebullet_left", (byte *)jorg_firebullet_left},
{"jorg_firebullet_right", (byte *)jorg_firebullet_right},
{"jorgBFG", (byte *)jorgBFG},
{"jorg_pain", (byte *)jorg_pain},
{"jorg_attack1", (byte *)jorg_attack1},
{"jorg_reattack1", (byte *)jorg_reattack1},
{"jorg_run", (byte *)jorg_run},
{"jorg_walk", (byte *)jorg_walk},
{"jorg_stand", (byte *)jorg_stand},
{"jorg_step_right", (byte *)jorg_step_right},
{"jorg_step_left", (byte *)jorg_step_left},
{"jorg_death_hit", (byte *)jorg_death_hit},
{"jorg_idle", (byte *)jorg_idle},
{"jorg_search", (byte *)jorg_search},
{"SP_monster_boss3_stand", (byte *)SP_monster_boss3_stand},
{"Think_Boss3Stand", (byte *)Think_Boss3Stand},
{"Use_Boss3", (byte *)Use_Boss3},
{"SP_monster_boss2", (byte *)SP_monster_boss2},
{"Boss2_CheckAttack", (byte *)Boss2_CheckAttack},
{"boss2_die", (byte *)boss2_die},
{"boss2_dead", (byte *)boss2_dead},
{"boss2_pain", (byte *)boss2_pain},
{"boss2_reattack_mg", (byte *)boss2_reattack_mg},
{"boss2_attack_mg", (byte *)boss2_attack_mg},
{"boss2_attack", (byte *)boss2_attack},
{"boss2_walk", (byte *)boss2_walk},
{"boss2_run", (byte *)boss2_run},
{"boss2_stand", (byte *)boss2_stand},
{"Boss2MachineGun", (byte *)Boss2MachineGun},
{"boss2_firebullet_left", (byte *)boss2_firebullet_left},
{"boss2_firebullet_right", (byte *)boss2_firebullet_right},
{"Boss2Rocket", (byte *)Boss2Rocket},
{"boss2_search", (byte *)boss2_search},
{"SP_monster_berserk", (byte *)SP_monster_berserk},
{"berserk_footstep", (byte *)berserk_footstep},
{"berserk_sidestep", (byte *)berserk_sidestep},
{"berserk_blocked", (byte *)berserk_blocked},
{"berserk_die", (byte *)berserk_die},
{"berserk_dead", (byte *)berserk_dead},
{"berserk_pain", (byte *)berserk_pain},
{"berserk_melee", (byte *)berserk_melee},
{"berserk_strike", (byte *)berserk_strike},
{"berserk_attack_club", (byte *)berserk_attack_club},
{"berserk_swing", (byte *)berserk_swing},
{"berserk_attack_spike", (byte *)berserk_attack_spike},
{"berserk_run", (byte *)berserk_run},
{"berserk_walk", (byte *)berserk_walk},
{"berserk_fidget", (byte *)berserk_fidget},
{"berserk_stand", (byte *)berserk_stand},
{"berserk_search", (byte *)berserk_search},
{"berserk_sight", (byte *)berserk_sight},
{"fire_bfg", (byte *)fire_bfg},
{"bfg_think", (byte *)bfg_think},
{"bfg_touch", (byte *)bfg_touch},
{"bfg_explode", (byte *)bfg_explode},
{"fire_rail", (byte *)fire_rail},
{"fire_rocket", (byte *)fire_rocket},
{"rocket_touch", (byte *)rocket_touch},
{"fire_grenade2", (byte *)fire_grenade2},
{"fire_grenade", (byte *)fire_grenade},
{"Grenade_Touch", (byte *)Grenade_Touch},
{"Grenade_Explode", (byte *)Grenade_Explode},
{"fire_blaster", (byte *)fire_blaster},
{"blaster_touch", (byte *)blaster_touch},
{"fire_shotgun", (byte *)fire_shotgun},
{"fire_bullet", (byte *)fire_bullet},
{"fire_lead", (byte *)fire_lead},
{"fire_hit", (byte *)fire_hit},
{"check_dodge", (byte *)check_dodge},
{"KillBox", (byte *)KillBox},
{"G_TouchSolids", (byte *)G_TouchSolids},
{"G_TouchTriggers", (byte *)G_TouchTriggers},
{"G_FreeEdict", (byte *)G_FreeEdict},
{"G_Spawn", (byte *)G_Spawn},
{"G_InitEdict", (byte *)G_InitEdict},
{"G_CopyString", (byte *)G_CopyString},
{"vectoangles2", (byte *)vectoangles2},
{"vectoangles", (byte *)vectoangles},
{"vectoyaw2", (byte *)vectoyaw2},
{"vectoyaw", (byte *)vectoyaw},
{"G_SetMovedir", (byte *)G_SetMovedir},
{"vtos", (byte *)vtos},
{"tv", (byte *)tv},
{"G_UseTargets", (byte *)G_UseTargets},
{"Think_Delay", (byte *)Think_Delay},
{"G_PickTarget", (byte *)G_PickTarget},
{"findradius2", (byte *)findradius2},
{"findradius", (byte *)findradius},
{"G_Find", (byte *)G_Find},
{"G_ProjectSource2", (byte *)G_ProjectSource2},
{"G_ProjectSource", (byte *)G_ProjectSource},
{"SP_turret_invisible_brain", (byte *)SP_turret_invisible_brain},
{"turret_brain_activate", (byte *)turret_brain_activate},
{"turret_brain_deactivate", (byte *)turret_brain_deactivate},
{"turret_brain_link", (byte *)turret_brain_link},
{"turret_brain_think", (byte *)turret_brain_think},
{"SP_turret_driver", (byte *)SP_turret_driver},
{"turret_driver_link", (byte *)turret_driver_link},
{"turret_driver_think", (byte *)turret_driver_think},
{"turret_driver_die", (byte *)turret_driver_die},
{"SP_turret_base", (byte *)SP_turret_base},
{"SP_turret_breach", (byte *)SP_turret_breach},
{"turret_breach_finish_init", (byte *)turret_breach_finish_init},
{"turret_breach_think", (byte *)turret_breach_think},
{"turret_breach_fire", (byte *)turret_breach_fire},
{"turret_blocked", (byte *)turret_blocked},
{"SnapToEights", (byte *)SnapToEights},
{"AnglesNormalize", (byte *)AnglesNormalize},
{"SP_trigger_monsterjump", (byte *)SP_trigger_monsterjump},
{"trigger_monsterjump_touch", (byte *)trigger_monsterjump_touch},
{"SP_trigger_gravity", (byte *)SP_trigger_gravity},
{"trigger_gravity_touch", (byte *)trigger_gravity_touch},
{"trigger_gravity_use", (byte *)trigger_gravity_use},
{"SP_trigger_hurt", (byte *)SP_trigger_hurt},
{"hurt_touch", (byte *)hurt_touch},
{"hurt_use", (byte *)hurt_use},
{"SP_trigger_push", (byte *)SP_trigger_push},
{"trigger_push_use", (byte *)trigger_push_use},
{"trigger_push_touch", (byte *)trigger_push_touch},
{"SP_trigger_always", (byte *)SP_trigger_always},
{"SP_trigger_counter", (byte *)SP_trigger_counter},
{"trigger_counter_use", (byte *)trigger_counter_use},
{"SP_trigger_key", (byte *)SP_trigger_key},
{"trigger_key_use", (byte *)trigger_key_use},
{"SP_trigger_relay", (byte *)SP_trigger_relay},
{"trigger_relay_use", (byte *)trigger_relay_use},
{"SP_trigger_once", (byte *)SP_trigger_once},
{"SP_trigger_multiple", (byte *)SP_trigger_multiple},
{"trigger_enable", (byte *)trigger_enable},
{"Touch_Multi", (byte *)Touch_Multi},
{"Use_Multi", (byte *)Use_Multi},
{"multi_trigger", (byte *)multi_trigger},
{"multi_wait", (byte *)multi_wait},
{"InitTrigger", (byte *)InitTrigger},
{"SP_target_earthquake", (byte *)SP_target_earthquake},
{"target_earthquake_use", (byte *)target_earthquake_use},
{"target_earthquake_think", (byte *)target_earthquake_think},
{"SP_target_lightramp", (byte *)SP_target_lightramp},
{"target_lightramp_use", (byte *)target_lightramp_use},
{"target_lightramp_think", (byte *)target_lightramp_think},
{"SP_target_laser", (byte *)SP_target_laser},
{"target_laser_start", (byte *)target_laser_start},
{"target_laser_use", (byte *)target_laser_use},
{"target_laser_off", (byte *)target_laser_off},
{"target_laser_on", (byte *)target_laser_on},
{"target_laser_think", (byte *)target_laser_think},
{"SP_target_crosslevel_target", (byte *)SP_target_crosslevel_target},
{"target_crosslevel_target_think", (byte *)target_crosslevel_target_think},
{"SP_target_crosslevel_trigger", (byte *)SP_target_crosslevel_trigger},
{"trigger_crosslevel_trigger_use", (byte *)trigger_crosslevel_trigger_use},
{"SP_target_blaster", (byte *)SP_target_blaster},
{"use_target_blaster", (byte *)use_target_blaster},
{"SP_target_spawner", (byte *)SP_target_spawner},
{"use_target_spawner", (byte *)use_target_spawner},
{"SP_target_splash", (byte *)SP_target_splash},
{"use_target_splash", (byte *)use_target_splash},
{"SP_target_changelevel", (byte *)SP_target_changelevel},
{"use_target_changelevel", (byte *)use_target_changelevel},
{"SP_target_explosion", (byte *)SP_target_explosion},
{"use_target_explosion", (byte *)use_target_explosion},
{"target_explosion_explode", (byte *)target_explosion_explode},
{"SP_target_goal", (byte *)SP_target_goal},
{"use_target_goal", (byte *)use_target_goal},
{"SP_target_secret", (byte *)SP_target_secret},
{"use_target_secret", (byte *)use_target_secret},
{"SP_target_help", (byte *)SP_target_help},
{"Use_Target_Help", (byte *)Use_Target_Help},
{"Target_Help_Think", (byte *)Target_Help_Think},
{"SP_target_speaker", (byte *)SP_target_speaker},
{"Use_Target_Speaker", (byte *)Use_Target_Speaker},
{"SP_target_temp_entity", (byte *)SP_target_temp_entity},
{"Use_Target_Tent", (byte *)Use_Target_Tent},
{"ServerCommand", (byte *)ServerCommand},
{"SVCmd_WriteIP_f", (byte *)SVCmd_WriteIP_f},
{"SVCmd_ListIP_f", (byte *)SVCmd_ListIP_f},
{"SVCmd_RemoveIP_f", (byte *)SVCmd_RemoveIP_f},
{"SVCmd_AddIP_f", (byte *)SVCmd_AddIP_f},
{"SV_FilterPacket", (byte *)SV_FilterPacket},
{"Svcmd_Test_f", (byte *)Svcmd_Test_f},
{"Vengeance_Launch", (byte *)Vengeance_Launch},
{"Hunter_Launch", (byte *)Hunter_Launch},
{"Defender_Launch", (byte *)Defender_Launch},
{"Own_Sphere", (byte *)Own_Sphere},
{"Sphere_Spawn", (byte *)Sphere_Spawn},
{"vengeance_think", (byte *)vengeance_think},
{"hunter_think", (byte *)hunter_think},
{"defender_think", (byte *)defender_think},
{"vengeance_pain", (byte *)vengeance_pain},
{"defender_pain", (byte *)defender_pain},
{"hunter_pain", (byte *)hunter_pain},
{"body_gib", (byte *)body_gib},
{"defender_shoot", (byte *)defender_shoot},
{"hunter_touch", (byte *)hunter_touch},
{"vengeance_touch", (byte *)vengeance_touch},
{"sphere_touch", (byte *)sphere_touch},
{"sphere_fire", (byte *)sphere_fire},
{"sphere_chase", (byte *)sphere_chase},
{"sphere_fly", (byte *)sphere_fly},
{"sphere_if_idle_die", (byte *)sphere_if_idle_die},
{"sphere_explode", (byte *)sphere_explode},
{"sphere_think_explode", (byte *)sphere_think_explode},
{"Widowlegs_Spawn", (byte *)Widowlegs_Spawn},
{"widowlegs_think", (byte *)widowlegs_think},
{"SpawnGrow_Spawn", (byte *)SpawnGrow_Spawn},
{"spawngrow_think", (byte *)spawngrow_think},
{"DetermineBBox", (byte *)DetermineBBox},
{"CheckGroundSpawnPoint", (byte *)CheckGroundSpawnPoint},
{"CheckSpawnPoint", (byte *)CheckSpawnPoint},
{"FindSpawnPoint", (byte *)FindSpawnPoint},
{"CreateGroundMonster", (byte *)CreateGroundMonster},
{"CreateFlyMonster", (byte *)CreateFlyMonster},
{"CreateMonster", (byte *)CreateMonster},
{"SP_worldspawn", (byte *)SP_worldspawn},
{"SpawnEntities", (byte *)SpawnEntities},
{"ED_NewString", (byte *)ED_NewString},
{"ED_CallSpawn", (byte *)ED_CallSpawn},
{"SV_Physics_NewToss", (byte *)SV_Physics_NewToss},
{"G_RunEntity", (byte *)G_RunEntity},
{"SV_Physics_Step", (byte *)SV_Physics_Step},
{"SV_AddRotationalFriction", (byte *)SV_AddRotationalFriction},
{"SV_Physics_Toss", (byte *)SV_Physics_Toss},
{"SV_Physics_Noclip", (byte *)SV_Physics_Noclip},
{"SV_Physics_None", (byte *)SV_Physics_None},
{"SV_Physics_Pusher", (byte *)SV_Physics_Pusher},
{"SV_Push", (byte *)SV_Push},
{"SV_PushEntity", (byte *)SV_PushEntity},
{"RealBoundingBox", (byte *)RealBoundingBox},
{"SV_AddGravity", (byte *)SV_AddGravity},
{"SV_FlyMove", (byte *)SV_FlyMove},
{"ClipVelocity", (byte *)ClipVelocity},
{"SV_Impact", (byte *)SV_Impact},
{"SV_RunThink", (byte *)SV_RunThink},
{"SV_CheckVelocity", (byte *)SV_CheckVelocity},
{"SV_TestEntityPosition", (byte *)SV_TestEntityPosition},
{"fire_tracker", (byte *)fire_tracker},
{"tracker_fly", (byte *)tracker_fly},
{"tracker_touch", (byte *)tracker_touch},
{"tracker_explode", (byte *)tracker_explode},
{"tracker_pain_daemon_spawn", (byte *)tracker_pain_daemon_spawn},
{"tracker_pain_daemon_think", (byte *)tracker_pain_daemon_think},
{"fire_blaster2", (byte *)fire_blaster2},
{"blaster2_touch", (byte *)blaster2_touch},
{"fire_heat", (byte *)fire_heat},
{"fire_beams", (byte *)fire_beams},
{"fire_tesla", (byte *)fire_tesla},
{"tesla_lava", (byte *)tesla_lava},
{"tesla_think", (byte *)tesla_think},
{"tesla_activate", (byte *)tesla_activate},
{"tesla_think_active", (byte *)tesla_think_active},
{"tesla_zap", (byte *)tesla_zap},
{"tesla_blow", (byte *)tesla_blow},
{"tesla_die", (byte *)tesla_die},
{"tesla_remove", (byte *)tesla_remove},
{"fire_nuke", (byte *)fire_nuke},
{"nuke_bounce", (byte *)nuke_bounce},
{"Nuke_Think", (byte *)Nuke_Think},
{"nuke_die", (byte *)nuke_die},
{"Nuke_Explode", (byte *)Nuke_Explode},
{"Nuke_Quake", (byte *)Nuke_Quake},
{"fire_player_melee", (byte *)fire_player_melee},
{"fire_prox", (byte *)fire_prox},
{"prox_land", (byte *)prox_land},
{"prox_open", (byte *)prox_open},
{"prox_seek", (byte *)prox_seek},
{"Prox_Field_Touch", (byte *)Prox_Field_Touch},
{"prox_die", (byte *)prox_die},
{"Prox_Explode", (byte *)Prox_Explode},
{"fire_flechette", (byte *)fire_flechette},
{"flechette_touch", (byte *)flechette_touch},
{"SP_trigger_disguise", (byte *)SP_trigger_disguise},
{"trigger_disguise_use", (byte *)trigger_disguise_use},
{"trigger_disguise_touch", (byte *)trigger_disguise_touch},
{"SP_trigger_teleport", (byte *)SP_trigger_teleport},
{"trigger_teleport_use", (byte *)trigger_teleport_use},
{"trigger_teleport_touch", (byte *)trigger_teleport_touch},
{"SP_info_teleport_destination", (byte *)SP_info_teleport_destination},
{"SP_target_orb", (byte *)SP_target_orb},
{"orb_think", (byte *)orb_think},
{"SP_target_blacklight", (byte *)SP_target_blacklight},
{"blacklight_think", (byte *)blacklight_think},
{"SP_target_killplayers", (byte *)SP_target_killplayers},
{"target_killplayers_use", (byte *)target_killplayers_use},
{"SP_target_anger", (byte *)SP_target_anger},
{"target_anger_use", (byte *)target_anger_use},
{"SP_target_steam", (byte *)SP_target_steam},
{"target_steam_start", (byte *)target_steam_start},
{"use_target_steam", (byte *)use_target_steam},
{"SP_func_force_wall", (byte *)SP_func_force_wall},
{"force_wall_use", (byte *)force_wall_use},
{"force_wall_think", (byte *)force_wall_think},
{"SP_func_door_secret2", (byte *)SP_func_door_secret2},
{"secret_touch", (byte *)secret_touch},
{"secret_blocked", (byte *)secret_blocked},
{"fd_secret_done", (byte *)fd_secret_done},
{"fd_secret_move6", (byte *)fd_secret_move6},
{"fd_secret_move5", (byte *)fd_secret_move5},
{"fd_secret_move4", (byte *)fd_secret_move4},
{"fd_secret_move3", (byte *)fd_secret_move3},
{"fd_secret_move2", (byte *)fd_secret_move2},
{"fd_secret_move1", (byte *)fd_secret_move1},
{"fd_secret_killed", (byte *)fd_secret_killed},
{"fd_secret_use", (byte *)fd_secret_use},
{"fire_doppleganger", (byte *)fire_doppleganger},
{"body_think", (byte *)body_think},
{"doppleganger_timeout", (byte *)doppleganger_timeout},
{"doppleganger_pain", (byte *)doppleganger_pain},
{"doppleganger_die", (byte *)doppleganger_die},
{"PrecacheForRandomRespawn", (byte *)PrecacheForRandomRespawn},
{"DoRandomRespawn", (byte *)DoRandomRespawn},
{"FindSubstituteItem", (byte *)FindSubstituteItem},
{"InitGameRules", (byte *)InitGameRules},
{"monster_jump_finished", (byte *)monster_jump_finished},
{"monster_jump_start", (byte *)monster_jump_start},
{"CountPlayers", (byte *)CountPlayers},
{"PickCoopTarget", (byte *)PickCoopTarget},
{"TargetTesla", (byte *)TargetTesla},
{"has_valid_enemy", (byte *)has_valid_enemy},
{"monster_duck_up", (byte *)monster_duck_up},
{"monster_duck_hold", (byte *)monster_duck_hold},
{"monster_duck_down", (byte *)monster_duck_down},
{"M_MonsterDodge", (byte *)M_MonsterDodge},
{"drawbbox", (byte *)drawbbox},
{"below", (byte *)below},
{"PredictAim", (byte *)PredictAim},
{"MarkTeslaArea", (byte *)MarkTeslaArea},
{"CheckForBadArea", (byte *)CheckForBadArea},
{"SpawnBadArea", (byte *)SpawnBadArea},
{"badarea_touch", (byte *)badarea_touch},
{"face_wall", (byte *)face_wall},
{"realrange", (byte *)realrange},
{"inback", (byte *)inback},
{"InitHintPaths", (byte *)InitHintPaths},
{"SP_hint_path", (byte *)SP_hint_path},
{"hint_path_touch", (byte *)hint_path_touch},
{"monsterlost_checkhint", (byte *)monsterlost_checkhint},
{"hintpath_stop", (byte *)hintpath_stop},
{"hintpath_go", (byte *)hintpath_go},
{"hintpath_other_end", (byte *)hintpath_other_end},
{"hintpath_findstart", (byte *)hintpath_findstart},
{"blocked_checknewenemy", (byte *)blocked_checknewenemy},
{"blocked_checkjump", (byte *)blocked_checkjump},
{"blocked_checkplat", (byte *)blocked_checkplat},
{"monster_done_dodge", (byte *)monster_done_dodge},
{"stationarymonster_start", (byte *)stationarymonster_start},
{"stationarymonster_start_go", (byte *)stationarymonster_start_go},
{"stationarymonster_triggered_start", (byte *)stationarymonster_triggered_start},
{"stationarymonster_triggered_spawn_use", (byte *)stationarymonster_triggered_spawn_use},
{"stationarymonster_triggered_spawn", (byte *)stationarymonster_triggered_spawn},
{"swimmonster_start", (byte *)swimmonster_start},
{"swimmonster_start_go", (byte *)swimmonster_start_go},
{"flymonster_start", (byte *)flymonster_start},
{"flymonster_start_go", (byte *)flymonster_start_go},
{"walkmonster_start", (byte *)walkmonster_start},
{"walkmonster_start_go", (byte *)walkmonster_start_go},
{"monster_start_go", (byte *)monster_start_go},
{"monster_start", (byte *)monster_start},
{"monster_death_use", (byte *)monster_death_use},
{"monster_triggered_start", (byte *)monster_triggered_start},
{"monster_triggered_spawn_use", (byte *)monster_triggered_spawn_use},
{"monster_triggered_spawn", (byte *)monster_triggered_spawn},
{"monster_use", (byte *)monster_use},
{"monster_think", (byte *)monster_think},
{"M_MoveFrame", (byte *)M_MoveFrame},
{"M_SetEffects", (byte *)M_SetEffects},
{"M_droptofloor", (byte *)M_droptofloor},
{"M_WorldEffects", (byte *)M_WorldEffects},
{"M_CatagorizePosition", (byte *)M_CatagorizePosition},
{"M_CheckGround", (byte *)M_CheckGround},
{"AttackFinished", (byte *)AttackFinished},
{"M_FlyCheck", (byte *)M_FlyCheck},
{"M_FliesOn", (byte *)M_FliesOn},
{"M_FliesOff", (byte *)M_FliesOff},
{"monster_fire_bfg", (byte *)monster_fire_bfg},
{"monster_fire_railgun", (byte *)monster_fire_railgun},
{"monster_fire_rocket", (byte *)monster_fire_rocket},
{"monster_fire_grenade", (byte *)monster_fire_grenade},
{"monster_fire_heat", (byte *)monster_fire_heat},
{"monster_fire_tracker", (byte *)monster_fire_tracker},
{"monster_fire_blaster2", (byte *)monster_fire_blaster2},
{"monster_fire_blaster", (byte *)monster_fire_blaster},
{"monster_fire_shotgun", (byte *)monster_fire_shotgun},
{"monster_fire_bullet", (byte *)monster_fire_bullet},
{"SP_misc_nuke_core", (byte *)SP_misc_nuke_core},
{"misc_nuke_core_use", (byte *)misc_nuke_core_use},
{"SP_misc_teleporter_dest", (byte *)SP_misc_teleporter_dest},
{"SP_misc_teleporter", (byte *)SP_misc_teleporter},
{"teleporter_touch", (byte *)teleporter_touch},
{"SP_func_clock", (byte *)SP_func_clock},
{"func_clock_use", (byte *)func_clock_use},
{"func_clock_think", (byte *)func_clock_think},
{"SP_target_string", (byte *)SP_target_string},
{"target_string_use", (byte *)target_string_use},
{"SP_target_character", (byte *)SP_target_character},
{"SP_misc_gib_head", (byte *)SP_misc_gib_head},
{"SP_misc_gib_leg", (byte *)SP_misc_gib_leg},
{"SP_misc_gib_arm", (byte *)SP_misc_gib_arm},
{"SP_light_mine2", (byte *)SP_light_mine2},
{"SP_light_mine1", (byte *)SP_light_mine1},
{"SP_misc_satellite_dish", (byte *)SP_misc_satellite_dish},
{"misc_satellite_dish_use", (byte *)misc_satellite_dish_use},
{"misc_satellite_dish_think", (byte *)misc_satellite_dish_think},
{"SP_misc_strogg_ship", (byte *)SP_misc_strogg_ship},
{"misc_strogg_ship_use", (byte *)misc_strogg_ship_use},
{"SP_misc_viper_bomb", (byte *)SP_misc_viper_bomb},
{"misc_viper_bomb_use", (byte *)misc_viper_bomb_use},
{"misc_viper_bomb_prethink", (byte *)misc_viper_bomb_prethink},
{"misc_viper_bomb_touch", (byte *)misc_viper_bomb_touch},
{"SP_misc_bigviper", (byte *)SP_misc_bigviper},
{"SP_misc_viper", (byte *)SP_misc_viper},
{"misc_viper_use", (byte *)misc_viper_use},
{"SP_misc_deadsoldier", (byte *)SP_misc_deadsoldier},
{"misc_deadsoldier_die", (byte *)misc_deadsoldier_die},
{"SP_misc_banner", (byte *)SP_misc_banner},
{"misc_banner_think", (byte *)misc_banner_think},
{"SP_monster_commander_body", (byte *)SP_monster_commander_body},
{"commander_body_drop", (byte *)commander_body_drop},
{"commander_body_use", (byte *)commander_body_use},
{"commander_body_think", (byte *)commander_body_think},
{"SP_misc_easterchick2", (byte *)SP_misc_easterchick2},
{"misc_easterchick2_think", (byte *)misc_easterchick2_think},
{"SP_misc_easterchick", (byte *)SP_misc_easterchick},
{"misc_easterchick_think", (byte *)misc_easterchick_think},
{"SP_misc_eastertank", (byte *)SP_misc_eastertank},
{"misc_eastertank_think", (byte *)misc_eastertank_think},
{"SP_misc_blackhole", (byte *)SP_misc_blackhole},
{"misc_blackhole_think", (byte *)misc_blackhole_think},
{"misc_blackhole_use", (byte *)misc_blackhole_use},
{"SP_misc_explobox", (byte *)SP_misc_explobox},
{"barrel_start", (byte *)barrel_start},
{"barrel_think", (byte *)barrel_think},
{"barrel_delay", (byte *)barrel_delay},
{"barrel_explode", (byte *)barrel_explode},
{"barrel_touch", (byte *)barrel_touch},
{"SP_func_explosive", (byte *)SP_func_explosive},
{"func_explosive_spawn", (byte *)func_explosive_spawn},
{"func_explosive_activate", (byte *)func_explosive_activate},
{"func_explosive_use", (byte *)func_explosive_use},
{"func_explosive_explode", (byte *)func_explosive_explode},
{"SP_func_object", (byte *)SP_func_object},
{"func_object_use", (byte *)func_object_use},
{"func_object_release", (byte *)func_object_release},
{"func_object_touch", (byte *)func_object_touch},
{"SP_func_wall", (byte *)SP_func_wall},
{"func_wall_use", (byte *)func_wall_use},
{"SP_light", (byte *)SP_light},
{"light_use", (byte *)light_use},
{"SP_info_notnull", (byte *)SP_info_notnull},
{"SP_info_null", (byte *)SP_info_null},
{"SP_viewthing", (byte *)SP_viewthing},
{"TH_viewthing", (byte *)TH_viewthing},
{"SP_point_combat", (byte *)SP_point_combat},
{"point_combat_touch", (byte *)point_combat_touch},
{"SP_path_corner", (byte *)SP_path_corner},
{"path_corner_touch", (byte *)path_corner_touch},
{"BecomeExplosion2", (byte *)BecomeExplosion2},
{"BecomeExplosion1", (byte *)BecomeExplosion1},
{"ThrowDebris", (byte *)ThrowDebris},
{"debris_die", (byte *)debris_die},
{"ThrowClientHead", (byte *)ThrowClientHead},
{"ThrowHead", (byte *)ThrowHead},
{"ThrowGib", (byte *)ThrowGib},
{"gib_die", (byte *)gib_die},
{"gib_touch", (byte *)gib_touch},
{"gib_think", (byte *)gib_think},
{"ClipGibVelocity", (byte *)ClipGibVelocity},
{"VelocityForDamage", (byte *)VelocityForDamage},
{"SP_func_areaportal", (byte *)SP_func_areaportal},
{"Use_Areaportal", (byte *)Use_Areaportal},
{"G_RunFrame", (byte *)G_RunFrame},
{"ExitLevel", (byte *)ExitLevel},
{"CheckDMRules", (byte *)CheckDMRules},
{"CheckNeedPass", (byte *)CheckNeedPass},
{"EndDMLevel", (byte *)EndDMLevel},
{"CreateTargetChangeLevel", (byte *)CreateTargetChangeLevel},
{"ClientEndServerFrames", (byte *)ClientEndServerFrames},
{"Com_Printf", (byte *)Com_Printf},
{"Sys_Error", (byte *)Sys_Error},
{"GetGameAPI", (byte *)GetGameAPI},
{"ShutdownGame", (byte *)ShutdownGame},
{"SP_xatrix_item", (byte *)SP_xatrix_item},
{"SetItemNames", (byte *)SetItemNames},
{"InitItems", (byte *)InitItems},
{"SP_item_health_mega", (byte *)SP_item_health_mega},
{"SP_item_health_large", (byte *)SP_item_health_large},
{"SP_item_health_small", (byte *)SP_item_health_small},
{"SP_item_health", (byte *)SP_item_health},
{"SpawnItem", (byte *)SpawnItem},
{"SetTriggeredSpawn", (byte *)SetTriggeredSpawn},
{"Item_TriggeredSpawn", (byte *)Item_TriggeredSpawn},
{"PrecacheItem", (byte *)PrecacheItem},
{"droptofloor", (byte *)droptofloor},
{"Use_Item", (byte *)Use_Item},
{"Drop_Item", (byte *)Drop_Item},
{"drop_make_touchable", (byte *)drop_make_touchable},
{"drop_temp_touch", (byte *)drop_temp_touch},
{"Touch_Item", (byte *)Touch_Item},
{"Drop_PowerArmor", (byte *)Drop_PowerArmor},
{"Pickup_PowerArmor", (byte *)Pickup_PowerArmor},
{"Use_PowerArmor", (byte *)Use_PowerArmor},
{"PowerArmorType", (byte *)PowerArmorType},
{"Pickup_Armor", (byte *)Pickup_Armor},
{"ArmorIndex", (byte *)ArmorIndex},
{"Pickup_Health", (byte *)Pickup_Health},
{"MegaHealth_think", (byte *)MegaHealth_think},
{"Drop_Ammo", (byte *)Drop_Ammo},
{"Pickup_Ammo", (byte *)Pickup_Ammo},
{"Add_Ammo", (byte *)Add_Ammo},
{"Pickup_Key", (byte *)Pickup_Key},
{"Use_Silencer", (byte *)Use_Silencer},
{"Use_Invulnerability", (byte *)Use_Invulnerability},
{"Use_Envirosuit", (byte *)Use_Envirosuit},
{"Use_Breather", (byte *)Use_Breather},
{"Use_Quad", (byte *)Use_Quad},
{"Use_Vengeance", (byte *)Use_Vengeance},
{"Use_Hunter", (byte *)Use_Hunter},
{"Use_Defender", (byte *)Use_Defender},
{"Pickup_Sphere", (byte *)Pickup_Sphere},
{"Pickup_Doppleganger", (byte *)Pickup_Doppleganger},
{"Use_Doppleganger", (byte *)Use_Doppleganger},
{"Use_Nuke", (byte *)Use_Nuke},
{"Use_Compass", (byte *)Use_Compass},
{"Use_Double", (byte *)Use_Double},
{"Use_IR", (byte *)Use_IR},
{"Pickup_Nuke", (byte *)Pickup_Nuke},
{"Pickup_Pack", (byte *)Pickup_Pack},
{"Pickup_Bandolier", (byte *)Pickup_Bandolier},
{"Pickup_AncientHead", (byte *)Pickup_AncientHead},
{"Pickup_Adrenaline", (byte *)Pickup_Adrenaline},
{"Drop_General", (byte *)Drop_General},
{"Pickup_Powerup", (byte *)Pickup_Powerup},
{"SetRespawn", (byte *)SetRespawn},
{"DoRespawn", (byte *)DoRespawn},
{"FindItem", (byte *)FindItem},
{"FindItemByClassname", (byte *)FindItemByClassname},
{"GetItemByIndex", (byte *)GetItemByIndex},
{"SP_func_killbox", (byte *)SP_func_killbox},
{"use_killbox", (byte *)use_killbox},
{"SP_func_door_secret", (byte *)SP_func_door_secret},
{"door_secret_die", (byte *)door_secret_die},
{"door_secret_blocked", (byte *)door_secret_blocked},
{"door_secret_done", (byte *)door_secret_done},
{"door_secret_move6", (byte *)door_secret_move6},
{"door_secret_move5", (byte *)door_secret_move5},
{"door_secret_move4", (byte *)door_secret_move4},
{"door_secret_move3", (byte *)door_secret_move3},
{"door_secret_move2", (byte *)door_secret_move2},
{"door_secret_move1", (byte *)door_secret_move1},
{"door_secret_use", (byte *)door_secret_use},
{"SP_func_conveyor", (byte *)SP_func_conveyor},
{"func_conveyor_use", (byte *)func_conveyor_use},
{"SP_func_timer", (byte *)SP_func_timer},
{"func_timer_use", (byte *)func_timer_use},
{"func_timer_think", (byte *)func_timer_think},
{"SP_trigger_elevator", (byte *)SP_trigger_elevator},
{"trigger_elevator_init", (byte *)trigger_elevator_init},
{"trigger_elevator_use", (byte *)trigger_elevator_use},
{"SP_func_train", (byte *)SP_func_train},
{"train_use", (byte *)train_use},
{"func_train_find", (byte *)func_train_find},
{"train_resume", (byte *)train_resume},
{"train_next", (byte *)train_next},
{"train_piece_wait", (byte *)train_piece_wait},
{"train_wait", (byte *)train_wait},
{"train_blocked", (byte *)train_blocked},
{"SP_func_water", (byte *)SP_func_water},
{"smart_water_blocked", (byte *)smart_water_blocked},
{"SP_func_door_rotating", (byte *)SP_func_door_rotating},
{"Door_Activate", (byte *)Door_Activate},
{"SP_func_door", (byte *)SP_func_door},
{"door_touch", (byte *)door_touch},
{"door_killed", (byte *)door_killed},
{"door_blocked", (byte *)door_blocked},
{"Think_SpawnDoorTrigger", (byte *)Think_SpawnDoorTrigger},
{"Think_CalcMoveSpeed", (byte *)Think_CalcMoveSpeed},
{"Touch_DoorTrigger", (byte *)Touch_DoorTrigger},
{"door_use", (byte *)door_use},
{"smart_water_go_up", (byte *)smart_water_go_up},
{"door_go_up", (byte *)door_go_up},
{"door_go_down", (byte *)door_go_down},
{"door_hit_bottom", (byte *)door_hit_bottom},
{"door_hit_top", (byte *)door_hit_top},
{"door_use_areaportals", (byte *)door_use_areaportals},
{"SP_func_button", (byte *)SP_func_button},
{"button_killed", (byte *)button_killed},
{"button_touch", (byte *)button_touch},
{"button_use", (byte *)button_use},
{"button_fire", (byte *)button_fire},
{"button_wait", (byte *)button_wait},
{"button_return", (byte *)button_return},
{"button_done", (byte *)button_done},
{"SP_func_rotating", (byte *)SP_func_rotating},
{"rotating_use", (byte *)rotating_use},
{"rotating_touch", (byte *)rotating_touch},
{"rotating_blocked", (byte *)rotating_blocked},
{"rotating_decel", (byte *)rotating_decel},
{"rotating_accel", (byte *)rotating_accel},
{"SP_func_plat2", (byte *)SP_func_plat2},
{"plat2_activate", (byte *)plat2_activate},
{"Use_Plat2", (byte *)Use_Plat2},
{"plat2_blocked", (byte *)plat2_blocked},
{"Touch_Plat_Center2", (byte *)Touch_Plat_Center2},
{"plat2_operate", (byte *)plat2_operate},
{"plat2_go_up", (byte *)plat2_go_up},
{"plat2_go_down", (byte *)plat2_go_down},
{"plat2_hit_bottom", (byte *)plat2_hit_bottom},
{"plat2_hit_top", (byte *)plat2_hit_top},
{"plat2_kill_danger_area", (byte *)plat2_kill_danger_area},
{"plat2_spawn_danger_area", (byte *)plat2_spawn_danger_area},
{"SP_func_plat", (byte *)SP_func_plat},
{"plat_spawn_inside_trigger", (byte *)plat_spawn_inside_trigger},
{"Touch_Plat_Center", (byte *)Touch_Plat_Center},
{"Use_Plat", (byte *)Use_Plat},
{"plat_blocked", (byte *)plat_blocked},
{"plat_go_up", (byte *)plat_go_up},
{"plat_go_down", (byte *)plat_go_down},
{"plat_hit_bottom", (byte *)plat_hit_bottom},
{"plat_hit_top", (byte *)plat_hit_top},
{"Think_AccelMove", (byte *)Think_AccelMove},
{"plat_Accelerate", (byte *)plat_Accelerate},
{"plat_CalcAcceleratedMove", (byte *)plat_CalcAcceleratedMove},
{"AngleMove_Calc", (byte *)AngleMove_Calc},
{"AngleMove_Begin", (byte *)AngleMove_Begin},
{"AngleMove_Final", (byte *)AngleMove_Final},
{"AngleMove_Done", (byte *)AngleMove_Done},
{"Move_Calc", (byte *)Move_Calc},
{"Move_Begin", (byte *)Move_Begin},
{"Move_Final", (byte *)Move_Final},
{"Move_Done", (byte *)Move_Done},
{"T_RadiusClassDamage", (byte *)T_RadiusClassDamage},
{"T_RadiusNukeDamage", (byte *)T_RadiusNukeDamage},
{"T_RadiusDamage", (byte *)T_RadiusDamage},
{"T_Damage", (byte *)T_Damage},
{"CheckTeamDamage", (byte *)CheckTeamDamage},
{"M_ReactToDamage", (byte *)M_ReactToDamage},
{"CheckArmor", (byte *)CheckArmor},
{"CheckPowerArmor", (byte *)CheckPowerArmor},
{"SpawnDamage", (byte *)SpawnDamage},
{"Killed", (byte *)Killed},
{"CanDamage", (byte *)CanDamage},
{"cleanupHealTarget", (byte *)cleanupHealTarget},
{"ClientCommand", (byte *)ClientCommand},
{"Cmd_PlayerList_f", (byte *)Cmd_PlayerList_f},
{"Cmd_Ent_Count_f", (byte *)Cmd_Ent_Count_f},
{"Cmd_Say_f", (byte *)Cmd_Say_f},
{"Cmd_Wave_f", (byte *)Cmd_Wave_f},
{"Cmd_Players_f", (byte *)Cmd_Players_f},
{"PlayerSort", (byte *)PlayerSort},
{"Cmd_PutAway_f", (byte *)Cmd_PutAway_f},
{"Cmd_Kill_f", (byte *)Cmd_Kill_f},
{"Cmd_InvDrop_f", (byte *)Cmd_InvDrop_f},
{"Cmd_WeapLast_f", (byte *)Cmd_WeapLast_f},
{"Cmd_WeapNext_f", (byte *)Cmd_WeapNext_f},
{"Cmd_WeapPrev_f", (byte *)Cmd_WeapPrev_f},
{"Cmd_InvUse_f", (byte *)Cmd_InvUse_f},
{"Cmd_Inven_f", (byte *)Cmd_Inven_f},
{"Cmd_Help_f", (byte *)Cmd_Help_f},
{"Cmd_Drop_f", (byte *)Cmd_Drop_f},
{"Cmd_Use_f", (byte *)Cmd_Use_f},
{"Cmd_Noclip_f", (byte *)Cmd_Noclip_f},
{"Cmd_Notarget_f", (byte *)Cmd_Notarget_f},
{"Cmd_God_f", (byte *)Cmd_God_f},
{"Cmd_Give_f", (byte *)Cmd_Give_f},
{"ValidateSelectedItem", (byte *)ValidateSelectedItem},
{"OnSameTeam", (byte *)OnSameTeam},
{"GetChaseTarget", (byte *)GetChaseTarget},
{"ChasePrev", (byte *)ChasePrev},
{"ChaseNext", (byte *)ChaseNext},
{"UpdateChaseCam", (byte *)UpdateChaseCam},
{"ai_run", (byte *)ai_run},
{"ai_checkattack", (byte *)ai_checkattack},
{"ai_run_slide", (byte *)ai_run_slide},
{"ai_run_missile", (byte *)ai_run_missile},
{"ai_run_melee", (byte *)ai_run_melee},
{"M_CheckAttack", (byte *)M_CheckAttack},
{"FacingIdeal", (byte *)FacingIdeal},
{"FindTarget", (byte *)FindTarget},
{"FoundTarget", (byte *)FoundTarget},
{"HuntTarget", (byte *)HuntTarget},
{"infront", (byte *)infront},
{"visible", (byte *)visible},
{"range", (byte *)range},
{"ai_turn", (byte *)ai_turn},
{"ai_charge", (byte *)ai_charge},
{"ai_walk", (byte *)ai_walk},
{"ai_stand", (byte *)ai_stand},
{"ai_move", (byte *)ai_move},
{"AI_SetSightClient", (byte *)AI_SetSightClient},
{"SP_dm_tag_token", (byte *)SP_dm_tag_token},
{"Tag_PostInitSetup", (byte *)Tag_PostInitSetup},
{"Tag_GameInit", (byte *)Tag_GameInit},
{"Tag_ChangeDamage", (byte *)Tag_ChangeDamage},
{"Tag_DogTag", (byte *)Tag_DogTag},
{"Tag_PlayerEffects", (byte *)Tag_PlayerEffects},
{"Tag_DropToken", (byte *)Tag_DropToken},
{"Tag_MakeTouchable", (byte *)Tag_MakeTouchable},
{"Tag_Respawn", (byte *)Tag_Respawn},
{"Tag_PickupToken", (byte *)Tag_PickupToken},
{"Tag_Score", (byte *)Tag_Score},
{"Tag_PlayerDisconnect", (byte *)Tag_PlayerDisconnect},
{"Tag_KillItBonus", (byte *)Tag_KillItBonus},
{"Tag_PlayerDeath", (byte *)Tag_PlayerDeath},
{"SP_dm_dball_goal", (byte *)SP_dm_dball_goal},
{"SP_dm_dball_speed_change", (byte *)SP_dm_dball_speed_change},
{"SP_dm_dball_ball_start", (byte *)SP_dm_dball_ball_start},
{"SP_dm_dball_team2_start", (byte *)SP_dm_dball_team2_start},
{"SP_dm_dball_team1_start", (byte *)SP_dm_dball_team1_start},
{"SP_dm_dball_ball", (byte *)SP_dm_dball_ball},
{"DBall_SpeedTouch", (byte *)DBall_SpeedTouch},
{"DBall_BallRespawn", (byte *)DBall_BallRespawn},
{"DBall_BallDie", (byte *)DBall_BallDie},
{"DBall_BallPain", (byte *)DBall_BallPain},
{"DBall_BallTouch", (byte *)DBall_BallTouch},
{"PickBallStart", (byte *)PickBallStart},
{"DBall_GoalTouch", (byte *)DBall_GoalTouch},
{"DBall_ChangeKnockback", (byte *)DBall_ChangeKnockback},
{"DBall_ChangeDamage", (byte *)DBall_ChangeDamage},
{"DBall_PostInitSetup", (byte *)DBall_PostInitSetup},
{"DBall_GameInit", (byte *)DBall_GameInit},
{"DBall_SelectSpawnPoint", (byte *)DBall_SelectSpawnPoint},
{"DBall_ClientBegin", (byte *)DBall_ClientBegin},
{"DBall_CheckDMRules", (byte *)DBall_CheckDMRules},
{"wait_and_change_think", (byte *)wait_and_change_think},
