/* =======================================================================
 *
 * Carrier animations.
 *
 * =======================================================================
 *
 */
 
#define FRAME_search01        	0
#define FRAME_search02        	1
#define FRAME_search03        	2
#define FRAME_search04        	3
#define FRAME_search05        	4
#define FRAME_search06        	5
#define FRAME_search07        	6
#define FRAME_search08        	7
#define FRAME_search09        	8
#define FRAME_search10        	9
#define FRAME_search11        	10
#define FRAME_search12        	11
#define FRAME_search13        	12
#define FRAME_firea01         	13
#define FRAME_firea02         	14
#define FRAME_firea03         	15
#define FRAME_firea04         	16
#define FRAME_firea05         	17
#define FRAME_firea06         	18
#define FRAME_firea07         	19
#define FRAME_firea08         	20
#define FRAME_firea09         	21
#define FRAME_firea10         	22
#define FRAME_firea11         	23
#define FRAME_firea12         	24
#define FRAME_firea13         	25
#define FRAME_firea14         	26
#define FRAME_firea15         	27
#define FRAME_fireb01         	28
#define FRAME_fireb02         	29
#define FRAME_fireb03         	30
#define FRAME_fireb04         	31
#define FRAME_fireb05         	32
#define FRAME_fireb06         	33
#define FRAME_fireb07         	34
#define FRAME_fireb08         	35
#define FRAME_fireb09         	36
#define FRAME_fireb10         	37
#define FRAME_fireb11         	38
#define FRAME_fireb12         	39
#define FRAME_fireb13         	40
#define FRAME_fireb14         	41
#define FRAME_fireb15         	42
#define FRAME_fireb16         	43
#define FRAME_spawn01         	44
#define FRAME_spawn02         	45
#define FRAME_spawn03         	46
#define FRAME_spawn04         	47
#define FRAME_spawn05         	48
#define FRAME_spawn06         	49
#define FRAME_spawn07         	50
#define FRAME_spawn08         	51
#define FRAME_spawn09         	52
#define FRAME_spawn10         	53
#define FRAME_spawn11         	54
#define FRAME_spawn12         	55
#define FRAME_spawn13         	56
#define FRAME_spawn14         	57
#define FRAME_spawn15         	58
#define FRAME_spawn16         	59
#define FRAME_spawn17         	60
#define FRAME_spawn18         	61
#define FRAME_death01         	62
#define FRAME_death02         	63
#define FRAME_death03         	64
#define FRAME_death04         	65
#define FRAME_death05         	66
#define FRAME_death06         	67
#define FRAME_death07         	68
#define FRAME_death08         	69
#define FRAME_death09         	70
#define FRAME_death10         	71
#define FRAME_death11         	72
#define FRAME_death12         	73
#define FRAME_death13         	74
#define FRAME_death14         	75
#define FRAME_death15         	76
#define FRAME_death16         	77
#define MODEL_SCALE				1.000000
