/* =======================================================================
 *
 * Stalker animations.
 *
 * =======================================================================
 */
 
#define FRAME_idle01          	0
#define FRAME_idle02          	1
#define FRAME_idle03          	2
#define FRAME_idle04          	3
#define FRAME_idle05          	4
#define FRAME_idle06          	5
#define FRAME_idle07          	6
#define FRAME_idle08          	7
#define FRAME_idle09          	8
#define FRAME_idle10          	9
#define FRAME_idle11          	10
#define FRAME_idle12          	11
#define FRAME_idle13          	12
#define FRAME_idle14          	13
#define FRAME_idle15          	14
#define FRAME_idle16          	15
#define FRAME_idle17          	16
#define FRAME_idle18          	17
#define FRAME_idle19          	18
#define FRAME_idle20          	19
#define FRAME_idle21          	20
#define FRAME_idle201         	21
#define FRAME_idle202         	22
#define FRAME_idle203         	23
#define FRAME_idle204         	24
#define FRAME_idle205         	25
#define FRAME_idle206         	26
#define FRAME_idle207         	27
#define FRAME_idle208         	28
#define FRAME_idle209         	29
#define FRAME_idle210         	30
#define FRAME_idle211         	31
#define FRAME_idle212         	32
#define FRAME_idle213         	33
#define FRAME_walk01          	34
#define FRAME_walk02          	35
#define FRAME_walk03          	36
#define FRAME_walk04          	37
#define FRAME_walk05          	38
#define FRAME_walk06          	39
#define FRAME_walk07          	40
#define FRAME_walk08          	41
#define FRAME_jump01          	42
#define FRAME_jump02          	43
#define FRAME_jump03          	44
#define FRAME_jump04          	45
#define FRAME_jump05          	46
#define FRAME_jump06          	47
#define FRAME_jump07          	48
#define FRAME_run01           	49
#define FRAME_run02           	50
#define FRAME_run03           	51
#define FRAME_run04           	52
#define FRAME_attack01        	53
#define FRAME_attack02        	54
#define FRAME_attack03        	55
#define FRAME_attack04        	56
#define FRAME_attack05        	57
#define FRAME_attack06        	58
#define FRAME_attack07        	59
#define FRAME_attack08        	60
#define FRAME_attack11        	61
#define FRAME_attack12        	62
#define FRAME_attack13        	63
#define FRAME_attack14        	64
#define FRAME_attack15        	65
#define FRAME_pain01          	66
#define FRAME_pain02          	67
#define FRAME_pain03          	68
#define FRAME_pain04          	69
#define FRAME_death01         	70
#define FRAME_death02         	71
#define FRAME_death03         	72
#define FRAME_death04         	73
#define FRAME_death05         	74
#define FRAME_death06         	75
#define FRAME_death07         	76
#define FRAME_death08         	77
#define FRAME_death09         	78
#define FRAME_twitch01        	79
#define FRAME_twitch02        	80
#define FRAME_twitch03        	81
#define FRAME_twitch04        	82
#define FRAME_twitch05        	83
#define FRAME_twitch06        	84
#define FRAME_twitch07        	85
#define FRAME_twitch08        	86
#define FRAME_twitch09        	87
#define FRAME_twitch10        	88
#define FRAME_reactive01      	89
#define FRAME_reactive02      	90
#define FRAME_reactive03      	91
#define FRAME_reactive04      	92
#define MODEL_SCALE				1.000000
