/* =======================================================================
 *
 * Turret animations.
 *
 * =======================================================================
 */
 
#define FRAME_stand01         	0
#define FRAME_stand02         	1
#define FRAME_active01        	2
#define FRAME_active02        	3
#define FRAME_active03        	4
#define FRAME_active04        	5
#define FRAME_active05        	6
#define FRAME_active06        	7
#define FRAME_run01           	8
#define FRAME_run02           	9
#define FRAME_pow01           	10
#define FRAME_pow02           	11
#define FRAME_pow03           	12
#define FRAME_pow04           	13
#define FRAME_death01         	14
#define FRAME_death02         	15
#define MODEL_SCALE				3.500000
