/* =======================================================================
 *
 * Animations for boss2.
 *
 * =======================================================================
 */
 
#define FRAME_stand30         	0
#define FRAME_stand31         	1
#define FRAME_stand32         	2
#define FRAME_stand33         	3
#define FRAME_stand34         	4
#define FRAME_stand35         	5
#define FRAME_stand36         	6
#define FRAME_stand37         	7
#define FRAME_stand38         	8
#define FRAME_stand39         	9
#define FRAME_stand40         	10
#define FRAME_stand41         	11
#define FRAME_stand42         	12
#define FRAME_stand43         	13
#define FRAME_stand44         	14
#define FRAME_stand45         	15
#define FRAME_stand46         	16
#define FRAME_stand47         	17
#define FRAME_stand48         	18
#define FRAME_stand49         	19
#define FRAME_stand50         	20
#define FRAME_stand1          	21
#define FRAME_stand2          	22
#define FRAME_stand3          	23
#define FRAME_stand4          	24
#define FRAME_stand5          	25
#define FRAME_stand6          	26
#define FRAME_stand7          	27
#define FRAME_stand8          	28
#define FRAME_stand9          	29
#define FRAME_stand10         	30
#define FRAME_stand11         	31
#define FRAME_stand12         	32
#define FRAME_stand13         	33
#define FRAME_stand14         	34
#define FRAME_stand15         	35
#define FRAME_stand16         	36
#define FRAME_stand17         	37
#define FRAME_stand18         	38
#define FRAME_stand19         	39
#define FRAME_stand20         	40
#define FRAME_stand21         	41
#define FRAME_stand22         	42
#define FRAME_stand23         	43
#define FRAME_stand24         	44
#define FRAME_stand25         	45
#define FRAME_stand26         	46
#define FRAME_stand27         	47
#define FRAME_stand28         	48
#define FRAME_stand29         	49
#define FRAME_walk1           	50
#define FRAME_walk2           	51
#define FRAME_walk3           	52
#define FRAME_walk4           	53
#define FRAME_walk5           	54
#define FRAME_walk6           	55
#define FRAME_walk7           	56
#define FRAME_walk8           	57
#define FRAME_walk9           	58
#define FRAME_walk10          	59
#define FRAME_walk11          	60
#define FRAME_walk12          	61
#define FRAME_walk13          	62
#define FRAME_walk14          	63
#define FRAME_walk15          	64
#define FRAME_walk16          	65
#define FRAME_walk17          	66
#define FRAME_walk18          	67
#define FRAME_walk19          	68
#define FRAME_walk20          	69
#define FRAME_attack1         	70
#define FRAME_attack2         	71
#define FRAME_attack3         	72
#define FRAME_attack4         	73
#define FRAME_attack5         	74
#define FRAME_attack6         	75
#define FRAME_attack7         	76
#define FRAME_attack8         	77
#define FRAME_attack9         	78
#define FRAME_attack10        	79
#define FRAME_attack11        	80
#define FRAME_attack12        	81
#define FRAME_attack13        	82
#define FRAME_attack14        	83
#define FRAME_attack15        	84
#define FRAME_attack16        	85
#define FRAME_attack17        	86
#define FRAME_attack18        	87
#define FRAME_attack19        	88
#define FRAME_attack20        	89
#define FRAME_attack21        	90
#define FRAME_attack22        	91
#define FRAME_attack23        	92
#define FRAME_attack24        	93
#define FRAME_attack25        	94
#define FRAME_attack26        	95
#define FRAME_attack27        	96
#define FRAME_attack28        	97
#define FRAME_attack29        	98
#define FRAME_attack30        	99
#define FRAME_attack31        	100
#define FRAME_attack32        	101
#define FRAME_attack33        	102
#define FRAME_attack34        	103
#define FRAME_attack35        	104
#define FRAME_attack36        	105
#define FRAME_attack37        	106
#define FRAME_attack38        	107
#define FRAME_attack39        	108
#define FRAME_attack40        	109
#define FRAME_pain2           	110
#define FRAME_pain3           	111
#define FRAME_pain4           	112
#define FRAME_pain5           	113
#define FRAME_pain6           	114
#define FRAME_pain7           	115
#define FRAME_pain8           	116
#define FRAME_pain9           	117
#define FRAME_pain10          	118
#define FRAME_pain11          	119
#define FRAME_pain12          	120
#define FRAME_pain13          	121
#define FRAME_pain14          	122
#define FRAME_pain15          	123
#define FRAME_pain16          	124
#define FRAME_pain17          	125
#define FRAME_pain18          	126
#define FRAME_pain19          	127
#define FRAME_pain20          	128
#define FRAME_pain21          	129
#define FRAME_pain22          	130
#define FRAME_pain23          	131
#define FRAME_death2          	132
#define FRAME_death3          	133
#define FRAME_death4          	134
#define FRAME_death5          	135
#define FRAME_death6          	136
#define FRAME_death7          	137
#define FRAME_death8          	138
#define FRAME_death9          	139
#define FRAME_death10         	140
#define FRAME_death11         	141
#define FRAME_death12         	142
#define FRAME_death13         	143
#define FRAME_death14         	144
#define FRAME_death15         	145
#define FRAME_death16         	146
#define FRAME_death17         	147
#define FRAME_death18         	148
#define FRAME_death19         	149
#define FRAME_death20         	150
#define FRAME_death21         	151
#define FRAME_death22         	152
#define FRAME_death23         	153
#define FRAME_death24         	154
#define FRAME_death25         	155
#define FRAME_death26         	156
#define FRAME_death27         	157
#define FRAME_death28         	158
#define FRAME_death29         	159
#define FRAME_death30         	160
#define FRAME_death31         	161
#define FRAME_death32         	162
#define FRAME_death33         	163
#define FRAME_death34         	164
#define FRAME_death35         	165
#define FRAME_death36         	166
#define FRAME_death37         	167
#define FRAME_death38         	168
#define FRAME_death39         	169
#define FRAME_death40         	170
#define FRAME_death41         	171
#define FRAME_death42         	172
#define FRAME_death43         	173
#define FRAME_death44         	174
#define FRAME_death45         	175
#define FRAME_death46         	176
#define FRAME_death47         	177
#define FRAME_death48         	178
#define FRAME_death49         	179
#define FRAME_death50         	180
#define MODEL_SCALE				1.000000
