/* =======================================================================
 *
 * Mechanic animations.
 *
 * =======================================================================
 */
 
#define FRAME_actvat01        	0
#define FRAME_actvat02        	1
#define FRAME_actvat03        	2
#define FRAME_actvat04        	3
#define FRAME_actvat05        	4
#define FRAME_actvat06        	5
#define FRAME_actvat07        	6
#define FRAME_actvat08        	7
#define FRAME_actvat09        	8
#define FRAME_actvat10        	9
#define FRAME_actvat11        	10
#define FRAME_actvat12        	11
#define FRAME_actvat13        	12
#define FRAME_actvat14        	13
#define FRAME_actvat15        	14
#define FRAME_actvat16        	15
#define FRAME_actvat17        	16
#define FRAME_actvat18        	17
#define FRAME_actvat19        	18
#define FRAME_actvat20        	19
#define FRAME_actvat21        	20
#define FRAME_actvat22        	21
#define FRAME_actvat23        	22
#define FRAME_actvat24        	23
#define FRAME_actvat25        	24
#define FRAME_actvat26        	25
#define FRAME_actvat27        	26
#define FRAME_actvat28        	27
#define FRAME_actvat29        	28
#define FRAME_actvat30        	29
#define FRAME_actvat31        	30
#define FRAME_attak101        	31
#define FRAME_attak102        	32
#define FRAME_attak103        	33
#define FRAME_attak104        	34
#define FRAME_attak105        	35
#define FRAME_attak106        	36
#define FRAME_attak107        	37
#define FRAME_attak108        	38
#define FRAME_attak109        	39
#define FRAME_attak110        	40
#define FRAME_attak111        	41
#define FRAME_attak112        	42
#define FRAME_attak113        	43
#define FRAME_attak114        	44
#define FRAME_attak201        	45
#define FRAME_attak202        	46
#define FRAME_attak203        	47
#define FRAME_attak204        	48
#define FRAME_attak205        	49
#define FRAME_attak206        	50
#define FRAME_attak207        	51
#define FRAME_attak208        	52
#define FRAME_attak209        	53
#define FRAME_attak210        	54
#define FRAME_attak211        	55
#define FRAME_attak212        	56
#define FRAME_attak213        	57
#define FRAME_attak214        	58
#define FRAME_attak215        	59
#define FRAME_attak216        	60
#define FRAME_attak217        	61
#define FRAME_attak218        	62
#define FRAME_attak219        	63
#define FRAME_attak220        	64
#define FRAME_attak221        	65
#define FRAME_attak222        	66
#define FRAME_attak223        	67
#define FRAME_attak224        	68
#define FRAME_attak225        	69
#define FRAME_attak301        	70
#define FRAME_attak302        	71
#define FRAME_attak303        	72
#define FRAME_attak304        	73
#define FRAME_attak305        	74
#define FRAME_attak306        	75
#define FRAME_attak307        	76
#define FRAME_attak308        	77
#define FRAME_attak309        	78
#define FRAME_attak310        	79
#define FRAME_attak311        	80
#define FRAME_attak312        	81
#define FRAME_attak313        	82
#define FRAME_attak314        	83
#define FRAME_attak315        	84
#define FRAME_attak316        	85
#define FRAME_attak317        	86
#define FRAME_attak318        	87
#define FRAME_attak319        	88
#define FRAME_attak320        	89
#define FRAME_attak321        	90
#define FRAME_attak322        	91
#define FRAME_attak323        	92
#define FRAME_attak324        	93
#define FRAME_attak325        	94
#define FRAME_attak326        	95
#define FRAME_attak327        	96
#define FRAME_attak328        	97
#define FRAME_attak329        	98
#define FRAME_attak330        	99
#define FRAME_attak331        	100
#define FRAME_attak332        	101
#define FRAME_attak333        	102
#define FRAME_attak334        	103
#define FRAME_death01         	104
#define FRAME_death02         	105
#define FRAME_death03         	106
#define FRAME_death04         	107
#define FRAME_death05         	108
#define FRAME_death06         	109
#define FRAME_death07         	110
#define FRAME_death08         	111
#define FRAME_death09         	112
#define FRAME_death10         	113
#define FRAME_death11         	114
#define FRAME_death12         	115
#define FRAME_death13         	116
#define FRAME_pain101         	117
#define FRAME_pain102         	118
#define FRAME_pain103         	119
#define FRAME_pain104         	120
#define FRAME_pain105         	121
#define FRAME_pain106         	122
#define FRAME_pain107         	123
#define FRAME_pain201         	124
#define FRAME_pain202         	125
#define FRAME_pain203         	126
#define FRAME_pain204         	127
#define FRAME_pain205         	128
#define FRAME_pain206         	129
#define FRAME_pain207         	130
#define FRAME_pain208         	131
#define FRAME_pain301         	132
#define FRAME_pain302         	133
#define FRAME_pain303         	134
#define FRAME_pain304         	135
#define FRAME_pain305         	136
#define FRAME_pain306         	137
#define FRAME_pain307         	138
#define FRAME_pain308         	139
#define FRAME_pain309         	140
#define FRAME_pain310         	141
#define FRAME_pain311         	142
#define FRAME_pain312         	143
#define FRAME_stand101        	144
#define FRAME_stand102        	145
#define FRAME_stand103        	146
#define FRAME_stand104        	147
#define FRAME_stand105        	148
#define FRAME_stand106        	149
#define FRAME_stand107        	150
#define FRAME_stand108        	151
#define FRAME_stand109        	152
#define FRAME_stand110        	153
#define FRAME_stand111        	154
#define FRAME_stand112        	155
#define FRAME_stand113        	156
#define FRAME_stand114        	157
#define FRAME_stand115        	158
#define FRAME_stand116        	159
#define FRAME_stand117        	160
#define FRAME_stand118        	161
#define FRAME_stand119        	162
#define FRAME_stand120        	163
#define FRAME_stand121        	164
#define FRAME_stand122        	165
#define FRAME_stand123        	166
#define FRAME_stand124        	167
#define FRAME_stand125        	168
#define FRAME_stand126        	169
#define FRAME_stand127        	170
#define FRAME_stand128        	171
#define FRAME_stand129        	172
#define FRAME_stand130        	173
#define FRAME_stand131        	174
#define FRAME_stand132        	175
#define FRAME_stand133        	176
#define FRAME_stand134        	177
#define FRAME_stand135        	178
#define FRAME_stand136        	179
#define FRAME_stand137        	180
#define FRAME_stand138        	181
#define FRAME_stand139        	182
#define FRAME_stand140        	183
#define FRAME_stand141        	184
#define FRAME_stand142        	185
#define FRAME_stand143        	186
#define FRAME_stand144        	187
#define FRAME_stand145        	188
#define FRAME_stand146        	189
#define FRAME_stand147        	190
#define FRAME_stand148        	191
#define FRAME_stand149        	192
#define FRAME_stand150        	193
#define FRAME_stand151        	194
#define FRAME_stand152        	195
#define FRAME_stand201        	196
#define FRAME_stand202        	197
#define FRAME_stand203        	198
#define FRAME_stand204        	199
#define FRAME_stand205        	200
#define FRAME_stand206        	201
#define FRAME_stand207        	202
#define FRAME_stand208        	203
#define FRAME_stand209        	204
#define FRAME_stand210        	205
#define FRAME_stand211        	206
#define FRAME_stand212        	207
#define FRAME_stand213        	208
#define FRAME_stand214        	209
#define FRAME_stand215        	210
#define FRAME_stand216        	211
#define FRAME_stand217        	212
#define FRAME_stand218        	213
#define FRAME_stand219        	214
#define FRAME_stand220        	215
#define FRAME_stand221        	216
#define FRAME_stand222        	217
#define FRAME_stand223        	218
#define FRAME_stand224        	219
#define FRAME_stand225        	220
#define FRAME_stand226        	221
#define FRAME_stand227        	222
#define FRAME_stand228        	223
#define FRAME_stand229        	224
#define FRAME_stand230        	225
#define FRAME_stand231        	226
#define FRAME_stand232        	227
#define FRAME_stand233        	228
#define FRAME_stand234        	229
#define FRAME_stand235        	230
#define FRAME_stand236        	231
#define FRAME_stand237        	232
#define FRAME_stand238        	233
#define FRAME_stand239        	234
#define FRAME_stand240        	235
#define FRAME_stand241        	236
#define FRAME_stand242        	237
#define FRAME_stand243        	238
#define FRAME_stand244        	239
#define FRAME_stand245        	240
#define FRAME_stand246        	241
#define FRAME_stand247        	242
#define FRAME_stand248        	243
#define FRAME_stand249        	244
#define FRAME_stand250        	245
#define FRAME_stand251        	246
#define FRAME_stand252        	247
#define MODEL_SCALE				1.000000
