/* =======================================================================
 *
 * Supertank aka "Boss1" animations.
 *
 * =======================================================================
 */ 
 
#define FRAME_attak1_1        	0
#define FRAME_attak1_2        	1
#define FRAME_attak1_3        	2
#define FRAME_attak1_4        	3
#define FRAME_attak1_5        	4
#define FRAME_attak1_6        	5
#define FRAME_attak1_7        	6
#define FRAME_attak1_8        	7
#define FRAME_attak1_9        	8
#define FRAME_attak1_10       	9
#define FRAME_attak1_11       	10
#define FRAME_attak1_12       	11
#define FRAME_attak1_13       	12
#define FRAME_attak1_14       	13
#define FRAME_attak1_15       	14
#define FRAME_attak1_16       	15
#define FRAME_attak1_17       	16
#define FRAME_attak1_18       	17
#define FRAME_attak1_19       	18
#define FRAME_attak1_20       	19
#define FRAME_attak2_1        	20
#define FRAME_attak2_2        	21
#define FRAME_attak2_3        	22
#define FRAME_attak2_4        	23
#define FRAME_attak2_5        	24
#define FRAME_attak2_6        	25
#define FRAME_attak2_7        	26
#define FRAME_attak2_8        	27
#define FRAME_attak2_9        	28
#define FRAME_attak2_10       	29
#define FRAME_attak2_11       	30
#define FRAME_attak2_12       	31
#define FRAME_attak2_13       	32
#define FRAME_attak2_14       	33
#define FRAME_attak2_15       	34
#define FRAME_attak2_16       	35
#define FRAME_attak2_17       	36
#define FRAME_attak2_18       	37
#define FRAME_attak2_19       	38
#define FRAME_attak2_20       	39
#define FRAME_attak2_21       	40
#define FRAME_attak2_22       	41
#define FRAME_attak2_23       	42
#define FRAME_attak2_24       	43
#define FRAME_attak2_25       	44
#define FRAME_attak2_26       	45
#define FRAME_attak2_27       	46
#define FRAME_attak3_1        	47
#define FRAME_attak3_2        	48
#define FRAME_attak3_3        	49
#define FRAME_attak3_4        	50
#define FRAME_attak3_5        	51
#define FRAME_attak3_6        	52
#define FRAME_attak3_7        	53
#define FRAME_attak3_8        	54
#define FRAME_attak3_9        	55
#define FRAME_attak3_10       	56
#define FRAME_attak3_11       	57
#define FRAME_attak3_12       	58
#define FRAME_attak3_13       	59
#define FRAME_attak3_14       	60
#define FRAME_attak3_15       	61
#define FRAME_attak3_16       	62
#define FRAME_attak3_17       	63
#define FRAME_attak3_18       	64
#define FRAME_attak3_19       	65
#define FRAME_attak3_20       	66
#define FRAME_attak3_21       	67
#define FRAME_attak3_22       	68
#define FRAME_attak3_23       	69
#define FRAME_attak3_24       	70
#define FRAME_attak3_25       	71
#define FRAME_attak3_26       	72
#define FRAME_attak3_27       	73
#define FRAME_attak4_1        	74
#define FRAME_attak4_2        	75
#define FRAME_attak4_3        	76
#define FRAME_attak4_4        	77
#define FRAME_attak4_5        	78
#define FRAME_attak4_6        	79
#define FRAME_backwd_1        	80
#define FRAME_backwd_2        	81
#define FRAME_backwd_3        	82
#define FRAME_backwd_4        	83
#define FRAME_backwd_5        	84
#define FRAME_backwd_6        	85
#define FRAME_backwd_7        	86
#define FRAME_backwd_8        	87
#define FRAME_backwd_9        	88
#define FRAME_backwd_10       	89
#define FRAME_backwd_11       	90
#define FRAME_backwd_12       	91
#define FRAME_backwd_13       	92
#define FRAME_backwd_14       	93
#define FRAME_backwd_15       	94
#define FRAME_backwd_16       	95
#define FRAME_backwd_17       	96
#define FRAME_backwd_18       	97
#define FRAME_death_1         	98
#define FRAME_death_2         	99
#define FRAME_death_3         	100
#define FRAME_death_4         	101
#define FRAME_death_5         	102
#define FRAME_death_6         	103
#define FRAME_death_7         	104
#define FRAME_death_8         	105
#define FRAME_death_9         	106
#define FRAME_death_10        	107
#define FRAME_death_11        	108
#define FRAME_death_12        	109
#define FRAME_death_13        	110
#define FRAME_death_14        	111
#define FRAME_death_15        	112
#define FRAME_death_16        	113
#define FRAME_death_17        	114
#define FRAME_death_18        	115
#define FRAME_death_19        	116
#define FRAME_death_20        	117
#define FRAME_death_21        	118
#define FRAME_death_22        	119
#define FRAME_death_23        	120
#define FRAME_death_24        	121
#define FRAME_death_31        	122
#define FRAME_death_32        	123
#define FRAME_death_33        	124
#define FRAME_death_45        	125
#define FRAME_death_46        	126
#define FRAME_death_47        	127
#define FRAME_forwrd_1        	128
#define FRAME_forwrd_2        	129
#define FRAME_forwrd_3        	130
#define FRAME_forwrd_4        	131
#define FRAME_forwrd_5        	132
#define FRAME_forwrd_6        	133
#define FRAME_forwrd_7        	134
#define FRAME_forwrd_8        	135
#define FRAME_forwrd_9        	136
#define FRAME_forwrd_10       	137
#define FRAME_forwrd_11       	138
#define FRAME_forwrd_12       	139
#define FRAME_forwrd_13       	140
#define FRAME_forwrd_14       	141
#define FRAME_forwrd_15       	142
#define FRAME_forwrd_16       	143
#define FRAME_forwrd_17       	144
#define FRAME_forwrd_18       	145
#define FRAME_left_1          	146
#define FRAME_left_2          	147
#define FRAME_left_3          	148
#define FRAME_left_4          	149
#define FRAME_left_5          	150
#define FRAME_left_6          	151
#define FRAME_left_7          	152
#define FRAME_left_8          	153
#define FRAME_left_9          	154
#define FRAME_left_10         	155
#define FRAME_left_11         	156
#define FRAME_left_12         	157
#define FRAME_left_13         	158
#define FRAME_left_14         	159
#define FRAME_left_15         	160
#define FRAME_left_16         	161
#define FRAME_left_17         	162
#define FRAME_left_18         	163
#define FRAME_pain1_1         	164
#define FRAME_pain1_2         	165
#define FRAME_pain1_3         	166
#define FRAME_pain1_4         	167
#define FRAME_pain2_5         	168
#define FRAME_pain2_6         	169
#define FRAME_pain2_7         	170
#define FRAME_pain2_8         	171
#define FRAME_pain3_9         	172
#define FRAME_pain3_10        	173
#define FRAME_pain3_11        	174
#define FRAME_pain3_12        	175
#define FRAME_right_1         	176
#define FRAME_right_2         	177
#define FRAME_right_3         	178
#define FRAME_right_4         	179
#define FRAME_right_5         	180
#define FRAME_right_6         	181
#define FRAME_right_7         	182
#define FRAME_right_8         	183
#define FRAME_right_9         	184
#define FRAME_right_10        	185
#define FRAME_right_11        	186
#define FRAME_right_12        	187
#define FRAME_right_13        	188
#define FRAME_right_14        	189
#define FRAME_right_15        	190
#define FRAME_right_16        	191
#define FRAME_right_17        	192
#define FRAME_right_18        	193
#define FRAME_stand_1         	194
#define FRAME_stand_2         	195
#define FRAME_stand_3         	196
#define FRAME_stand_4         	197
#define FRAME_stand_5         	198
#define FRAME_stand_6         	199
#define FRAME_stand_7         	200
#define FRAME_stand_8         	201
#define FRAME_stand_9         	202
#define FRAME_stand_10        	203
#define FRAME_stand_11        	204
#define FRAME_stand_12        	205
#define FRAME_stand_13        	206
#define FRAME_stand_14        	207
#define FRAME_stand_15        	208
#define FRAME_stand_16        	209
#define FRAME_stand_17        	210
#define FRAME_stand_18        	211
#define FRAME_stand_19        	212
#define FRAME_stand_20        	213
#define FRAME_stand_21        	214
#define FRAME_stand_22        	215
#define FRAME_stand_23        	216
#define FRAME_stand_24        	217
#define FRAME_stand_25        	218
#define FRAME_stand_26        	219
#define FRAME_stand_27        	220
#define FRAME_stand_28        	221
#define FRAME_stand_29        	222
#define FRAME_stand_30        	223
#define FRAME_stand_31        	224
#define FRAME_stand_32        	225
#define FRAME_stand_33        	226
#define FRAME_stand_34        	227
#define FRAME_stand_35        	228
#define FRAME_stand_36        	229
#define FRAME_stand_37        	230
#define FRAME_stand_38        	231
#define FRAME_stand_39        	232
#define FRAME_stand_40        	233
#define FRAME_stand_41        	234
#define FRAME_stand_42        	235
#define FRAME_stand_43        	236
#define FRAME_stand_44        	237
#define FRAME_stand_45        	238
#define FRAME_stand_46        	239
#define FRAME_stand_47        	240
#define FRAME_stand_48        	241
#define FRAME_stand_49        	242
#define FRAME_stand_50        	243
#define FRAME_stand_51        	244
#define FRAME_stand_52        	245
#define FRAME_stand_53        	246
#define FRAME_stand_54        	247
#define FRAME_stand_55        	248
#define FRAME_stand_56        	249
#define FRAME_stand_57        	250
#define FRAME_stand_58        	251
#define FRAME_stand_59        	252
#define FRAME_stand_60        	253
#define MODEL_SCALE				1.000000
